@@ -16,12 +16,15 @@
 
 // ----------------------------------------------------------------------------
 // QT Includes
+#include <qapplication.h>
 #include <qlayout.h>
 #include <qvbox.h>
 #include <qlabel.h>
 #include <qgroupbox.h>
 #include <qtabwidget.h>
 #include <qvalidator.h>
+#include <qheader.h>
+#include <qlistview.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -41,10 +44,16 @@
 #endif
 
 #include <kmessagebox.h>
+#include <klistview.h>
+#include <kpushbutton.h>
+#include <kguiitem.h>
+#include <kglobalsettings.h>
+#include <knuminput.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 #include "ksettingsdlg.h"
+#include "../kmymoneyutils.h"
 
 /** Standard constructor for the class.
   * The constructor passes some additional parameters to the base class KDialogBase
@@ -58,8 +67,24 @@ KSettingsDlg::KSettingsDlg(QWidget *parent, const char *name, bool modal)
   setPageGeneral();
   setPageAccountsView();
   setPageList();
+  setHomePage();
+  setPageSchedule();
+  setPageColour();
+  setPageFont();
+  
   configRead();
+  
   m_bDoneApply=false;
+  
+  KPushButton *reset = static_cast<KPushButton *>(actionButton(User1));
+  
+  KIconLoader* il = KGlobal::iconLoader();
+  KGuiItem resetButtonItem( i18n( "&Reset" ),
+                    QIconSet(il->loadIcon("undo", KIcon::Small, KIcon::SizeSmall)),
+                    i18n("Reset all settings"),
+                    i18n("Use this to reset all settings to the state they were when the dialog was opened."));
+  reset->setGuiItem(resetButtonItem);
+
 }
 
 /** Standard destructor for the class.
@@ -76,28 +101,58 @@ void KSettingsDlg::setPageGeneral()
   QVBox *qvboxMainFrame = addVBoxPage( i18n("General"), i18n("General settings"),
     DesktopIcon("misc"));
 
+  // Startup page options
+  // --------------------
+
+/* These settings are deprecated  
   // Create a group box to hold the available options
-  QButtonGroup *qbuttongroup = new QButtonGroup(qvboxMainFrame, "GroupBox1");
-  qbuttongroup->setTitle( i18n( "Startup options" ) );
-  qbuttongroup->setColumnLayout(0, Qt::Vertical );
-  qbuttongroup->layout()->setSpacing( 0 );
-  qbuttongroup->layout()->setMargin( 0 );
+  QButtonGroup *qfilebuttongroup = new QButtonGroup(qvboxMainFrame, "GroupBox1");
+  qfilebuttongroup->setTitle( i18n( "Startup file options" ) );
+  qfilebuttongroup->setColumnLayout(0, Qt::Vertical );
+  qfilebuttongroup->layout()->setSpacing( 0 );
+  qfilebuttongroup->layout()->setMargin( 0 );
 
   // Create a layout to organize the widgets.
-  QVBoxLayout *qvboxlayout = new QVBoxLayout(qbuttongroup->layout());
-  qvboxlayout->setAlignment( Qt::AlignTop );
-  qvboxlayout->setSpacing( 6 );
-  qvboxlayout->setMargin( 11 );
+  QVBoxLayout *qvboxfilelayout = new QVBoxLayout(qfilebuttongroup->layout());
+  qvboxfilelayout->setAlignment( Qt::AlignTop );
+  qvboxfilelayout->setSpacing( 6 );
+  qvboxfilelayout->setMargin( 11 );
 
   // Create a check box to be in the group box
-  m_qradiobuttonStartPrompt = new QRadioButton("start_prompt", qbuttongroup);
+  m_qradiobuttonStartPrompt = new QRadioButton("start_prompt", qfilebuttongroup);
   m_qradiobuttonStartPrompt->setText( i18n( "Start with dialog prompt (default)" ) );
-  qvboxlayout->addWidget(m_qradiobuttonStartPrompt);
+  qvboxfilelayout->addWidget(m_qradiobuttonStartPrompt);
 
   // Create another check box to the group box
-  m_qradiobuttonStartFile = new QRadioButton("start_file", qbuttongroup);
+  m_qradiobuttonStartFile = new QRadioButton("start_file", qfilebuttongroup);
   m_qradiobuttonStartFile->setText( i18n( "Start with last file used" ) );
-  qvboxlayout->addWidget(m_qradiobuttonStartFile);
+  qvboxfilelayout->addWidget(m_qradiobuttonStartFile);
+*/
+  // Startup file options
+  // --------------------
+
+  // Create a group box to hold the available options
+  QButtonGroup *qpagebuttongroup = new QButtonGroup(qvboxMainFrame, "GroupBox2");
+  qpagebuttongroup->setTitle( i18n( "Startup page options" ) );
+  qpagebuttongroup->setColumnLayout(0, Qt::Vertical );
+  qpagebuttongroup->layout()->setSpacing( 0 );
+  qpagebuttongroup->layout()->setMargin( 0 );
+
+  // Create a layout to organize the widgets.
+  QVBoxLayout *qvboxpagelayout = new QVBoxLayout(qpagebuttongroup->layout());
+  qvboxpagelayout->setAlignment( Qt::AlignTop );
+  qvboxpagelayout->setSpacing( 6 );
+  qvboxpagelayout->setMargin( 11 );
+
+  // Create a check box to be in the group box
+  m_qradiobuttonStartHome = new QRadioButton("start_home", qpagebuttongroup);
+  m_qradiobuttonStartHome->setText( i18n( "Start with home page" ) );
+  qvboxpagelayout->addWidget(m_qradiobuttonStartHome);
+
+  // Create another check box to the group box
+  m_qradiobuttonStartLast = new QRadioButton("start_last", qpagebuttongroup);
+  m_qradiobuttonStartLast->setText( i18n( "Start with last selected page" ) );
+  qvboxpagelayout->addWidget(m_qradiobuttonStartLast);
 }
 
 void KSettingsDlg::setPageAccountsView()
@@ -129,6 +184,7 @@ void KSettingsDlg::setPageAccountsView()
   m_qradiobuttonAccountView->setText( i18n( "Use the new accounts view" ) );
   qvboxlayout->addWidget(m_qradiobuttonAccountView);
 
+/*
   // Create a group box to hold the available options for the wizard
   QButtonGroup *qwizardbuttongroup = new QButtonGroup(qvboxMainFrame, "GroupBox1");
   qwizardbuttongroup->setTitle( i18n( "New Account Settings" ) );
@@ -151,6 +207,84 @@ void KSettingsDlg::setPageAccountsView()
   m_qradiobuttonAccountWizard = new QRadioButton("account_view", qwizardbuttongroup);
   m_qradiobuttonAccountWizard->setText( i18n( "Use the new account wizard" ) );
   qwizardvboxlayout->addWidget(m_qradiobuttonAccountWizard);
+*/
+  // Create a check box for the hide category feature
+  m_qcheckboxHideCategory = new QCheckBox("hide_categories", qvboxMainFrame);
+  m_qcheckboxHideCategory->setText( i18n( "Don't show unused categories" ) );
+}
+
+void KSettingsDlg::setHomePage()
+{
+  // Create the page.
+  QVBox *qvboxMainFrame = addVBoxPage( i18n("Home"), i18n("Home page settings"),
+    DesktopIcon("home"));
+
+  QFrame* frame = new QFrame(qvboxMainFrame, "frame");
+  
+  QHBoxLayout* Form1Layout = new QHBoxLayout( frame, 11, 6, "Form1Layout");
+
+  m_homePageList = new KListView( frame, "KListView1" );
+  m_homePageList->setMinimumSize( QSize( 250, 0 ) );
+  m_homePageList->setSelectionMode(QListView::Single);
+  m_homePageList->setSorting(-1);
+  m_homePageList->header()->hide();
+  m_homePageList->setAllColumnsShowFocus(true);
+  m_homePageList->setColumnWidth(0, 250);
+  
+  Form1Layout->addWidget( m_homePageList );
+
+  QVBoxLayout*  Layout5 = new QVBoxLayout( 0, 0, 6, "Layout5");
+  QSpacerItem* spacer = new QSpacerItem( 0, 40, QSizePolicy::Minimum, QSizePolicy::Expanding );
+  Layout5->addItem( spacer );
+
+  QHBoxLayout* Layout2 = new QHBoxLayout( 0, 0, 6, "Layout2");
+
+  QVBoxLayout* Layout1 = new QVBoxLayout( 0, 0, 6, "Layout1");
+
+  
+  KIconLoader* il = KGlobal::iconLoader();
+  KGuiItem upButtonItem( i18n( "&Up" ),
+                    QIconSet(il->loadIcon("up", KIcon::Small, KIcon::SizeSmall)),
+                    i18n("Move selected item up"),
+                    i18n("Use this to move the selected item up by one position in the list."));
+  m_upButton = new KPushButton( upButtonItem, frame, "KPushButtonUp" );
+  Layout1->addWidget( m_upButton );
+
+  KGuiItem downButtonItem( i18n( "&Down" ),
+                    QIconSet(il->loadIcon("down", KIcon::Small, KIcon::SizeSmall)),
+                    i18n("Move selected item down"),
+                    i18n("Use this to move the selected item down by one position in the list."));
+  m_downButton = new KPushButton(downButtonItem, frame, "KPushButtonDown" );
+  Layout1->addWidget( m_downButton );
+  Layout2->addLayout( Layout1 );
+  
+  QSpacerItem* spacer_2 = new QSpacerItem( 177, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
+  Layout2->addItem( spacer_2 );
+  Layout5->addLayout( Layout2 );
+  QSpacerItem* spacer_3 = new QSpacerItem( 0, 40, QSizePolicy::Minimum, QSizePolicy::Expanding );
+  Layout5->addItem( spacer_3 );
+
+  QLabel* TextLabel1 = new QLabel( frame, "TextLabel1" );
+  TextLabel1->setText( i18n(
+     "Selected entries are shown on the home page of the application.\n\n"
+     "Use the buttons and checkboxes to customize the layout of the home page."
+     ) );
+  TextLabel1->setAlignment( int( QLabel::WordBreak | QLabel::AlignTop ) );
+
+  Layout5->addWidget( TextLabel1 );
+  QSpacerItem* spacer_4 = new QSpacerItem( 0, 40, QSizePolicy::Minimum, QSizePolicy::Expanding );
+  Layout5->addItem( spacer_4 );
+  Form1Layout->addLayout( Layout5 );
+
+  m_homePageList->addColumn("");
+
+  m_currentItem = -1;   // none selected
+
+  connect(m_homePageList, SIGNAL(selectionChanged(QListViewItem*)),
+          this, SLOT(slotSelectHomePageItem(QListViewItem *)));
+
+  connect(m_upButton, SIGNAL(clicked()), this, SLOT(slotMoveUp()));          
+  connect(m_downButton, SIGNAL(clicked()), this, SLOT(slotMoveDown()));
 }
 
 /** Called to create the Main List page shown in the dialog.
@@ -223,6 +357,7 @@ void KSettingsDlg::setPageList()
   m_qcheckboxTypeToNr = new QCheckBox(i18n("Insert transaction type into Nr. field for new transactions"), qwidgetPage);
   qvboxlayoutPage->addWidget(m_qcheckboxTypeToNr);
 
+#if 0
   // Create a group to hold two radio buttons
   QButtonGroup *qbuttongroup = new QButtonGroup(qwidgetPage, "ButtonGroup1");
   qbuttongroup->setTitle(i18n("Row Colour options"));
@@ -245,10 +380,32 @@ void KSettingsDlg::setPageList()
   m_qradiobuttonOtherRow = new QRadioButton(qbuttongroup, "m_every_other");
   m_qradiobuttonOtherRow->setText( i18n( "Change colour every other row" ) );
   qvboxlayout->addWidget(m_qradiobuttonOtherRow);
+  
   qvboxlayoutPage->addWidget(qbuttongroup);
+#endif
+
+  // Add a vertical spacer to take up the remaining available space
+  QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
+  qvboxlayoutPage->addItem( spacer );
+
+  // Add the page to the tab
+  qtabwidget->insertTab(qwidgetPage, i18n("General"));
+
+
+
+
+
+
+  // Create a new tab for the filter options
+  QWidget *qwidgetFilter = new QWidget(qtabwidget, "filterTab");
+
+  // Create a vertical layout
+  QVBoxLayout *qvboxlayoutFilter = new QVBoxLayout(qwidgetFilter);
+  qvboxlayoutFilter->setSpacing( 6 );
+  qvboxlayoutFilter->setMargin( 11 );
 
   // Create a group to hold two radio buttons
-  QButtonGroup *qbuttongroupDates = new QButtonGroup(qwidgetPage, "ButtonGroup1");
+  QButtonGroup *qbuttongroupDates = new QButtonGroup(qwidgetFilter, "ButtonGroup1");
   qbuttongroupDates->setTitle(i18n("Restrict by date"));
   qbuttongroupDates->setColumnLayout(0, Qt::Vertical );
   qbuttongroupDates->layout()->setSpacing( 0 );
@@ -263,87 +420,14 @@ void KSettingsDlg::setPageList()
   QLabel *m_qlabelStartDate = new QLabel(i18n("Start Date: "), qbuttongroupDates);
   qhboxlayout2->addWidget(m_qlabelStartDate);
   qhboxlayout2->addWidget(m_dateinputStart);
-  qvboxlayoutPage->addWidget(qbuttongroupDates);
-
+  qvboxlayoutFilter->addWidget(qbuttongroupDates);
 
   // Add a vertical spacer to take up the remaining available space
-  QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
-  qvboxlayoutPage->addItem( spacer );
+  spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
+  qvboxlayoutFilter->addItem( spacer );
 
   // Add the page to the tab
-  qtabwidget->insertTab(qwidgetPage, i18n("General"));
-
-  // Create a new tab for the colour options
-  QWidget *qwidgetColour = new QWidget(qtabwidget, "tab");
-
-  // Create a vertical layout
-  QVBoxLayout *qvboxlayoutColour = new QVBoxLayout(qwidgetColour);
-  qvboxlayoutColour->setSpacing( 6 );
-  qvboxlayoutColour->setMargin( 11 );
-
-  // Create a horizontal layout to include the label and button
-  QHBoxLayout *qhboxlayoutColour = new QHBoxLayout;
-  qhboxlayoutColour->setSpacing( 6 );
-  qhboxlayoutColour->setMargin( 0 );
-
-  // Add the label and button
-  QLabel *qlabelListColour = new QLabel(i18n( "List view colour :" ), qwidgetColour);
-  qhboxlayoutColour->addWidget(qlabelListColour);
-  m_kcolorbuttonList = new KColorButton(qwidgetColour, "colour_list");
-  qhboxlayoutColour->addWidget(m_kcolorbuttonList);
-
-  // Add the horizontal layout
-  qvboxlayoutColour->addLayout(qhboxlayoutColour);
-
-  // Create another horizontal layout to include the label and button
-  QHBoxLayout *qhboxlayoutBGColour = new QHBoxLayout;
-  qhboxlayoutBGColour->setSpacing( 6 );
-  qhboxlayoutBGColour->setMargin( 0 );
-
-  // Add the label and button
-  QLabel *qlabelListBGColour = new QLabel(i18n( "List background colour :" ), qwidgetColour);
-  qhboxlayoutBGColour->addWidget(qlabelListBGColour);
-  m_kcolorbuttonBack = new KColorButton(qwidgetColour, "colour_back");
-  qhboxlayoutBGColour->addWidget(m_kcolorbuttonBack);
-
-  // Add the horizontal layout
-  qvboxlayoutColour->addLayout(qhboxlayoutBGColour);
-
-  // Create another horizontal layout to include the label and button
-  QHBoxLayout *qhboxlayoutGridColour = new QHBoxLayout;
-  qhboxlayoutBGColour->setSpacing( 6 );
-  qhboxlayoutBGColour->setMargin( 0 );
-
-  // Add the label and button
-  QLabel *qlabelListGridColour = new QLabel(i18n( "List grid colour :" ), qwidgetColour);
-  qhboxlayoutGridColour->addWidget(qlabelListGridColour);
-  m_kcolorbuttonGrid = new KColorButton(qwidgetColour, "colour_grid");
-  qhboxlayoutGridColour->addWidget(m_kcolorbuttonGrid);
-
-  // Add the horizontal layout
-  qvboxlayoutColour->addLayout(qhboxlayoutGridColour);
-
-
-  // Add a vertical spacer to take up the remaining available space
-  QSpacerItem* qspaceritemColour = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
-  qvboxlayoutColour->addItem( qspaceritemColour );
-
-  // Add the page to the tab widget
-  qtabwidget->insertTab(qwidgetColour, i18n( "Color"));
-
-  // Create another tab adding a font chooser widget
-  QVBox *qvboxInsideTab1 = new QVBox( this, "tab1" );
-  qvboxInsideTab1->setSpacing( 6 );
-  qvboxInsideTab1->setMargin( 11 );
-  m_kfontchooserHeader = new KFontChooser(qvboxInsideTab1);
-  qtabwidget->insertTab(qvboxInsideTab1, i18n("Header Font"));
-
-  // Create another tab adding a font chooser widget
-  QVBox *qvboxInsideTab2 = new QVBox( this, "tab2" );
-  qvboxInsideTab2->setSpacing( 6 );
-  qvboxInsideTab2->setMargin( 11 );
-  m_kfontchooserCell = new KFontChooser(qvboxInsideTab2);
-  qtabwidget->addTab(qvboxInsideTab2, i18n("Cell Font"));
+  qtabwidget->insertTab(qwidgetFilter, i18n("Filter"));
 }
 
 /** Read all the settings in from the global KConfig object and set all the
@@ -353,17 +437,22 @@ void KSettingsDlg::configRead()
 {
   KConfig *kconfig = KGlobal::config();
   kconfig->setGroup("Settings Dialog");
-  QSize *qsizeDefaultSize = new QSize(470,470);
-  this->resize(kconfig->readSizeEntry("Geometry", qsizeDefaultSize));
+  QSize qsizeDefaultSize(470,470);
+  this->resize(kconfig->readSizeEntry("Geometry", &qsizeDefaultSize));
 
   kconfig->setGroup("General Options");
-  m_bTempStartPrompt = kconfig->readBoolEntry("StartDialog", true);
-  m_qradiobuttonStartPrompt->setChecked(m_bTempStartPrompt);
-  m_qradiobuttonStartFile->setChecked(!m_bTempStartPrompt);
+  // FIXME: to be removed
+  // m_bTempStartPrompt = kconfig->readBoolEntry("StartDialog", true);
+  // m_qradiobuttonStartPrompt->setChecked(m_bTempStartPrompt);
+  // m_qradiobuttonStartFile->setChecked(!m_bTempStartPrompt);
 
-  m_bTempAccountWizard = kconfig->readBoolEntry("NewAccountWizard", true);
-  m_qradiobuttonAccountWizard->setChecked(m_bTempAccountWizard);
-  m_qradiobuttonAccountDialog->setChecked(!m_bTempAccountWizard);
+  m_bTempStartPage = kconfig->readBoolEntry("StartLastViewSelected", false);
+  m_qradiobuttonStartHome->setChecked(!m_bTempStartPage);
+  m_qradiobuttonStartLast->setChecked(m_bTempStartPage);
+  
+  // m_bTempAccountWizard = kconfig->readBoolEntry("NewAccountWizard", true);
+  // m_qradiobuttonAccountWizard->setChecked(m_bTempAccountWizard);
+  // m_qradiobuttonAccountDialog->setChecked(!m_bTempAccountWizard);
 
   m_bTempLedgerLens = kconfig->readBoolEntry("LedgerLens", true);
   m_qcheckboxLedgerLens->setChecked(m_bTempLedgerLens);
@@ -384,44 +473,40 @@ void KSettingsDlg::configRead()
     m_qcheckboxTypeToNr->setEnabled(false);
   }
 
+ 
   kconfig->setGroup("List Options");
 
-  QFont qfontDefault = QFont("helvetica", 12);
-  QColor qcolorDefault = Qt::white;
-  QColor qcolorDefaultBG = Qt::gray;
-  QColor qcolorDefaultGrid = Qt::black;
-
-  m_qcolorTempList = kconfig->readColorEntry("listColor", &qcolorDefault);
+  m_qcolorTempList = KMyMoneyUtils::defaultListColour();
   m_kcolorbuttonList->setColor(m_qcolorTempList);
 
-  m_qcolorTempListBG = kconfig->readColorEntry("listBGColor", &qcolorDefaultBG);
+  m_qcolorTempListBG = KMyMoneyUtils::defaultBackgroundColour();
   m_kcolorbuttonBack->setColor(m_qcolorTempListBG);
 
-  m_qcolorTempListGrid = kconfig->readColorEntry("listGridColor", &qcolorDefaultGrid);
+  m_qcolorTempListGrid = KMyMoneyUtils::defaultGridColour();
   m_kcolorbuttonGrid->setColor(m_qcolorTempListGrid);
 
+  QFont qfontDefault = KMyMoneyUtils::headerFont();
   m_qfontTempHeader = kconfig->readFontEntry("listHeaderFont", &qfontDefault);
   m_kfontchooserHeader->setFont(m_qfontTempHeader);
 
+  qfontDefault = KMyMoneyUtils::cellFont();
   m_qfontTempCell = kconfig->readFontEntry("listCellFont", &qfontDefault);
   m_kfontchooserCell->setFont(m_qfontTempCell);
 
-/*
-  m_qstringTempRowCount = kconfig->readEntry("RowCount", "2");
-  m_klineeditRowCount->setText(m_qstringTempRowCount);
-*/
+  m_bTempUseSystemFont = kconfig->readBoolEntry("useSystemFont", true);
+  m_systemFont->setChecked(m_bTempUseSystemFont);
 
   m_bTempShowGrid = kconfig->readBoolEntry("ShowGrid", true);
   m_qcheckboxShowGrid->setChecked(m_bTempShowGrid);
 
-/*
-  m_bTempTextPrompt = kconfig->readBoolEntry("TextPrompt", true);
-  m_qcheckboxTextPrompt->setChecked(m_bTempTextPrompt);
-*/
+  m_bTempHideCategory = kconfig->readBoolEntry("HideUnusedCategory", false);
+  m_qcheckboxHideCategory->setChecked(m_bTempHideCategory);
 
+#if 0  
   m_bTempColourPerTransaction = kconfig->readBoolEntry("ColourPerTransaction", true);
   m_qradiobuttonPerTransaction->setChecked(m_bTempColourPerTransaction);
   m_qradiobuttonOtherRow->setChecked(!m_bTempColourPerTransaction);
+#endif
 
   QDateTime defaultDate;
   defaultDate.setTime_t(0);
@@ -429,9 +514,22 @@ void KSettingsDlg::configRead()
   m_qdateTempStart = kconfig->readDateTimeEntry("StartDate", &defaultDate).date();
   m_dateinputStart->setDate(m_qdateTempStart);
 
-  m_bTempNormalView = kconfig->readBoolEntry("NormalAccountsView", true);
+  m_bTempNormalView = kconfig->readBoolEntry("NormalAccountsView", false);
   m_qradiobuttonNormalView->setChecked(m_bTempNormalView);
   m_qradiobuttonAccountView->setChecked(!m_bTempNormalView);
+
+  kconfig->setGroup("Homepage Options");
+  m_tempHomePageItems = kconfig->readListEntry("Itemlist");
+  QStringList list = m_tempHomePageItems;
+  KMyMoneyUtils::addDefaultHomePageItems(list);
+  fillHomePageItems(list);
+
+  kconfig->setGroup("Schedule Options");
+  m_bTempCheckSchedule = kconfig->readBoolEntry("CheckSchedules", true);
+  m_qradiobuttonCheckSchedules->setChecked(m_bTempCheckSchedule);
+  m_intSchedulePreview->setEnabled(m_bTempCheckSchedule);
+  m_iTempSchedulePreview = kconfig->readNumEntry("CheckSchedulePreview", 0);
+  m_intSchedulePreview->setValue(m_iTempSchedulePreview);
 }
 
 /** Write out all the settings to the global KConfig object.
@@ -448,10 +546,12 @@ void KSettingsDlg::configWrite()
   kconfig->writeEntry("listGridColor", m_kcolorbuttonGrid->color());
   kconfig->writeEntry("listHeaderFont", m_kfontchooserHeader->font());
   kconfig->writeEntry("listCellFont", m_kfontchooserCell->font());
+  kconfig->writeEntry("useSystemFont", m_systemFont->isChecked());
+
   // kconfig->writeEntry("RowCount", m_klineeditRowCount->text());
-  // kconfig->writeEntry("ShowGrid", m_qcheckboxShowGrid->isChecked());
-  kconfig->writeEntry("ColourPerTransaction", m_qradiobuttonPerTransaction->isChecked());
-  // kconfig->writeEntry("TextPrompt", m_qcheckboxTextPrompt->isChecked());
+  kconfig->writeEntry("ShowGrid", m_qcheckboxShowGrid->isChecked());
+  // kconfig->writeEntry("ColourPerTransaction", m_qradiobuttonPerTransaction->isChecked());
+  kconfig->writeEntry("HideUnusedCategory", m_qcheckboxHideCategory->isChecked());
 #if QT_VERSION > 300
   kconfig->writeEntry("StartDate", QDateTime(m_dateinputStart->getQDate()));
 #else
@@ -461,12 +561,21 @@ void KSettingsDlg::configWrite()
   kconfig->writeEntry("NormalAccountsView", m_qradiobuttonNormalView->isChecked());
 
   kconfig->setGroup("General Options");
-  kconfig->writeEntry("StartDialog", m_qradiobuttonStartPrompt->isChecked());
-  kconfig->writeEntry("NewAccountWizard", m_qradiobuttonAccountWizard->isChecked());
+  // kconfig->writeEntry("StartDialog", m_qradiobuttonStartPrompt->isChecked());
+  kconfig->writeEntry("StartLastViewSelected", m_qradiobuttonStartLast->isChecked());
+  // kconfig->writeEntry("NewAccountWizard", true);
   kconfig->writeEntry("LedgerLens", m_qcheckboxLedgerLens->isChecked());
   kconfig->writeEntry("TransactionForm", m_qcheckboxTransactionForm->isChecked());
   kconfig->writeEntry("CopyTypeToNr", m_qcheckboxTypeToNr->isChecked());
   kconfig->writeEntry("AlwaysShowNrField", m_qcheckboxShowNrField->isChecked());
+
+  kconfig->setGroup("Homepage Options");
+  kconfig->writeEntry("Itemlist", homePageItems());
+
+  kconfig->setGroup("Schedule Options");
+  kconfig->writeEntry("CheckSchedules", m_qradiobuttonCheckSchedules->isChecked());
+  kconfig->writeEntry("CheckSchedulePreview", m_intSchedulePreview->value());
+
   kconfig->sync();
 }
 
@@ -515,8 +624,8 @@ void KSettingsDlg::slotCancel()
   kconfig->writeEntry("listCellFont", m_qfontTempCell);
   // kconfig->writeEntry("RowCount", m_qstringTempRowCount);
   kconfig->writeEntry("ShowGrid", m_bTempShowGrid);
-  kconfig->writeEntry("ColourPerTransaction", m_bTempColourPerTransaction);
-  kconfig->writeEntry("TextPrompt", m_bTempTextPrompt);
+  // kconfig->writeEntry("ColourPerTransaction", m_bTempColourPerTransaction);
+  kconfig->writeEntry("HideUnusedCategory", m_bTempHideCategory);
 #if QT_VERSION > 300
   kconfig->writeEntry("StartDate", QDateTime(m_qdateTempStart));
 #else
@@ -526,13 +635,18 @@ void KSettingsDlg::slotCancel()
   kconfig->writeEntry("NormalAccountsView", m_bTempNormalView);
 
   kconfig->setGroup("General Options");
-  kconfig->writeEntry("StartDialog", m_bTempStartPrompt);
-  kconfig->writeEntry("NewAccountWizard", m_bTempAccountWizard);
+  // kconfig->writeEntry("StartDialog", m_bTempStartPrompt);
+  kconfig->writeEntry("StartLastViewSelected", m_bTempStartPage);
+  // kconfig->writeEntry("NewAccountWizard", m_bTempAccountWizard);
   kconfig->writeEntry("LedgerLens", m_bTempLedgerLens);
   kconfig->writeEntry("TransactionForm", m_bTempTransactionForm);
   kconfig->writeEntry("CopyTypeToNr", m_bTempTypeToNr);
   kconfig->writeEntry("AlwaysShowNrField", m_bTempShowNrField);
-
+  kconfig->writeEntry("CheckSchedule", m_bTempCheckSchedule);
+  kconfig->writeEntry("CheckSchedulePreview", m_iTempSchedulePreview);
+  
+  kconfig->setGroup("Homepage Options");
+  kconfig->writeEntry("Itemlist", m_tempHomePageItems);
   kconfig->sync();
 
   if (m_bDoneApply)
@@ -548,25 +662,38 @@ void KSettingsDlg::slotCancel()
 **/
 void KSettingsDlg::slotUser1()
 {
-  m_qradiobuttonStartPrompt->setChecked(m_bTempStartPrompt);
+  // m_qradiobuttonStartPrompt->setChecked(m_bTempStartPrompt);
+  // m_qradiobuttonStartFile->setChecked(!m_bTempStartPrompt);
   m_kcolorbuttonList->setColor(m_qcolorTempList);
   m_kcolorbuttonBack->setColor(m_qcolorTempListBG);
   m_kfontchooserHeader->setFont(m_qfontTempHeader);
   m_kfontchooserCell->setFont(m_qfontTempCell);
+  m_systemFont->setChecked(m_bTempUseSystemFont);
   // m_klineeditRowCount->setText(m_qstringTempRowCount);
   m_qcheckboxShowGrid->setChecked(m_bTempShowGrid);
+#if 0  
   m_qradiobuttonPerTransaction->setChecked(m_bTempColourPerTransaction);
   m_qradiobuttonOtherRow->setChecked(!m_bTempColourPerTransaction);
-  // m_qcheckboxTextPrompt->setChecked(m_bTempTextPrompt);
+#endif  
+  m_qcheckboxHideCategory->setChecked(m_bTempHideCategory);
   m_dateinputStart->setDate(m_qdateTempStart);
   m_qradiobuttonNormalView->setChecked(m_bTempNormalView);
   m_qradiobuttonAccountView->setChecked(!m_bTempNormalView);
-  m_qradiobuttonAccountWizard->setChecked(m_bTempAccountWizard);
-  m_qradiobuttonAccountDialog->setChecked(!m_bTempAccountWizard);
+  // m_qradiobuttonAccountWizard->setChecked(m_bTempAccountWizard);
+  // m_qradiobuttonAccountDialog->setChecked(!m_bTempAccountWizard);
   m_qcheckboxLedgerLens->setChecked(m_bTempLedgerLens);
   m_qcheckboxTransactionForm->setChecked(m_bTempTransactionForm);
   m_qcheckboxTypeToNr->setChecked(m_bTempTypeToNr);
   m_qcheckboxShowNrField->setChecked(m_bTempShowNrField);
+  m_qradiobuttonStartLast->setChecked(m_bTempStartPage);
+  m_qradiobuttonStartHome->setChecked(!m_bTempStartPage);
+  m_qradiobuttonCheckSchedules->setChecked(m_bTempCheckSchedule);
+  m_intSchedulePreview->setEnabled(m_bTempCheckSchedule);
+  m_intSchedulePreview->setValue(m_iTempSchedulePreview);
+  
+  QStringList list = m_tempHomePageItems;
+  KMyMoneyUtils::addDefaultHomePageItems(list);
+  fillHomePageItems(list);
 }
 
 void KSettingsDlg::slotNrFieldToggled(bool state)
@@ -578,3 +705,207 @@ void KSettingsDlg::slotNrFieldToggled(bool state)
     m_qcheckboxTypeToNr->setEnabled(false);
   }
 }
+
+const QStringList KSettingsDlg::homePageItems(void) const
+{
+  QStringList list;
+  QListViewItem *it;
+
+  for(it = m_homePageList->firstChild(); it; it = it->nextSibling()) {
+    int item = KMyMoneyUtils::stringToHomePageItem(it->text(0));
+    if(!(static_cast<QCheckListItem*>(it)->isOn()))
+      item = -item;
+    list << QString::number(item);
+  }
+  return list;
+}
+
+void KSettingsDlg::fillHomePageItems(QStringList& list)
+{
+  QStringList::ConstIterator it;
+  int w = 0;
+  m_homePageList->clear();
+  QCheckListItem *sel = 0;
+    
+  m_upButton->setEnabled(false);
+  m_downButton->setEnabled(false);
+
+  QFontMetrics fm( KGlobalSettings::generalFont());
+  QCheckListItem* last = 0;
+
+  for(it = list.begin(); it != list.end(); ++it) {
+    int idx = (*it).toInt();
+    bool enabled = idx > 0;
+    if(!enabled) idx = -idx;
+    QCheckListItem* item = new QCheckListItem(m_homePageList, KMyMoneyUtils::homePageItemToString(idx), QCheckListItem::CheckBox);
+    if(last)
+      item->moveItem(last);
+
+    // qDebug("Adding %s", item->text(0).latin1());
+    item->setOn(enabled);
+    if(item->width(fm, m_homePageList, 0) > w)
+      w = item->width(fm, m_homePageList, 0);
+      
+    if(sel == 0)
+      sel = item;
+    last = item;
+  }
+
+  if(sel) {
+    m_homePageList->setSelected(sel, true);
+    slotSelectHomePageItem(sel);
+  }
+
+  m_homePageList->setMinimumWidth(w+30);
+  m_homePageList->setMaximumWidth(w+30);
+  m_homePageList->setColumnWidth(0, w+28);
+}
+
+void KSettingsDlg::slotSystemFontToggled(bool state)
+{
+  m_fontTabWidget->setEnabled(!state);
+}
+
+void KSettingsDlg::slotSelectHomePageItem(QListViewItem *item)
+{
+  m_upButton->setEnabled(m_homePageList->firstChild() != item);
+  m_downButton->setEnabled(item->nextSibling());  
+}
+
+void KSettingsDlg::slotMoveUp(void)
+{
+  QListViewItem *item = m_homePageList->currentItem();
+  QListViewItem *prev = item->itemAbove();
+  if(prev) {
+    prev->moveItem(item);
+    slotSelectHomePageItem(item);
+  }
+}
+
+void KSettingsDlg::slotMoveDown(void)
+{
+  QListViewItem *item = m_homePageList->currentItem();
+  QListViewItem *next = item->nextSibling();
+  if(next) {
+    item->moveItem(next);
+    slotSelectHomePageItem(item);
+  }
+}
+
+void KSettingsDlg::setPageSchedule()
+{
+  // Create the main frame to hold the widgets
+  QVBox *qvboxMainFrame = addVBoxPage( i18n("Schedules"), i18n("Schedule settings"),
+    DesktopIcon("schedule"));
+
+  // Startup schedule options
+  // --------------------
+  QGroupBox* groupBox1 = new QGroupBox( qvboxMainFrame, "groupBox1" );
+  groupBox1->setTitle( i18n( "Schedule startup options" ) );
+  groupBox1->setGeometry( QRect( 10, 10, 374, 370 ) );
+  groupBox1->setColumnLayout(0, Qt::Vertical );
+  groupBox1->layout()->setSpacing( 6 );
+  groupBox1->layout()->setMargin( 11 );
+
+  QVBoxLayout* groupBox1Layout = new QVBoxLayout( groupBox1->layout() );
+  groupBox1Layout->setAlignment( Qt::AlignTop );
+
+  m_qradiobuttonCheckSchedules = new QCheckBox( groupBox1, "m_qradiobuttonCheckSchedules" );
+  m_qradiobuttonCheckSchedules->setText( i18n( "Check schedules upon startup" ) );
+  groupBox1Layout->addWidget( m_qradiobuttonCheckSchedules );
+
+  QHBoxLayout* layout1 = new QHBoxLayout( 0, 0, 6, "layout1");
+
+  QLabel* textLabel1 = new QLabel( groupBox1, "textLabel1" );
+  textLabel1->setText( i18n( "Enter transactions this number of days in advance" ) );
+  layout1->addWidget( textLabel1 );
+
+  m_intSchedulePreview = new KIntNumInput( groupBox1, "m_intSchedulePreview" );
+  m_intSchedulePreview->setRange(0, INT_MAX, 1, false);
+  layout1->addWidget( m_intSchedulePreview );
+  groupBox1Layout->addLayout( layout1 );
+  QSpacerItem* spacer = new QSpacerItem( 20, 21, QSizePolicy::Minimum, QSizePolicy::Expanding );
+  groupBox1Layout->addItem( spacer );
+
+  connect(m_qradiobuttonCheckSchedules, SIGNAL(toggled(bool)), m_intSchedulePreview, SLOT(setEnabled(bool)));
+}
+
+void KSettingsDlg::setPageColour()
+{
+  // Create the page.
+  QVBox *qvboxMainFrame = addVBoxPage( i18n("Colours"), i18n("Colour settings"),
+    DesktopIcon("colorscm"));
+
+  QGroupBox* groupBox2 = new QGroupBox( qvboxMainFrame, "groupBox2" );
+  groupBox2->setTitle( i18n( "List Colours" ) );
+  groupBox2->setGeometry( QRect( 20, 10, 430, 340 ) );
+  groupBox2->setColumnLayout(0, Qt::Vertical );
+  groupBox2->layout()->setSpacing( 6 );
+  groupBox2->layout()->setMargin( 11 );
+
+  QVBoxLayout* groupBox2Layout = new QVBoxLayout( groupBox2->layout() );
+  groupBox2Layout->setAlignment( Qt::AlignTop );
+
+  QHBoxLayout* layout2 = new QHBoxLayout( 0, 0, 6, "layout2");
+
+  QLabel* textLabel2 = new QLabel( groupBox2, "textLabel2" );
+  textLabel2->setText( i18n( "List view colour :" ) );
+  layout2->addWidget( textLabel2 );
+
+  m_kcolorbuttonList = new KColorButton( groupBox2, "m_kcolorbuttonList" );
+  layout2->addWidget( m_kcolorbuttonList );
+  groupBox2Layout->addLayout( layout2 );
+
+  QHBoxLayout* layout3 = new QHBoxLayout( 0, 0, 6, "layout3");
+
+  QLabel* textLabel2_2 = new QLabel( groupBox2, "textLabel2_2" );
+  textLabel2_2->setText( i18n( "List background colour :" ) );
+  layout3->addWidget( textLabel2_2 );
+
+  m_kcolorbuttonBack = new KColorButton( groupBox2, "m_kcolorbuttonBack" );
+  layout3->addWidget( m_kcolorbuttonBack );
+  groupBox2Layout->addLayout( layout3 );
+
+  QHBoxLayout* layout4 = new QHBoxLayout( 0, 0, 6, "layout4");
+
+  QLabel* textLabel2_3 = new QLabel( groupBox2, "textLabel2_3" );
+  textLabel2_3->setText( i18n( "List grid colour :" ) );
+  layout4->addWidget( textLabel2_3 );
+
+  m_kcolorbuttonGrid = new KColorButton( groupBox2, "m_kcolorbuttonGrid" );
+  layout4->addWidget( m_kcolorbuttonGrid );
+  groupBox2Layout->addLayout( layout4 );
+
+  QSpacerItem* spacer = new QSpacerItem( 20, 31, QSizePolicy::Minimum, QSizePolicy::Expanding );
+  groupBox2Layout->addItem( spacer );
+}
+
+void KSettingsDlg::setPageFont()
+{
+  // Create the page.
+  QVBox *qvboxMainFrame = addVBoxPage( i18n("Fonts"), i18n("Font settings"),
+    DesktopIcon("font"));
+
+  m_systemFont = new QCheckBox(qvboxMainFrame, "systemFont");
+  m_systemFont->setText(i18n("Use system fonts"));
+  connect(m_systemFont, SIGNAL(toggled(bool)), this, SLOT(slotSystemFontToggled(bool)));
+
+  m_fontTabWidget = new QTabWidget( qvboxMainFrame, "tabWidget2" );
+
+  QWidget* tab = new QWidget( m_fontTabWidget, "tab" );
+  QVBoxLayout* tabLayout = new QVBoxLayout( tab, 11, 6, "tabLayout");
+
+  m_kfontchooserHeader = new KFontChooser(tab, "m_kfontchooserHeader" );
+  tabLayout->addWidget( m_kfontchooserHeader );
+  m_fontTabWidget->insertTab( tab, QString("") );
+
+  QWidget* tab_2 = new QWidget( m_fontTabWidget, "tab_2" );
+  QVBoxLayout* tabLayout_2 = new QVBoxLayout( tab_2, 11, 6, "tabLayout_2");
+
+  m_kfontchooserCell = new KFontChooser( tab_2, "m_kfontchooserCell" );
+  tabLayout_2->addWidget( m_kfontchooserCell );
+  m_fontTabWidget->insertTab( tab_2, QString("") );
+
+  m_fontTabWidget->changeTab( tab, i18n( "Header Font" ) );
+  m_fontTabWidget->changeTab( tab_2, i18n( "Cell Font" ) );
+}
