@@ -27,7 +27,7 @@
 #include <qlabel.h>
 #include <qtable.h>
 #include <qtimer.h>
-#include <qlist.h>
+#include <qptrlist.h>
 #include <qbuttongroup.h>
 #include <qradiobutton.h>
 
@@ -43,6 +43,8 @@
 #include <klocale.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
+#include <kguiitem.h>
+#include <kpushbutton.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -53,11 +55,13 @@
 #include "../widgets/kmymoneycategory.h"
 #include "../widgets/kmymoneyedit.h"
 #include "../widgets/kmymoneylineedit.h"
+#include "../mymoney/mymoneyfile.h"
 
 KSplitTransactionDlg::KSplitTransactionDlg(const MyMoneyTransaction& t,
                                            const MyMoneyAccount& acc,
                                            const bool amountValid,
                                            const bool deposit,
+                                           const MyMoneyMoney& calculatedValue,
                                            QWidget* parent, const char* name)
   : kSplitTransactionDlgDecl(parent, name, true),
   m_transaction(t),
@@ -66,23 +70,24 @@ KSplitTransactionDlg::KSplitTransactionDlg(const MyMoneyTransaction& t,
   m_amountValid(amountValid),
   m_numExtraLines(0),
   m_editRow(-1),
-  m_isDeposit(deposit)
+  m_isDeposit(deposit),
+  m_calculatedValue(calculatedValue)
 {
   // setup the transactions table
   transactionsTable->setInlineEditingMode(false);
   transactionsTable->setNumRows(1);
   transactionsTable->setNumCols(3);
   transactionsTable->horizontalHeader()->setLabel(0, i18n("Category"));
-	transactionsTable->horizontalHeader()->setLabel(1, i18n("Memo"));
-	transactionsTable->horizontalHeader()->setLabel(2, i18n("Amount"));
-	transactionsTable->setSelectionMode(QTable::NoSelection);
- 	transactionsTable->setLeftMargin(0);
-	transactionsTable->verticalHeader()->hide();
+  transactionsTable->horizontalHeader()->setLabel(1, i18n("Memo"));
+  transactionsTable->horizontalHeader()->setLabel(2, i18n("Amount"));
+  transactionsTable->setSelectionMode(QTable::NoSelection);
+  transactionsTable->setLeftMargin(0);
+  transactionsTable->verticalHeader()->hide();
   transactionsTable->setColumnStretchable(0, false);
   transactionsTable->setColumnStretchable(1, false);
-	transactionsTable->setColumnStretchable(2, false);
-	transactionsTable->horizontalHeader()->setResizeEnabled(false);
-	transactionsTable->horizontalHeader()->setMovingEnabled(false);
+  transactionsTable->setColumnStretchable(2, false);
+  transactionsTable->horizontalHeader()->setResizeEnabled(false);
+  transactionsTable->horizontalHeader()->setMovingEnabled(false);
 
   // for deposits, we invert the sign of all splits.
   // don't forget to revert when we're done ;-)
@@ -94,6 +99,27 @@ KSplitTransactionDlg::KSplitTransactionDlg(const MyMoneyTransaction& t,
     }
   }
 
+  // add icons to buttons
+  KIconLoader *il = KGlobal::iconLoader();
+  KGuiItem finishButtenItem( i18n( "&Finish" ),
+                    QIconSet(il->loadIcon("button_ok", KIcon::Small, KIcon::SizeSmall)),
+                    i18n("Accept splits and return to transaction form"),
+                    i18n("Use this to accept all changes to the splits and return to the transaction"));
+  finishBtn->setGuiItem(finishButtenItem);
+    
+  KGuiItem cancelButtenItem( i18n( "&Cancel" ),
+                    QIconSet(il->loadIcon("button_cancel", KIcon::Small, KIcon::SizeSmall)),
+                    i18n("Reject all changes to the splits and return to transaction form"),
+                    i18n("Use this to reject all changes to the splits and return to the transaction"));
+  cancelBtn->setGuiItem(cancelButtenItem);
+                      
+  KGuiItem clearButtenItem( i18n( "Clear &All" ),
+                    QIconSet(il->loadIcon("edittrash", KIcon::Small, KIcon::SizeSmall)),
+                    i18n("Clear all splits"),
+                    i18n("Use this to clear all splits of this transaction"));
+  clearAllBtn->setGuiItem(clearButtenItem);
+
+  
   // make finish the default
   finishBtn->setDefault(true);
 
@@ -103,9 +129,7 @@ KSplitTransactionDlg::KSplitTransactionDlg(const MyMoneyTransaction& t,
   clearAllBtn->setFocusPolicy(QWidget::NoFocus);
   transactionsTable->setFocus();
 
-	KConfig *config = KGlobal::config();
-  QFont defaultFont = QFont("helvetica", 12);
-  transactionsTable->horizontalHeader()->setFont(config->readFontEntry("listHeaderFont", &defaultFont));
+  transactionsTable->horizontalHeader()->setFont(KMyMoneyUtils::headerFont());
 
   // initialize the display
   updateSplit();
@@ -114,8 +138,8 @@ KSplitTransactionDlg::KSplitTransactionDlg(const MyMoneyTransaction& t,
   transactionsTable->setCurrentRow(m_transaction.splits().count()-1);
 
   // connect signals with slots
-	connect(cancelBtn, SIGNAL(clicked()), this, SLOT(slotCancelClicked()));
-	connect(finishBtn, SIGNAL(clicked()), this, SLOT(slotFinishClicked()));
+  connect(cancelBtn, SIGNAL(clicked()), this, SLOT(slotCancelClicked()));
+  connect(finishBtn, SIGNAL(clicked()), this, SLOT(slotFinishClicked()));
   connect(clearAllBtn, SIGNAL(clicked()), this, SLOT(slotClearAllClicked()));
 
   connect(transactionsTable, SIGNAL(clicked(int, int, int, const QPoint&)),
@@ -138,11 +162,10 @@ KSplitTransactionDlg::KSplitTransactionDlg(const MyMoneyTransaction& t,
     this, SLOT(slotFinishClicked()));
 
   // setup the context menu
-  KIconLoader *kiconloader = KGlobal::iconLoader();
   m_contextMenu = new KPopupMenu(this);
-  m_contextMenu->insertTitle(kiconloader->loadIcon("transaction", KIcon::MainToolbar), i18n("Transaction Options"));
-  m_contextMenu->insertItem(kiconloader->loadIcon("edit", KIcon::Small), i18n("Edit ..."), this, SLOT(slotStartEdit()));
-  m_contextMenuDelete = m_contextMenu->insertItem(kiconloader->loadIcon("delete", KIcon::Small),
+  m_contextMenu->insertTitle(il->loadIcon("transaction", KIcon::MainToolbar), i18n("Transaction Options"));
+  m_contextMenu->insertItem(il->loadIcon("edit", KIcon::Small), i18n("Edit ..."), this, SLOT(slotStartEdit()));
+  m_contextMenuDelete = m_contextMenu->insertItem(il->loadIcon("delete", KIcon::Small),
                         i18n("Delete ..."),
                         this, SLOT(slotDeleteSplit()));
 
@@ -157,10 +180,7 @@ KSplitTransactionDlg::KSplitTransactionDlg(const MyMoneyTransaction& t,
 
 void KSplitTransactionDlg::createInputWidgets(const int row)
 {
-  KConfig *config = KGlobal::config();
-  config->setGroup("List Options");
-  QFont cellFont = QFont("helvetica", 12);
-  cellFont = config->readFontEntry("listCellFont", &cellFont);
+  QFont cellFont = KMyMoneyUtils::cellFont();
   
 
   // create the widgets
@@ -168,12 +188,17 @@ void KSplitTransactionDlg::createInputWidgets(const int row)
   m_editAmount->setFont(cellFont);
   m_editCategory = new kMyMoneyCategory(0);
   m_editCategory->setFont(cellFont);
-  m_editMemo = new kMyMoneyLineEdit(0);
+  m_editMemo = new kMyMoneyLineEdit(0, 0, AlignLeft|AlignVCenter);
   m_editMemo->setFont(cellFont);
 
   m_editCategory->loadText(transactionsTable->text(row, 0));
   m_editMemo->loadText(transactionsTable->text(row, 1));
   m_editAmount->loadText(m_split.value().formatMoney());
+  // don't allow automatically calculated values to be modified
+  if(m_split.value() == MyMoneyMoney::minValue+1) {
+    m_editAmount->setEnabled(false);
+    m_editAmount->loadText("will be calculated");
+  }
 
   transactionsTable->setCellWidget(row, 0, m_editCategory);
   transactionsTable->setCellWidget(row, 1, m_editMemo);
@@ -222,7 +247,7 @@ void KSplitTransactionDlg::initSize(void)
   QDialog::resize(width(), height()+1);
 }
 
-void KSplitTransactionDlg::resizeEvent(QResizeEvent* ev)
+void KSplitTransactionDlg::resizeEvent(QResizeEvent* /* ev */)
 {
   int w = transactionsTable->visibleWidth() - m_amountWidth;
 
@@ -247,8 +272,23 @@ void KSplitTransactionDlg::slotFinishClicked()
   }
 
   if(diffAmount() != 0) {
-    MyMoneySplit split = m_transaction.split(m_account.id());
+    MyMoneySplit split = m_transaction.splitByAccount(m_account.id());
     kSplitCorrectionDlgDecl* dlg = new kSplitCorrectionDlgDecl(0, 0, true);
+
+    // add icons to buttons
+    KIconLoader *il = KGlobal::iconLoader();
+    KGuiItem okButtenItem( i18n("&Ok" ),
+                      QIconSet(il->loadIcon("button_ok", KIcon::Small, KIcon::SizeSmall)),
+                      i18n("Accepts the selected action and continues"),
+                      i18n("Use this to accept the action and perform it"));
+    dlg->okBtn->setGuiItem(okButtenItem);
+
+    KGuiItem cancelButtenItem( i18n( "&Cancel" ),
+                      QIconSet(il->loadIcon("button_cancel", KIcon::Small, KIcon::SizeSmall)),
+                      i18n("Return to split transaction dialog"),
+                      i18n("Use this to continue editing the splits"));
+    dlg->cancelBtn->setGuiItem(cancelButtenItem);
+
     QString total = (-split.value()).formatMoney();
     QString sums = splitsValue().formatMoney();
     QString diff = diffAmount().formatMoney();
@@ -329,7 +369,7 @@ void KSplitTransactionDlg::slotClearAllClicked()
   answer = KMessageBox::warningContinueCancel (NULL,
      i18n("You are about to delete all splits of this transaction. "
           "Do you really want to continue?"),
-     i18n("KMyMoney2"),
+     i18n("KMyMoney"),
      i18n("Continue")
      );
 
@@ -351,7 +391,7 @@ void KSplitTransactionDlg::slotClearAllClicked()
 void KSplitTransactionDlg::updateSums(void)
 {
   MyMoneyMoney splits(splitsValue());
-  MyMoneySplit split = m_transaction.split(m_account.id());
+  MyMoneySplit split = m_transaction.splitByAccount(m_account.id());
 
   if(m_amountValid == false) {
     split.setValue(-splits);
@@ -365,13 +405,14 @@ void KSplitTransactionDlg::updateSums(void)
 
 MyMoneyMoney KSplitTransactionDlg::splitsValue(void)
 {
-  MyMoneyMoney splitsValue(0);
+  MyMoneyMoney splitsValue(m_calculatedValue);
   QValueList<MyMoneySplit> list = getSplits();
   QValueList<MyMoneySplit>::ConstIterator it;
 
   // calculate the current sum of all split parts
   for(it = list.begin(); it != list.end(); ++it) {
-    splitsValue += (*it).value();
+    if((*it).value() != MyMoneyMoney::minValue+1)
+      splitsValue += (*it).value();
   }
 
   return splitsValue;
@@ -384,7 +425,7 @@ MyMoneyMoney KSplitTransactionDlg::diffAmount(void)
   // if there is an amount specified in the transaction, we need to calculate the
   // difference, otherwise we display the difference as 0 and display the same sum.
   if(m_amountValid) {
-    MyMoneySplit split = m_transaction.split(m_account.id());
+    MyMoneySplit split = m_transaction.splitByAccount(m_account.id());
 
     diff = -(splitsValue() + split.value());
   }
@@ -400,8 +441,8 @@ void KSplitTransactionDlg::updateTransactionTableSize(void)
 
   // see if we need some extra lines to fill the current size with the grid
   m_numExtraLines = (tableHeight / rowHeight) - splitCount;
-  if(m_numExtraLines < 0)
-    m_numExtraLines = 0;
+  if(m_numExtraLines < 1)
+    m_numExtraLines = 1;
 
   transactionsTable->setNumRows(splitCount + m_numExtraLines);
   transactionsTable->setMaxRows(splitCount);
@@ -425,16 +466,13 @@ const QValueList<MyMoneySplit> KSplitTransactionDlg::getSplits(void) const
   return list;
 }
 
-void KSplitTransactionDlg::updateSplit(int row, int col)
+void KSplitTransactionDlg::updateSplit(int row, int /* col */)
 {
   unsigned long rowCount=0;
 
   QValueList<MyMoneySplit> list = getSplits();
 
-  KConfig *config = KGlobal::config();
-  config->setGroup("List Options");
-  QFont defaultFont = QFont("helvetica", 12);
-  transactionsTable->horizontalHeader()->setFont(config->readFontEntry("listHeaderFont", &defaultFont));
+  transactionsTable->horizontalHeader()->setFont(KMyMoneyUtils::headerFont());
 
   if (row==-1) { // We are going to refresh the whole list
 
@@ -446,7 +484,7 @@ void KSplitTransactionDlg::updateSplit(int row, int col)
     for(it = list.begin(); it != list.end(); ++it) {
       QString colText;
       MyMoneyMoney value = (*it).value();
-      if((*it).accountId() != "") {
+      if(!(*it).accountId().isEmpty()) {
         try {
           colText = MyMoneyFile::instance()->accountToCategory((*it).accountId());
         } catch(MyMoneyException *e) {
@@ -455,8 +493,12 @@ void KSplitTransactionDlg::updateSplit(int row, int col)
         }
       }
       QString amountTxt = value.formatMoney();
+      if(value == MyMoneyMoney::minValue+1) {
+        amountTxt = i18n("will be calculated");
+      }
+
       if(colText.isEmpty() && (*it).memo().isEmpty() && value == 0)
-        amountTxt = "";
+        amountTxt = QString();
 
       unsigned width = transactionsTable->fontMetrics().width(amountTxt);
       if(width > m_amountWidth)
@@ -484,7 +526,7 @@ void KSplitTransactionDlg::updateSplit(int row, int col)
       MyMoneyMoney value = s.value();
       QString colText;
 
-      if(s.accountId() != "") {
+      if(!s.accountId().isEmpty()) {
         try {
           colText = MyMoneyFile::instance()->accountToCategory(s.accountId());
         } catch(MyMoneyException *e) {
@@ -493,8 +535,11 @@ void KSplitTransactionDlg::updateSplit(int row, int col)
         }
       }
       QString amountTxt = value.formatMoney();
+      if(value == MyMoneyMoney::minValue+1) {
+        amountTxt = i18n("will be calculated");
+      }
       if(colText.isEmpty() && s.memo().isEmpty() && value == 0)
-        amountTxt = "";
+        amountTxt = QString();
 
       unsigned width = transactionsTable->fontMetrics().width(amountTxt);
       if(width > m_amountWidth)
@@ -530,7 +575,7 @@ void KSplitTransactionDlg::slotDeleteSplit(int row)
     answer = KMessageBox::warningContinueCancel (NULL,
        i18n("You are about to delete this part of the transaction. "
             "Do you really want to continue?"),
-       i18n("KMyMoney2"),
+       i18n("KMyMoney"),
        i18n("Continue")
        );
     if(answer == KMessageBox::Continue) {
@@ -553,7 +598,7 @@ void KSplitTransactionDlg::slotStartEdit(void)
   slotStartEdit(transactionsTable->currentRow(), 0, Qt::LeftButton, QPoint(0, 0));
 }
 
-void KSplitTransactionDlg::slotStartEdit(int row, int col, int button, const QPoint&  point)
+void KSplitTransactionDlg::slotStartEdit(int row, int col, int button, const QPoint&  /* point */)
 {
   // only start editing if inside used area
   if(row <= static_cast<int> (m_transaction.splits().count()-1)) {
@@ -571,7 +616,14 @@ void KSplitTransactionDlg::slotStartEdit(int row, int col, int button, const QPo
         m_split = getSplits()[row];
 
       showWidgets(row);
-      m_editCategory->setFocus();
+      // if it's a loan payment transfer, we don't allow to modify the category ;-)
+      if(m_split.action() == MyMoneySplit::ActionAmortization
+      || m_split.action() == MyMoneySplit::ActionInterest) {
+        m_editCategory->setEnabled(false);
+        m_editMemo->setFocus();
+      } else {
+        m_editCategory->setFocus();
+      }
     }
   }
 }
@@ -634,7 +686,7 @@ void KSplitTransactionDlg::slotNavigationKey(int key)
   slotFocusChange(row, 0, Qt::LeftButton, QPoint(0, 0));
 }
 
-void KSplitTransactionDlg::slotFocusChange(int realrow, int col, int button, const QPoint&  point)
+void KSplitTransactionDlg::slotFocusChange(int realrow, int col, int button, const QPoint& /* point */)
 {
   int   row = realrow;
 
@@ -651,7 +703,7 @@ void KSplitTransactionDlg::slotFocusChange(int realrow, int col, int button, con
       && !m_editCategory->text().isEmpty()) {
         // there's data in the split -> update it
         try {
-          if(m_split.id() != "") {
+          if(!m_split.id().isEmpty()) {
             m_transaction.modifySplit(m_split);
           } else {
             m_transaction.addSplit(m_split);
@@ -722,7 +774,7 @@ void KSplitTransactionDlg::slotCategoryChanged(const QString& category)
   try {
     // First, we check if the category exists
     QCString id = MyMoneyFile::instance()->categoryToAccount(category);
-    if(id == "") {
+    if(id.isEmpty()) {
       // FIXME:
       KMessageBox::sorry(0, i18n("Direct creation of new account not yet implemented"));
       m_editCategory->resetText();
@@ -738,7 +790,7 @@ void KSplitTransactionDlg::slotCategoryChanged(const QString& category)
     }
 
     m_split.setAccountId(id);
-
+/*
     // Now we check, if a split referencing this account already exists
     // within this transaction. If so, we join them.
     QValueList<MyMoneySplit> list = getSplits();
@@ -752,7 +804,7 @@ void KSplitTransactionDlg::slotCategoryChanged(const QString& category)
       if((*it).accountId() == m_split.accountId()) {
         (*it).setValue((*it).value() + m_split.value());
         m_transaction.modifySplit(*it);
-        if(m_split.id() != "")
+        if(!m_split.id().isEmpty())
           m_transaction.removeSplit(m_split);
         hideWidgets();
         updateTransactionTableSize();
@@ -760,7 +812,7 @@ void KSplitTransactionDlg::slotCategoryChanged(const QString& category)
         break;
       }
     }
-
+*/
     m_editCategory->loadText(category);
 
   } catch(MyMoneyException *e) {
