@@ -178,6 +178,20 @@ public:
   void setUserTelephone(const QString& val);
   void setUserEmail(const QString& val);
 
+  /**
+    * This method can be used to turn on/off the notifications
+    * of the engine. During bulk-updates it's sometimes useful
+    * to turn off notifications for an improved performance.
+    *
+    * Engine notifications that are generated while the notifications
+    * are suspended will be postponed until the notifications are
+    * re-enabled.
+    *
+    * @param state if @p true, no notifications will be send out,
+    *              if @p false, notifications will be send out.
+    */
+  void suspendNotify(const bool state);
+  
   /**
     * This method is used to attach a storage to the MyMoneyFile object
     * Without an attached storage object, the MyMoneyFile object is
@@ -206,6 +220,14 @@ public:
     */
   bool storageAttached(void) const { return m_storage != 0; };
 
+  /**
+    * This method returns a pointer to the storage object
+    *
+    * @return const pointer to the current attached storage object.
+    *         If no object is attached, returns 0.
+    */
+  IMyMoneyStorage* const storage(void) const { return m_storage; };
+
   /**
     * This method is used to return the standard liability account
     * @return MyMoneyAccount liability account(group)
@@ -238,6 +260,14 @@ public:
     */
   bool dirty(void) const;
 
+  /**
+    * This method is used to force the attached storage object to
+    * be dirty. This is used by the application to re-set the dirty
+    * flag after a failed upload to a server when the save operation
+    * to a local temp file was OK.
+    */
+  void setDirty(void) const;
+  
   /**
     * Adds an institution to the file-global institution pool. A
     * respective institution-ID will be generated for this object.
@@ -420,20 +450,32 @@ public:
     */
   const MyMoneyTransaction& transaction(const QCString& account, const int idx) const;
 
+  /**
+    * This method is used to pull a list of transactions from the file
+    * global transaction pool. It returns all those transactions
+    * that match the filter passed as argument. If the filter is empty,
+    * the whole journal will be returned.
+    *
+    * @param filter MyMoneyTransactionFilter object with the match criteria
+    *
+    * @return set of transactions in form of a QValueList<MyMoneyTransaction>
+    */
+  const QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const;
+
   /**
     * This method is used to pull a list of transactions from the file
     * global transaction pool. It returns either the whole journal or
     * the set of transaction referenced by a specific account depending
     * on the argument given.
     *
-    * @param account QCString reference to account id. If account equals ""
+    * @param account QCString reference to account id. If account is empty
     *                all transactions (the journal) is returned. If account
-    *                is not equal to "" it returns the set of transactions
+    *                is not empty, it returns the set of transactions
     *                that have splits in this account.
     *
     * @return set of transactions in form of a QValueList<MyMoneyTransaction>
     */
-  const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const;
+  // const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const;
 
   /**
     * This method is used to remove a transaction from the transaction
@@ -465,14 +507,25 @@ public:
     * This method returns the number of transactions currently known to file
     * in the range 0..MAXUINT
     *
-    * @param account QCString reference to account id. If account equals ""
+    * @param account QCString reference to account id. If account is empty
     +                all transactions (the journal) will be counted. If account
-    *                is not equal to "" it returns the number of transactions
+    *                is not empty it returns the number of transactions
     *                that have splits in this account.
     *
     * @return number of transactions in journal/account
     */
-  const unsigned int transactionCount(const QCString& account = "") const;
+  const unsigned int transactionCount(const QCString& account = QCString()) const;
+
+  /**
+    * This method returns a QMap filled with the number of transactions
+    * per account. The account id serves as index into the map. If one
+    * needs to have all transactionCounts() for many accounts, this method
+    * is faster than calling transactionCount(const QCString& account) many
+    * times.
+    *
+    * @return QMap with numbers of transactions per account
+    */
+  const QMap<QCString, unsigned long> transactionCountMap(void) const;
 
   /**
     * This method returns the number of institutions currently known to file
@@ -503,7 +556,7 @@ public:
     * This method returns a list of the institutions
     * inside a MyMoneyFile object
     *
-    * @return QValueList containing the institution information
+    * @return QValueList containing the institution objects
     */
   const QValueList<MyMoneyInstitution> institutionList(void) const;
 
@@ -520,7 +573,7 @@ public:
     * This method returns a list of the accounts
     * inside a MyMoneyFile object
     *
-    * @return QMap<QString,MyMoneyAccount> with accounts
+    * @return QValueList containing the account objects
     */
   const QValueList<MyMoneyAccount> accountList(void) const;
 
@@ -562,6 +615,15 @@ public:
     */
   const QCString nameToAccount(const QString& name) const;
 
+  /**
+    * This method is used to extract the parent part of an account hierarchy
+    * name who's parts are seperated by MyMoneyAccount::AccountSeperator.
+    *
+    * @param name full account name
+    * @return parent name (full account name excluding the last part)
+    */
+  const QString parentName(const QString& name) const;
+  
   /**
     * This method is used to attach an observer to a subject
     * represented by it's id. Whenever the object represented
@@ -680,7 +742,85 @@ public:
     */
   void deletePair(const QCString& key);
 
-public:
+  /**
+    * This method is used to add a scheduled transaction to the engine.
+    * It must be sure, that the id of the object is not filled. When the
+    * method returns to the caller, the id will be filled with the
+    * newly created object id value.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param sched reference to the MyMoneySchedule object
+    */
+  void addSchedule(MyMoneySchedule& sched);
+
+  /**
+    * This method is used to modify an existing MyMoneySchedule
+    * object. Therefor, the id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param sched const reference to the MyMoneySchedule object to be updated
+    */
+  void modifySchedule(const MyMoneySchedule& sched);
+
+  /**
+    * This method is used to remove an existing MyMoneySchedule object
+    * from the engine. The id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param sched const reference to the MyMoneySchedule object to be updated
+    */
+  void removeSchedule(const MyMoneySchedule& sched);
+
+  /**
+    * This method is used to retrieve a single MyMoneySchedule object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneySchedule object
+    * @return MyMoneySchedule object
+    */
+  const MyMoneySchedule schedule(const QCString& id) const;
+
+  /**
+    * This method is used to extract a list of scheduled transactions
+    * according to the filter criteria passed as arguments.
+    *
+    * @param accountId only search for scheduled transactions that reference
+    *                  accound @p accountId. If accountId is the empty string,
+    *                  this filter is off. Default is @p QCString().
+    * @param type      only schedules of type @p type are searched for.
+    *                  See MyMoneySchedule::typeE for details.
+    *                  Default is MyMoneySchedule::TYPE_ANY
+    * @param occurance only schedules of occurance type @p occurance are searched for.
+    *                  See MyMoneySchedule::occuranceE for details.
+    *                  Default is MyMoneySchedule::OCCUR_ANY
+    * @param paymentType only schedules of payment method @p paymentType
+    *                  are searched for.
+    *                  See MyMoneySchedule::paymentTypeE for details.
+    *                  Default is MyMoneySchedule::STYPE_ANY
+    * @param startDate only schedules with payment dates after @p startDate
+    *                  are searched for. Default is all dates (QDate()).
+    * @param endDate   only schedules with payment dates ending prior to @p endDate
+    *                  are searched for. Default is all dates (QDate()).
+    * @param overdue   if true, only those schedules that are overdue are
+    *                  searched for. Default is false (all schedules will be returned).
+    *
+    * @return const QValueList<MyMoneySchedule> list of schedule objects.
+    */
+  const QValueList<MyMoneySchedule> scheduleList(const QCString& = QCString(),
+                                     const MyMoneySchedule::typeE = MyMoneySchedule::TYPE_ANY,
+                                     const MyMoneySchedule::occurenceE = MyMoneySchedule::OCCUR_ANY,
+                                     const MyMoneySchedule::paymentTypeE = MyMoneySchedule::STYPE_ANY,
+                                     const QDate& = QDate(),
+                                     const QDate& = QDate(),
+                                     const bool = false) const;
+
+  const QStringList consistencyCheck(void);
+    
   /**
     * MyMoneyFile::NotifyClassAccount
     * is a special id that will be notified whenever any account is changed.
@@ -722,7 +862,44 @@ public:
     */
   static const QCString NotifyClassInstitution;
 
+  /**
+    * MyMoneyFile::NotifyClassSchedule
+    * is a special id that will be notified whenever any schedule is changed
+    */
+  static const QCString NotifyClassSchedule;
 
+  /**
+    * MyMoneyFile::NotifyClassAnyChange
+    * is a special id that will be notified whenever any object of the engine changes
+    */
+  static const QCString NotifyClassAnyChange;
+
+  /**
+    * createCategory creates a category from a text name.
+    *
+    * The whole account hierarchy is created if it doesnt
+    * already exist.  e.g if name = Bills:Credit Card and
+    * base = expense(), Bills will first be checked to see if
+    * it exists and created if not.  Credit Card will then
+    * be created with Bills as it's parent.  The Credit Card account
+    * will have it's id returned.
+    *
+    * @param base The base account (expense or income)
+    * @param name The category to create
+    *
+    * @return The category account id or empty on error.
+    *
+    * @exception An exception will be thrown, if @p base is not equal
+    *            expense() or income().
+    **/
+  QCString createCategory(const MyMoneyAccount& base, const QString& name);
+
+  QValueList<MyMoneySchedule> scheduleListEx( int scheduleTypes,
+                                              int scheduleOcurrences,
+                                              int schedulePaymentTypes,
+                                              QDate startDate,
+                                              const QCStringList& accounts=QCStringList()) const;
+  
 protected:
   /**
     * This is the constructor for a new empty file description
@@ -736,7 +913,7 @@ private:
 private:
   /**
     * This method is used to add an id to the list of objects
-    * to be notified. If id == "", then nothing is added to the list.
+    * to be notified. If id is empty, then nothing is added to the list.
     *
     * @param id id of object to be notified
     * @see attach, detach
@@ -752,7 +929,7 @@ private:
     * This method is used to notify the objects observing
     * a specific id
     */
-  void notify(const QCString& id) const;
+  void notify(const QCString& id);
 
   /**
     * This method is used to notify all objects observing
@@ -777,6 +954,7 @@ private:
       throw new MYMONEYEXCEPTION("No storage object attached to MyMoneyFile");
   }
 
+private:
   /**
     * This member points to the storage strategy
     */
@@ -794,6 +972,10 @@ private:
     */
   QMap<QCString, bool> m_notificationList;
 
+  /**
+    */
+  bool m_suspendNotify;
+  
   static MyMoneyFile* _instance;
 };
 #endif
