@@ -41,13 +41,13 @@ void MyMoneyFileTest::tearDown () {
 }
 
 void MyMoneyFileTest::testEmptyConstructor() {
-	CPPUNIT_ASSERT(m->userName() == "");
-	CPPUNIT_ASSERT(m->userStreet() == "");
-	CPPUNIT_ASSERT(m->userTown() == "");
-	CPPUNIT_ASSERT(m->userCounty() == "");
-	CPPUNIT_ASSERT(m->userPostcode() == "");
-	CPPUNIT_ASSERT(m->userTelephone() == "");
-	CPPUNIT_ASSERT(m->userEmail() == "");
+	CPPUNIT_ASSERT(m->userName().isEmpty());
+	CPPUNIT_ASSERT(m->userStreet().isEmpty());
+	CPPUNIT_ASSERT(m->userTown().isEmpty());
+	CPPUNIT_ASSERT(m->userCounty().isEmpty());
+	CPPUNIT_ASSERT(m->userPostcode().isEmpty());
+	CPPUNIT_ASSERT(m->userTelephone().isEmpty());
+	CPPUNIT_ASSERT(m->userEmail().isEmpty());
 
 	CPPUNIT_ASSERT(m->institutionCount() == 0);
 	CPPUNIT_ASSERT(m->dirty() == false);
@@ -68,7 +68,7 @@ void MyMoneyFileTest::testAddOneInstitution() {
 	// MyMoneyInstitution institution_file("", institution);
 	MyMoneyInstitution institution_id("I000002", institution);
 	MyMoneyInstitution institution_noname(institution);
-	institution_noname.setName("");
+	institution_noname.setName(QString());
 
 	QString id;
 
@@ -295,13 +295,13 @@ void MyMoneyFileTest::testInstitutionModify() {
 }
 
 void MyMoneyFileTest::testSetFunctions() {
-	CPPUNIT_ASSERT(m->userName() == "");
-	CPPUNIT_ASSERT(m->userStreet() == "");
-	CPPUNIT_ASSERT(m->userTown() == "");
-	CPPUNIT_ASSERT(m->userCounty() == "");
-	CPPUNIT_ASSERT(m->userPostcode() == "");
-	CPPUNIT_ASSERT(m->userTelephone() == "");
-	CPPUNIT_ASSERT(m->userEmail() == "");
+	CPPUNIT_ASSERT(m->userName().isEmpty());
+	CPPUNIT_ASSERT(m->userStreet().isEmpty());
+	CPPUNIT_ASSERT(m->userTown().isEmpty());
+	CPPUNIT_ASSERT(m->userCounty().isEmpty());
+	CPPUNIT_ASSERT(m->userPostcode().isEmpty());
+	CPPUNIT_ASSERT(m->userTelephone().isEmpty());
+	CPPUNIT_ASSERT(m->userEmail().isEmpty());
 	
 	storage->m_dirty = false;
 	m->setUserName("Name");
@@ -920,9 +920,12 @@ void MyMoneyFileTest::testModifyTransactionNewAccount() {
 		CPPUNIT_ASSERT(m->account("A000002").transactionCount() == 0);
 		CPPUNIT_ASSERT(m->account("A000003").transactionCount() == 1);
 */
-		CPPUNIT_ASSERT(m->transactionList("A000001").count() == 1);
-		CPPUNIT_ASSERT(m->transactionList("A000002").count() == 0);
-		CPPUNIT_ASSERT(m->transactionList("A000003").count() == 1);
+		MyMoneyTransactionFilter f1("A000001");
+		MyMoneyTransactionFilter f2("A000002");
+		MyMoneyTransactionFilter f3("A000003");
+		CPPUNIT_ASSERT(m->transactionList(f1).count() == 1);
+		CPPUNIT_ASSERT(m->transactionList(f2).count() == 0);
+		CPPUNIT_ASSERT(m->transactionList(f3).count() == 1);
 
 		m->modifyTransaction(t);
 		t = m->transaction("T000000000000000001");
@@ -934,9 +937,9 @@ void MyMoneyFileTest::testModifyTransactionNewAccount() {
 		CPPUNIT_ASSERT(m->account("A000002").transactionCount() == 1);
 		CPPUNIT_ASSERT(m->account("A000003").transactionCount() == 1);
 */
-		CPPUNIT_ASSERT(m->transactionList("A000001").count() == 0);
-		CPPUNIT_ASSERT(m->transactionList("A000002").count() == 1);
-		CPPUNIT_ASSERT(m->transactionList("A000003").count() == 1);
+		CPPUNIT_ASSERT(m->transactionList(f1).count() == 0);
+		CPPUNIT_ASSERT(m->transactionList(f2).count() == 1);
+		CPPUNIT_ASSERT(m->transactionList(f3).count() == 1);
 
 		CPPUNIT_ASSERT(observer->updated().count() == 3);
 		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
@@ -965,9 +968,12 @@ void MyMoneyFileTest::testRemoveTransaction () {
 		CPPUNIT_ASSERT(m->account("A000002").transactionCount() == 0);
 		CPPUNIT_ASSERT(m->account("A000003").transactionCount() == 0);
 */
-		CPPUNIT_ASSERT(m->transactionList("A000001").count() == 0);
-		CPPUNIT_ASSERT(m->transactionList("A000002").count() == 0);
-		CPPUNIT_ASSERT(m->transactionList("A000003").count() == 0);
+		MyMoneyTransactionFilter f1("A000001");
+		MyMoneyTransactionFilter f2("A000002");
+		MyMoneyTransactionFilter f3("A000003");
+		CPPUNIT_ASSERT(m->transactionList(f1).count() == 0);
+		CPPUNIT_ASSERT(m->transactionList(f2).count() == 0);
+		CPPUNIT_ASSERT(m->transactionList(f3).count() == 0);
 
 		CPPUNIT_ASSERT(observer->updated().count() == 2);
 		CPPUNIT_ASSERT(observer->updated().contains(MyMoneyFile::NotifyClassAccount) == 1);
@@ -1278,8 +1284,21 @@ void MyMoneyFileTest::testCategory2Account() {
 		m->reparentAccount(b, a);
 		CPPUNIT_ASSERT(m->categoryToAccount("Expense1") == "A000003");
 		CPPUNIT_ASSERT(m->categoryToAccount("Expense1:Expense2") == "A000004");
-		CPPUNIT_ASSERT(m->categoryToAccount("Acc2") == "");
+		CPPUNIT_ASSERT(m->categoryToAccount("Acc2").isEmpty());
 	} catch(MyMoneyException *e) {
 		unexpectedException(e);
 	}
 }
+
+void MyMoneyFileTest::testAttachedStorage() {
+	CPPUNIT_ASSERT(m->storageAttached() == true);
+	CPPUNIT_ASSERT(m->storage() != 0);
+	IMyMoneyStorage *p = m->storage();
+	m->detachStorage(p);
+	CPPUNIT_ASSERT(m->storageAttached() == false);
+	CPPUNIT_ASSERT(m->storage() == 0);
+	m->attachStorage(p);
+	CPPUNIT_ASSERT(m->storageAttached() == true);
+	CPPUNIT_ASSERT(m->storage() != 0);
+}
+
