@@ -18,12 +18,26 @@
 
 #include <stdlib.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifndef HAVE_ATOLL
 #  ifdef HAVE_STRTOLL
 #    define atoll(a) strtoll(a, 0, 10)
 #  endif
 #endif
 
+#include <cmath>
+
+#ifdef _GLIBCPP_HAVE_MODFL
+#define HAVE_LONG_DOUBLE  1
+#endif
+
+#ifndef HAVE_LONG_DOUBLE
+#define HAVE_LONG_DOUBLE  0
+#endif
+
 // So we can save this object
 #include <qstring.h>
 #include <qdatastream.h>
@@ -51,13 +65,19 @@ public:
   MyMoneyMoney( long ldAmountInPence );
   MyMoneyMoney( int iAmountInPence );
   MyMoneyMoney( double dAmountInPence );
+#if HAVE_LONG_DOUBLE
+  MyMoneyMoney( long double dAmountInPence );
+#endif
 
   // copy constructor
   MyMoneyMoney( const MyMoneyMoney& AmountInPence );
 
   signed64 value(void) const { return m_64Value; };
+  const MyMoneyMoney abs(void) const { return *this < 0 ? -(*this) : *this; };
   const QString formatMoney(/*QString locale="C", bool addPrefixPostfix=false*/void) const;
-
+  const QString toString(void) const;
+  void fromString(const QString& str);
+  
   static void setThousandSeparator(const unsigned char);
   static void setDecimalSeparator(const unsigned char);
   static unsigned char thousandSeparator(void);
@@ -145,6 +165,9 @@ public:
   MyMoneyMoney& operator--();
   MyMoneyMoney  operator--( int );
 
+  static signed64 maxValue;
+  static signed64 minValue;
+  
 private:
   signed64 m_64Value;
 
@@ -219,6 +242,22 @@ inline MyMoneyMoney::MyMoneyMoney(const double dAmountInPence)
   m_64Value = static_cast<signed64> (dAmountInPence * 100 + adj);
 }
 
+#if HAVE_LONG_DOUBLE
+////////////////////////////////////////////////////////////////////////////////
+//      Name: MyMoneyMoney
+//   Purpose: Constructor - constructs object from an amount in a long double value
+//   Returns: None
+//    Throws: Nothing.
+// Arguments: dAmountInPence - long double object containing amount in pence
+//
+////////////////////////////////////////////////////////////////////////////////
+inline MyMoneyMoney::MyMoneyMoney(const long double dAmountInPence)
+{
+  long double adj = dAmountInPence < 0 ? -0.5 : 0.5;
+  m_64Value = static_cast<signed64> (dAmountInPence * 100 + adj);
+}
+#endif
+
 ////////////////////////////////////////////////////////////////////////////////
 //      Name: MyMoneyMoney
 //   Purpose: Constructor - constructs object from an amount in a long value
