@@ -14,7 +14,11 @@
  *                                                                         *
  ***************************************************************************/
 
+#define __STDC_LIMIT_MACROS         // force definition of min and max values 
+
+#include <iostream>
 #include "mymoneymoneytest.h"
+#include <stdint.h>
 
 MyMoneyMoneyTest::MyMoneyMoneyTest()
 {
@@ -101,6 +105,9 @@ void MyMoneyMoneyTest::testStringConstructor()
 	CPPUNIT_ASSERT(m1->value() == 12345LL);
 	delete m1;
 
+	m1 = new MyMoneyMoney("12345/100");
+	CPPUNIT_ASSERT(m1->value() == 12345LL);
+	delete m1;
 }
 
 void MyMoneyMoneyTest::testEquality()
@@ -221,6 +228,12 @@ void MyMoneyMoneyTest::testFormatMoney()
 
 	m1 = 100000;
 	CPPUNIT_ASSERT(m1.formatMoney() == QString("1,000.00"));
+
+	m1 = INT64_MAX;
+	CPPUNIT_ASSERT(m1.formatMoney() == QString("92,233,720,368,547,758.07"));
+
+	m1 = INT64_MIN;
+	CPPUNIT_ASSERT(m1.formatMoney() == QString("-92,233,720,368,547,758.08"));
 }
 
 void MyMoneyMoneyTest::testRelation()
@@ -257,3 +270,37 @@ void MyMoneyMoneyTest::testDoubleConstructor()
 		CPPUNIT_ASSERT(t == r);
 	}
 }
+
+void MyMoneyMoneyTest::testAbsoluteFunction()
+{
+	MyMoneyMoney m1(-100);
+	MyMoneyMoney m2(100);
+
+	CPPUNIT_ASSERT(m2.abs() == 100);
+	CPPUNIT_ASSERT(m1.abs() == 100);
+}
+
+void MyMoneyMoneyTest::testToString()
+{
+	MyMoneyMoney m1(-100);
+	MyMoneyMoney m2(1234);
+	MyMoneyMoney m3;
+
+	CPPUNIT_ASSERT(m1.toString() == QString("-100/100"));
+	CPPUNIT_ASSERT(m2.toString() == QString("1234/100"));
+	CPPUNIT_ASSERT(m3.toString() == QString("0/100"));
+}
+
+void MyMoneyMoneyTest::testFromString()
+{
+	MyMoneyMoney m;
+
+	m.fromString("-100/100");
+	CPPUNIT_ASSERT(m.value() == -100LL);
+	m.fromString("1234/100");
+	CPPUNIT_ASSERT(m.value() == 1234LL);
+	m.fromString("1234/10");
+	CPPUNIT_ASSERT(m.value() == 0LL);
+	
+}
+
