@@ -36,12 +36,12 @@
 #include "../mymoneyaccount.h"
 #include "../mymoneytransaction.h"
 #include "../mymoneypayee.h"
+#include "../mymoneyscheduled.h"
 #include "../mymoneyobserver.h"
-
-class MyMoneyTransactionFilter;
+#include "../mymoneytransactionfilter.h"
 
 /**
-  *@author Thomas Baumgart
+  * @author Thomas Baumgart
   */
 
 /**
@@ -52,8 +52,8 @@ class MyMoneyTransactionFilter;
   */
 class IMyMoneyStorage {
 public: 
-	IMyMoneyStorage();
-	virtual ~IMyMoneyStorage();
+  IMyMoneyStorage();
+  virtual ~IMyMoneyStorage();
 
   // general get functions
   virtual const QString& userName(void) const = 0;
@@ -80,6 +80,13 @@ public:
   virtual const QString value(const QCString& key) const = 0;
   virtual void deletePair(const QCString& key) = 0;
 
+  /**
+    * This method is used to duplicate an IMyMoneyStorage object and return
+    * a pointer to the newly created copy. The caller of this method is the
+    * new owner of the object and must destroy it.
+    */
+  virtual IMyMoneyStorage* const duplicate(void) = 0;
+
   /**
     * This method is used to create a new account
     *
@@ -166,7 +173,7 @@ public:
     *
     * An exception will be thrown upon error conditions.
     *
-    * @param institution MyMoneyInsitution the account should be added to
+    * @param institution MyMoneyInstitution the account should be added to
     * @param account MyMoneyAccount to be added
     */
   virtual void addAccount(MyMoneyInstitution& institution, MyMoneyAccount& account) = 0;
@@ -276,6 +283,14 @@ public:
     */
   virtual bool dirty(void) const = 0;
 
+  /**
+    * This method can be used by an external object to force the
+    * storage object to be dirty. This is used e.g. when an upload
+    * to an external destination failed but the previous storage
+    * to a local disk was ok.
+    */
+  virtual void setDirty(void) = 0;
+  
   /**
     * This method returns the number of accounts currently known to this storage
     * in the range 0..MAXUINT
@@ -299,8 +314,9 @@ public:
     * An exception will be thrown upon error conditions.
     *
     * @param account reference to the new account information
+    * @param skipCheck allows to skip the builtin consistency checks
     */
-  virtual void modifyAccount(const MyMoneyAccount& account) = 0;
+  virtual void modifyAccount(const MyMoneyAccount& account, const bool skipCheck = false) = 0;
 
   /**
     * Modifies an already existing institution in the file global
@@ -346,14 +362,37 @@ public:
     * This method returns the number of transactions currently known to file
     * in the range 0..MAXUINT
     *
-    * @param account QCString reference to account id. If account equals ""
+    * @param account QCString reference to account id. If account is empty
     +                all transactions (the journal) will be counted. If account
-    *                is not equal to "" it returns the number of transactions
+    *                is not empty it returns the number of transactions
     *                that have splits in this account.
     *
     * @return number of transactions in journal/account
     */
-  virtual const unsigned int transactionCount(const QCString& account = "") const = 0;
+  virtual const unsigned int transactionCount(const QCString& account = QCString()) const = 0;
+
+  /**
+    * This method returns a QMap filled with the number of transactions
+    * per account. The account id serves as index into the map. If one
+    * needs to have all transactionCounts() for many accounts, this method
+    * is faster than calling transactionCount(const QCString& account) many
+    * times.
+    *
+    * @return QMap with numbers of transactions per account
+    */
+  virtual const QMap<QCString, unsigned long> transactionCountMap(void) const = 0;
+
+  /**
+    * This method is used to pull a list of transactions from the file
+    * global transaction pool. It returns all those transactions
+    * that match the filter passed as argument. If the filter is empty,
+    * the whole journal will be returned.
+    *
+    * @param filter MyMoneyTransactionFilter object with the match criteria
+    *
+    * @return set of transactions in form of a QValueList<MyMoneyTransaction>
+    */
+  virtual const QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const = 0;
 
   /**
     * This method is used to pull a list of transactions from the file
@@ -361,14 +400,14 @@ public:
     * the set of transaction referenced by a specific account depending
     * on the argument given.
     *
-    * @param account QCString reference to account id. If account equals "" all
+    * @param account QCString reference to account id. If account is empty all
     +                transactions (the journal) is returned. If account
-    *                is not equal to "" it returns the set of transactions
+    *                is not empty it returns the set of transactions
     *                that have splits in this account.
     *
     * @return set of transactions in form of a QValueList<MyMoneyTransaction>
     */
-  virtual const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const = 0;
+  // virtual const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const = 0;
 
   /**
     * Deletes an existing account from the file global account pool
@@ -451,6 +490,90 @@ public:
     * @return MyMoneyAccount income account(group)
     */
   virtual const MyMoneyAccount income(void) const = 0;
+
+  /**
+    * This method is used to add a scheduled transaction to the engine.
+    * It must be sure, that the id of the object is not filled. When the
+    * method returns to the caller, the id will be filled with the
+    * newly created object id value.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param sched reference to the MyMoneySchedule object
+    */
+  virtual void addSchedule(MyMoneySchedule& sched) = 0;
+
+  /**
+    * This method is used to modify an existing MyMoneySchedule
+    * object. Therefor, the id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param sched const reference to the MyMoneySchedule object to be updated
+    */
+  virtual void modifySchedule(const MyMoneySchedule& sched) = 0;
+
+  /**
+    * This method is used to remove an existing MyMoneySchedule object
+    * from the engine. The id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param sched const reference to the MyMoneySchedule object to be updated
+    */
+  virtual void removeSchedule(const MyMoneySchedule& sched) = 0;
+
+  /**
+    * This method is used to retrieve a single MyMoneySchedule object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneySchedule object
+    * @return MyMoneySchedule object
+    */
+  virtual const MyMoneySchedule schedule(const QCString& id) const = 0;
+
+  /**
+    * This method is used to extract a list of scheduled transactions
+    * according to the filter criteria passed as arguments.
+    *
+    * @param accountId only search for scheduled transactions that reference
+    *                  accound @p accountId. If accountId is the empty string,
+    *                  this filter is off. Default is @p QCString().
+    * @param type      only schedules of type @p type are searched for.
+    *                  See MyMoneySchedule::typeE for details.
+    *                  Default is MyMoneySchedule::TYPE_ANY
+    * @param occurance only schedules of occurance type @p occurance are searched for.
+    *                  See MyMoneySchedule::occuranceE for details.
+    *                  Default is MyMoneySchedule::OCCUR_ANY
+    * @param paymentType only schedules of payment method @p paymentType
+    *                  are searched for.
+    *                  See MyMoneySchedule::paymentTypeE for details.
+    *                  Default is MyMoneySchedule::STYPE_ANY
+    * @param startDate only schedules with payment dates after @p startDate
+    *                  are searched for. Default is all dates (QDate()).
+    * @param endDate   only schedules with payment dates ending prior to @p endDate
+    *                  are searched for. Default is all dates (QDate()).
+    * @param overdue   if true, only those schedules that are overdue are
+    *                  searched for. Default is false (all schedules will be returned).
+    *
+    * @return const QValueList<MyMoneySchedule> list of schedule objects.
+    */
+  virtual const QValueList<MyMoneySchedule> scheduleList(const QCString& = QCString(),
+                                     const MyMoneySchedule::typeE = MyMoneySchedule::TYPE_ANY,
+                                     const MyMoneySchedule::occurenceE = MyMoneySchedule::OCCUR_ANY,
+                                     const MyMoneySchedule::paymentTypeE = MyMoneySchedule::STYPE_ANY,
+                                     const QDate& = QDate(),
+                                     const QDate& = QDate(),
+                                     const bool = false) const = 0;
+
+  virtual QValueList<MyMoneySchedule> scheduleListEx( int scheduleTypes,
+                                              int scheduleOcurrences,
+                                              int schedulePaymentTypes,
+                                              QDate startDate,
+                                              const QCStringList& accounts=QCStringList()) const = 0;
+
 };
 
 #endif
