@@ -21,6 +21,13 @@
  ***************************************************************************/
 
 #include "mymoneyseqaccessmgr.h"
+#include "../mymoneytransactionfilter.h"
+
+const bool MyMoneyBalanceCacheItem::operator ==(const MyMoneyBalanceCacheItem & right) const
+{
+  return ((balance == right.balance)
+        && (valid == right.valid));
+}
 
 MyMoneySeqAccessMgr::MyMoneySeqAccessMgr()
 {
@@ -28,13 +35,14 @@ MyMoneySeqAccessMgr::MyMoneySeqAccessMgr()
   m_nextInstitutionID = 0;
   m_nextTransactionID = 0;
   m_nextPayeeID = 0;
+  m_nextScheduleID = 0;
   m_userName =
   m_userStreet =
   m_userTown =
   m_userCounty =
   m_userPostcode =
   m_userTelephone =
-  m_userEmail = "";
+  m_userEmail = QString();
   m_dirty = false;
   m_institutionList.clear();
   m_accountList.clear();
@@ -43,16 +51,16 @@ MyMoneySeqAccessMgr::MyMoneySeqAccessMgr()
   // setup standard accounts
   MyMoneyAccount acc_l;
   acc_l.setAccountType(MyMoneyAccount::Liability);
-	acc_l.setName("Liability");
+  acc_l.setName("Liability");
   MyMoneyAccount acc_a;
   acc_a.setAccountType(MyMoneyAccount::Asset);
-	acc_a.setName("Asset");
+  acc_a.setName("Asset");
   MyMoneyAccount acc_e;
   acc_e.setAccountType(MyMoneyAccount::Expense);
-	acc_e.setName("Expense");
+  acc_e.setName("Expense");
   MyMoneyAccount acc_i;
   acc_i.setAccountType(MyMoneyAccount::Income);
-	acc_i.setName("Income");
+  acc_i.setName("Income");
 
   MyMoneyAccount* a;
   a = new MyMoneyAccount(STD_ACC_LIABILITY, acc_l);
@@ -83,6 +91,13 @@ MyMoneySeqAccessMgr::~MyMoneySeqAccessMgr()
 {
 }
 
+MyMoneySeqAccessMgr* const MyMoneySeqAccessMgr::duplicate(void)
+{
+  MyMoneySeqAccessMgr* that = new MyMoneySeqAccessMgr();
+  *that = *this;
+  return that;
+}
+
 const bool MyMoneySeqAccessMgr::isStandardAccount(const QCString& id) const
 {
   return id == STD_ACC_LIABILITY
@@ -156,6 +171,9 @@ const MyMoneyPayee MyMoneySeqAccessMgr::payee(const QCString& id) const
 
 const MyMoneyPayee MyMoneySeqAccessMgr::payeeByName(const QString& payee) const
 {
+  if(payee.isEmpty())
+    return MyMoneyPayee();
+    
   QMap<QCString, MyMoneyPayee>::ConstIterator it_p;
 
   for(it_p = m_payeeList.begin(); it_p != m_payeeList.end(); ++it_p) {
@@ -319,6 +337,22 @@ const unsigned int MyMoneySeqAccessMgr::transactionCount(const QCString& account
   return cnt;
 }
 
+const QMap<QCString, unsigned long> MyMoneySeqAccessMgr::transactionCountMap(void) const
+{
+  QMap<QCString, unsigned long> map;
+  QMap<QCString, MyMoneyTransaction>::ConstIterator it_t;
+  QValueList<MyMoneySplit>::ConstIterator it_s;
+
+  // scan all transactions
+  for(it_t = m_transactionList.begin(); it_t != m_transactionList.end(); ++it_t) {
+    // scan all splits of this transaction
+    for(it_s = (*it_t).splits().begin(); it_s != (*it_t).splits().end(); ++it_s) {
+      map[(*it_s).accountId()]++;
+    }
+  }
+  return map;
+}
+
 const unsigned int MyMoneySeqAccessMgr::institutionCount(void) const
 {
   return m_institutionList.count();
@@ -361,6 +395,14 @@ const QCString MyMoneySeqAccessMgr::nextTransactionID(void)
   return id;
 }
 
+const QCString MyMoneySeqAccessMgr::nextScheduleID(void)
+{
+  QCString id;
+  id.setNum(++m_nextScheduleID);
+  id = "SCH" + id.rightJustify(SCHEDULE_ID_SIZE, '0');
+  return id;
+}
+
 void MyMoneySeqAccessMgr::addTransaction(MyMoneyTransaction& transaction, const bool skipAccountUpdate)
 {
   // perform some checks to see that the transaction stuff is OK. For
@@ -370,9 +412,10 @@ void MyMoneySeqAccessMgr::addTransaction(MyMoneyTransaction& transaction, const
   // * the referenced accounts in the splits exist
 
   // first perform all the checks
-  if(transaction.id() != ""
-  || !transaction.postDate().isValid())
-    throw new MYMONEYEXCEPTION("invalid transaction to be added");
+  if(!transaction.id().isEmpty())
+    throw new MYMONEYEXCEPTION("transaction already contains an id");
+  if(!transaction.postDate().isValid())
+    throw new MYMONEYEXCEPTION("invalid post date");
 
   // now check the splits
   QValueList<MyMoneySplit>::ConstIterator it_s;
@@ -380,7 +423,7 @@ void MyMoneySeqAccessMgr::addTransaction(MyMoneyTransaction& transaction, const
     // the following lines will throw an exception if the
     // account or payee do not exist
     account((*it_s).accountId());
-    if((*it_s).payeeId() != "")
+    if(!(*it_s).payeeId().isEmpty())
       payee((*it_s).payeeId());
   }
 
@@ -404,7 +447,6 @@ void MyMoneySeqAccessMgr::addTransaction(MyMoneyTransaction& transaction, const
       acc = m_accountList.find((*it).accountId());
       if(acc != m_accountList.end()) {
         (*acc).touch();
-        refreshAccountTransactionList(acc);
         invalidateBalanceCache((*acc).id());
       }
     }
@@ -428,50 +470,6 @@ void MyMoneySeqAccessMgr::touch(void)
   m_lastModificationDate = QDate::currentDate();
 }
 
-void MyMoneySeqAccessMgr::refreshAllAccountTransactionLists(void)
-{
-/* removed with MyMoneyAccount::Transaction
-  QMap<QCString, MyMoneyTransaction>::ConstIterator it_t;
-  QMap<QCString, MyMoneyAccount>::Iterator it_a;
-  QValueList<MyMoneySplit>::ConstIterator it_s;
-
-  for(it_a = m_accountList.begin(); it_a != m_accountList.end(); ++it_a)
-    (*it_a).clearTransactions();
-
-  // scan all transactions
-  for(it_t = m_transactionList.begin(); it_t != m_transactionList.end(); ++it_t) {
-    // scan all splits of this transaction
-    for(it_s = (*it_t).splits().begin(); it_s != (*it_t).splits().end(); ++it_s) {
-      MyMoneyAccount::Transaction val((*it_t).id() ,(*it_s).value());
-      it_a = m_accountList.find((*it_s).accountId());
-      (*it_a).addTransaction(val);
-    }
-  }
-*/
-}
-
-void MyMoneySeqAccessMgr::refreshAccountTransactionList(QMap<QCString, MyMoneyAccount>::Iterator acc) const
-{
-/* removed with MyMoneyAccount::Transaction
-  QMap<QCString, MyMoneyTransaction>::ConstIterator it_t;
-  QValueList<MyMoneySplit>::ConstIterator it_s;
-
-  (*acc).clearTransactions();
-
-  // scan all transactions
-  for(it_t = m_transactionList.begin(); it_t != m_transactionList.end(); ++it_t) {
-    // scan all splits of this transaction
-    for(it_s = (*it_t).splits().begin(); it_s != (*it_t).splits().end(); ++it_s) {
-      // is it a split in our account?
-      if((*it_s).accountId() == (*acc).id()) {
-        MyMoneyAccount::Transaction val((*it_t).id() ,(*it_s).value());
-        (*acc).addTransaction(val);
-      }
-    }
-  }
-*/
-}
-
 const bool MyMoneySeqAccessMgr::hasActiveSplits(const QCString& id) const
 {
   QMap<QCString, MyMoneyTransaction>::ConstIterator it;
@@ -505,7 +503,7 @@ const QValueList<MyMoneyInstitution> MyMoneySeqAccessMgr::institutionList(void)
   return list;
 }
 
-void MyMoneySeqAccessMgr::modifyAccount(const MyMoneyAccount& account)
+void MyMoneySeqAccessMgr::modifyAccount(const MyMoneyAccount& account, const bool skipCheck)
 {
   QMap<QCString, MyMoneyAccount>::Iterator pos;
 
@@ -516,11 +514,12 @@ void MyMoneySeqAccessMgr::modifyAccount(const MyMoneyAccount& account)
     // this is the case, when the file and the id
     // as well as the type are equal.
     if((*pos).parentAccountId() == account.parentAccountId()
-    && (*pos).accountType() == account.accountType()) {
+    && (*pos).accountType() == account.accountType()
+    || skipCheck == true) {
       // if it points to a different institution, then update both
       if((*pos).institutionId() != account.institutionId()) {
         // check if new institution exists
-        if(account.institutionId() != "")
+        if(!account.institutionId().isEmpty())
           institution(account.institutionId());
 
         QMap<QCString, MyMoneyInstitution>::Iterator oldInst, newInst;
@@ -578,7 +577,7 @@ void MyMoneySeqAccessMgr::modifyTransaction(const MyMoneyTransaction& transactio
   // * the splits must have valid account ids
 
   // first perform all the checks
-  if(transaction.id() == ""
+  if(transaction.id().isEmpty()
 //  || transaction.file() != this
   || !transaction.postDate().isValid())
     throw new MYMONEYEXCEPTION("invalid transaction to be modified");
@@ -589,7 +588,7 @@ void MyMoneySeqAccessMgr::modifyTransaction(const MyMoneyTransaction& transactio
     // the following lines will throw an exception if the
     // account or payee do not exist
     account((*it_s).accountId());
-    if((*it_s).payeeId() != "")
+    if(!(*it_s).payeeId().isEmpty())
       payee((*it_s).payeeId());
   }
 
@@ -635,7 +634,6 @@ void MyMoneySeqAccessMgr::modifyTransaction(const MyMoneyTransaction& transactio
     acc = m_accountList.find(it_a.key());
     if(acc != m_accountList.end()) {
       (*acc).touch();
-      refreshAccountTransactionList(acc);
       invalidateBalanceCache((*acc).id());
     }
   }
@@ -646,7 +644,7 @@ void MyMoneySeqAccessMgr::reparentAccount(MyMoneyAccount &account, MyMoneyAccoun
   reparentAccount(account, parent, true);
 }
 
-void MyMoneySeqAccessMgr::reparentAccount(MyMoneyAccount &account, MyMoneyAccount& parent, const bool sendNotification)
+void MyMoneySeqAccessMgr::reparentAccount(MyMoneyAccount &account, MyMoneyAccount& parent, const bool /* sendNotification */)
 {
   QMap<QCString, MyMoneyAccount>::Iterator oldParent;
   QMap<QCString, MyMoneyAccount>::Iterator newParent;
@@ -670,6 +668,9 @@ void MyMoneySeqAccessMgr::reparentAccount(MyMoneyAccount &account, MyMoneyAccoun
   (*newParent).addAccountId(account.id());
   (*childAccount).setParentAccountId(parent.id());
 
+  // make sure the type is the same as the new parent
+  (*childAccount).setAccountType((*newParent).accountType());
+  
   parent = *newParent;
   account = *childAccount;
 
@@ -682,7 +683,7 @@ void MyMoneySeqAccessMgr::removeTransaction(const MyMoneyTransaction& transactio
   QMap<QCString, bool> modifiedAccounts;
 
   // first perform all the checks
-  if(transaction.id() == "")
+  if(transaction.id().isEmpty())
     throw new MYMONEYEXCEPTION("invalid transaction to be deleted");
 
   QMap<QCString, QCString>::Iterator it_k;
@@ -715,7 +716,6 @@ void MyMoneySeqAccessMgr::removeTransaction(const MyMoneyTransaction& transactio
     acc = m_accountList.find(it_a.key());
     if(acc != m_accountList.end()) {
       (*acc).touch();
-      refreshAccountTransactionList(acc);
       invalidateBalanceCache((*acc).id());
     }
   }
@@ -816,7 +816,7 @@ void MyMoneySeqAccessMgr::removeInstitution(const MyMoneyInstitution& institutio
       QCStringList::Iterator it_a;
       for(it_a = accounts.begin(); it_a != accounts.end(); ++it_a) {
         MyMoneyAccount acc = account(*it_a);
-        acc.setInstitutionId("");
+        acc.setInstitutionId(QCString());
         try {
           modifyAccount(acc);
         } catch(MyMoneyException *e) {
@@ -833,34 +833,30 @@ void MyMoneySeqAccessMgr::removeInstitution(const MyMoneyInstitution& institutio
     throw new MYMONEYEXCEPTION("invalid institution");
 }
 
-const QValueList<MyMoneyTransaction> MyMoneySeqAccessMgr::transactionList(const QCString& account) const
+const QValueList<MyMoneyTransaction> MyMoneySeqAccessMgr::transactionList(MyMoneyTransactionFilter& filter) const
 {
   QValueList<MyMoneyTransaction> list;
   QMap<QCString, MyMoneyTransaction>::ConstIterator it_t;
-  QValueList<MyMoneySplit>::ConstIterator it_s;
 
-  // scan all transactions
   for(it_t = m_transactionList.begin(); it_t != m_transactionList.end(); ++it_t) {
-    bool append = true;
-
-    if(account.length() != 0) {
-      // scan all splits of this transaction
-      for(it_s = (*it_t).splits().begin(); it_s != (*it_t).splits().end(); ++it_s) {
-        // is it a split in our account?
-        if((*it_s).accountId() == account) {
-          // since a transaction can only have one split referencing
-          // each account, we're done with the splits here!
-          break;
-        }
-      }
-      // reset flag, if no split contains the account id
-      if(it_s == (*it_t).splits().end())
-        append = false;
-
-    }
-
-    if(append == true)
+#if 0
+    if(filter.match(*it_t, this))
       list.append(*it_t);
+#else
+    // This code is used now. It adds the transaction to the list for
+    // each matching split exactly once. This allows to show information
+    // about different splits in the same register view (e.g. search result)
+    //
+    // I have no idea, if this has some impact on the functionality. So far,
+    // I could not see it.  (ipwizard 9/5/2003)
+    if(filter.match(*it_t, this)) {
+      if(filter.matchingSplits().count() > 0) {
+        for(unsigned i=0; i < filter.matchingSplits().count(); ++i)
+          list.append(*it_t);
+      } else
+        list.append(*it_t);
+    }
+#endif
   }
   return list;
 }
@@ -899,7 +895,16 @@ const MyMoneyTransaction& MyMoneySeqAccessMgr::transaction(const QCString& accou
 
   // new implementation if the above code does not work anymore
   QValueList<MyMoneyTransaction> list;
-  list = transactionList(account);
+  MyMoneyAccount acc = m_accountList[account];
+  MyMoneyTransactionFilter filter;
+
+  if(acc.accountGroup() == MyMoneyAccount::Income
+  || acc.accountGroup() == MyMoneyAccount::Expense)
+    filter.addCategory(account);
+  else
+    filter.addAccount(account);
+    
+  list = transactionList(filter);
   if(idx < 0 || idx >= static_cast<int> (list.count()))
     throw new MYMONEYEXCEPTION("Unknown idx for transaction");
 
@@ -927,11 +932,12 @@ const MyMoneyMoney MyMoneySeqAccessMgr::balance(const QCString& id)
     QValueList<MyMoneyTransaction> list;
     QValueList<MyMoneyTransaction>::ConstIterator it;
     MyMoneySplit split;
-    list = transactionList(id);
+    MyMoneyTransactionFilter filter(id);
+    list = transactionList(filter);
 
     for(it = list.begin(); it != list.end(); ++it) {
       try {
-        split = (*it).split(id);
+        split = (*it).splitByAccount(id);
         result += split.value();
 
       } catch(MyMoneyException *e) {
@@ -1082,7 +1088,7 @@ void MyMoneySeqAccessMgr::loadInstitution(const MyMoneyInstitution& inst)
   it = m_institutionList.find(inst.id());
   if(it != m_institutionList.end()) {
     QString msg = "Duplicate institution  '";
-    msg += inst.id() + "' during loadInsitution()";
+    msg += inst.id() + "' during loadInstitution()";
     throw new MYMONEYEXCEPTION(msg);
   }
   m_institutionList[inst.id()] = inst;
@@ -1145,3 +1151,213 @@ void MyMoneySeqAccessMgr::setPairs(const QMap<QCString, QString>& list)
 {
   MyMoneyKeyValueContainer::setPairs(list);
 }
+
+void MyMoneySeqAccessMgr::addSchedule(MyMoneySchedule& sched)
+{
+  // first perform all the checks
+  if(!sched.id().isEmpty())
+    throw new MYMONEYEXCEPTION("schedule already contains an id");
+
+  // The following will throw an exception when it fails
+  sched.validate(false);
+
+  sched.setId(nextScheduleID());
+
+  m_scheduleList[sched.id()] = sched;
+
+  // mark file as changed
+  touch();
+}
+
+void MyMoneySeqAccessMgr::modifySchedule(const MyMoneySchedule& sched)
+{
+  QMap<QCString, MyMoneySchedule>::Iterator it;
+
+  it = m_scheduleList.find(sched.id());
+  if(it == m_scheduleList.end()) {
+    QString msg = "Unknown schedule '" + sched.id() + "'";
+    throw new MYMONEYEXCEPTION(msg);
+  }
+
+  *it = sched;
+  // mark file as changed
+  touch();
+}
+
+void MyMoneySeqAccessMgr::removeSchedule(const MyMoneySchedule& sched)
+{
+  QMap<QCString, MyMoneySchedule>::Iterator it;
+
+  it = m_scheduleList.find(sched.id());
+  if(it == m_scheduleList.end()) {
+    QString msg = "Unknown schedule '" + sched.id() + "'";
+    throw new MYMONEYEXCEPTION(msg);
+  }
+
+  m_scheduleList.remove(it);
+  touch();
+}
+
+const MyMoneySchedule MyMoneySeqAccessMgr::schedule(const QCString& id) const
+{
+  QMap<QCString, MyMoneySchedule>::ConstIterator pos;
+
+  // locate the schedule and if present, return it's data
+  pos = m_scheduleList.find(id);
+  if(pos != m_scheduleList.end())
+    return (*pos);
+
+  // throw an exception, if it does not exist
+  QString msg = "Unknown schedule id '" + id + "'";
+  throw new MYMONEYEXCEPTION(msg);
+}
+
+const QValueList<MyMoneySchedule> MyMoneySeqAccessMgr::scheduleList(
+                          const QCString& accountId,
+                          const MyMoneySchedule::typeE type,
+                          const MyMoneySchedule::occurenceE occurence,
+                          const MyMoneySchedule::paymentTypeE paymentType,
+                          const QDate& startDate,
+                          const QDate& endDate,
+                          const bool overdue) const
+{
+  QMap<QCString, MyMoneySchedule>::ConstIterator pos;
+  QValueList<MyMoneySchedule> list;
+
+  // qDebug("scheduleList()");
+  
+  for(pos = m_scheduleList.begin(); pos != m_scheduleList.end(); ++pos) {
+    // qDebug("  '%s'", (*pos).id().data());
+    
+    if(type != MyMoneySchedule::TYPE_ANY) {
+      if(type != (*pos).type()) {
+        continue;
+      }
+    }
+    
+    if(occurence != MyMoneySchedule::OCCUR_ANY) {
+      if(occurence != (*pos).occurence()) {
+        continue;
+      }
+    }
+    
+    if(paymentType != MyMoneySchedule::STYPE_ANY) {
+      if(paymentType != (*pos).paymentType()) {
+        continue;
+      }
+    }
+    
+    if(!accountId.isEmpty()) {
+      MyMoneyTransaction t = (*pos).transaction();
+      QValueList<MyMoneySplit>::ConstIterator it;
+      QValueList<MyMoneySplit> splits;
+      splits = t.splits();
+      for(it = splits.begin(); it != splits.end(); ++it) {
+        if((*it).accountId() == accountId)
+          break;
+      }
+      if(it == splits.end()) {
+        continue;
+      }
+    }
+
+    if(startDate.isValid() && endDate.isValid()) {
+      if((*pos).paymentDates(startDate, endDate).count() == 0) {
+        continue;
+      }
+    }
+    
+    if(startDate.isValid() && !endDate.isValid()) {
+      if(!(*pos).nextPayment(startDate.addDays(-1)).isValid()) {
+        continue;
+      }
+    }
+    
+    if(!startDate.isValid() && endDate.isValid()) {
+      if((*pos).startDate() > endDate) {
+        continue;
+      }
+    }
+
+    if(overdue) {
+      if (!(*pos).isOverdue())
+        continue;
+/*
+      QDate nextPayment = (*pos).nextPayment((*pos).lastPayment());
+      if(!nextPayment.isValid())
+        continue;
+      if(nextPayment >= QDate::currentDate())
+        continue;
+*/
+    }
+
+    // qDebug("Adding '%s'", (*pos).name().latin1());
+    list << *pos;
+  }
+  return list;
+}
+
+void MyMoneySeqAccessMgr::loadSchedule(const MyMoneySchedule& sched)
+{
+  QMap<QCString, MyMoneySchedule>::ConstIterator it;
+
+  it = m_scheduleList.find(sched.id());
+  if(it != m_scheduleList.end()) {
+    QString msg = "Duplicate scheduled '";
+    msg += sched.id() + "' during loadSchedule()";
+    throw new MYMONEYEXCEPTION(msg);
+  }
+  m_scheduleList[sched.id()] = sched;
+}
+
+void MyMoneySeqAccessMgr::loadScheduleId(const unsigned long id)
+{
+  m_nextScheduleID = id;
+}
+
+QValueList<MyMoneySchedule> MyMoneySeqAccessMgr::scheduleListEx(int scheduleTypes,
+                                                                int scheduleOcurrences,
+                                                                int schedulePaymentTypes,
+                                                                QDate date,
+                                                                const QCStringList& accounts) const
+{
+//  qDebug("scheduleListEx");
+  
+  QMap<QCString, MyMoneySchedule>::ConstIterator pos;
+  QValueList<MyMoneySchedule> list;
+
+  if (!date.isValid())
+    return list;
+
+  for(pos = m_scheduleList.begin(); pos != m_scheduleList.end(); ++pos)
+  {
+    if (scheduleTypes && !(scheduleTypes & (*pos).type()))
+      continue; 
+
+    if (scheduleOcurrences && !(scheduleOcurrences & (*pos).occurence()))
+      continue;
+      
+    if (schedulePaymentTypes && !(schedulePaymentTypes & (*pos).paymentType()))
+      continue;
+
+    if((*pos).paymentDates(date, date).count() == 0)
+      continue;
+
+    if ((*pos).isFinished())
+      continue;
+
+    if ((*pos).hasRecordedPayment(date))
+      continue;
+
+    if (accounts.count() > 0)
+    {
+      if (accounts.contains((*pos).account().id()))
+        continue;
+    }
+    
+//    qDebug("\tAdding '%s'", (*pos).name().latin1());
+    list << *pos;
+  }
+  
+  return list;  
+}
