@@ -36,8 +36,6 @@
   *@author Thomas Baumgart
   */
 
-class MyMoneyTransactionFilter;
-
 /**
   * This member represents an item in the balance cache. The balance cache
   * is used for fast processing of the balance of an account. Several
@@ -58,6 +56,7 @@ public:
   MyMoneyBalanceCacheItem() { valid = false; };
   MyMoneyBalanceCacheItem(const MyMoneyMoney& val) { balance = val; valid = true; };
 
+  const bool operator == (const MyMoneyBalanceCacheItem& right) const;
   bool          valid;
   MyMoneyMoney  balance;
 };
@@ -86,8 +85,8 @@ public:
 #define STD_ACC_EXPENSE   "AStd::Expense"
 #define STD_ACC_INCOME    "AStd::Income"
 
-	MyMoneySeqAccessMgr();
-	~MyMoneySeqAccessMgr();
+  MyMoneySeqAccessMgr();
+  ~MyMoneySeqAccessMgr();
 
   // general get functions
   const QString& userName(void) const { return m_userName; };
@@ -111,6 +110,13 @@ public:
   void setCreationDate(const QDate& val) { m_creationDate = val; touch(); };
   void setLastModificationDate(const QDate& val) { m_lastModificationDate = val; m_dirty = false; };
 
+  /**
+    * This method is used to duplicate the MyMoneySeqAccessMgr object and return
+    * a pointer to the newly created copy. The caller of this method is the
+    * new owner of the object and must destroy it.
+    */
+  MyMoneySeqAccessMgr * const duplicate(void);
+  
   /**
     * Returns the account addressed by it's id.
     *
@@ -231,7 +237,7 @@ public:
     *
     * An exception will be thrown upon error conditions.
     *
-    * @param institution MyMoneyInsitution the account should be added to
+    * @param institution MyMoneyInstitution the account should be added to
     * @param account MyMoneyAccount to be added
     */
   void addAccount(MyMoneyInstitution& institution, MyMoneyAccount& account);
@@ -266,8 +272,13 @@ public:
     * An exception will be thrown upon error conditions.
     *
     * @param account reference to the new account information
+    * @param skipCheck if @p true, skips the built in consistency check for
+    *                  the object to be updated. Do not set this parameter
+    *                  to true. This is only used for the MyMoneyFile::consistencyCheck()
+    *                  procedure to be able to reload accounts. The default
+    *                  setting of this parameter is @p false.
     */
-  void modifyAccount(const MyMoneyAccount& account);
+  void modifyAccount(const MyMoneyAccount& account, const bool skipCheck = false);
 
   /**
     * Modifies an already existing institution in the file global
@@ -391,6 +402,14 @@ public:
     */
   bool dirty(void) const { return m_dirty; }
 
+  /**
+    * This method can be used by an external object to force the
+    * storage object to be dirty. This is used e.g. when an upload
+    * to an external destination failed but the previous storage
+    * to a local disk was ok.
+    */
+  void setDirty(void) { m_dirty = true; };
+
   /**
     * This method returns a list of the institutions
     * inside a MyMoneyFile object
@@ -407,33 +426,50 @@ public:
     */
   const QValueList<MyMoneyAccount> accountList(void) const;
 
+  /**
+    * This method is used to pull a list of transactions from the file
+    * global transaction pool. It returns all those transactions
+    * that match the filter passed as argument. If the filter is empty,
+    * the whole journal will be returned.
+    *
+    * @param filter MyMoneyTransactionFilter object with the match criteria
+    *
+    * @return set of transactions in form of a QValueList<MyMoneyTransaction>
+    */
+  const QValueList<MyMoneyTransaction> transactionList(MyMoneyTransactionFilter& filter) const;
+    
   /**
     * This method is used to pull a list of transactions from the file
     * global transaction pool. It returns either the whole journal or
     * the set of transaction referenced by a specific account depending
     * on the argument given.
     *
-    * @param account QCString reference to account id. If account equals ""
+    * @param account QCString reference to account id. If account is empty
     +                all transactions (the journal) is returned. If account
-    *                is not equal to "" it returns the set of transactions
+    *                is not empty it returns the set of transactions
     *                that have splits in this account.
     *
     * @return set of transactions in form of a QValueList<MyMoneyTransaction>
+    *
+    * @note This function is provided for convenience and historic reasons. It
+    *       makes use of the above mentioned method.
     */
-  const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const;
+  // const QValueList<MyMoneyTransaction> transactionList(const QCString& account = "") const;
 
   /**
     * This method returns the number of transactions currently known to file
     * in the range 0..MAXUINT
     *
-    * @param account QCString reference to account id. If account equals ""
+    * @param account QCString reference to account id. If account is empty
     +                all transactions (the journal) will be counted. If account
-    *                is not equal to "" it returns the number of transactions
+    *                is not empty it returns the number of transactions
     *                that have splits in this account.
     *
     * @return number of transactions in journal/account
     */
-  const unsigned int transactionCount(const QCString& account = "") const;
+  const unsigned int transactionCount(const QCString& account = QCString()) const;
+
+  const QMap<QCString, unsigned long> transactionCountMap(void) const;
 
   /**
     * This method returns the number of institutions currently known to file
@@ -475,29 +511,24 @@ public:
     */
   const MyMoneyAccount income(void) const { return account(STD_ACC_INCOME); };
 
-  /**
-    * This method reconstructs the transaction list of all accounts
-    * in the m_accountList.
-    *
-    * @see refreshAccountTransactionList, addTransaction
-    */
-  void refreshAllAccountTransactionLists(void);
-
   virtual void loadAccount(const MyMoneyAccount& acc);
   virtual void loadTransaction(const MyMoneyTransaction& tr);
   virtual void loadInstitution(const MyMoneyInstitution& inst);
   virtual void loadPayee(const MyMoneyPayee& payee);
-
+  virtual void loadSchedule(const MyMoneySchedule& sched);
+  
   virtual void loadAccountId(const unsigned long id);
   virtual void loadTransactionId(const unsigned long id);
   virtual void loadPayeeId(const unsigned long id);
   virtual void loadInstitutionId(const unsigned long id);
-
+  virtual void loadScheduleId(const unsigned long id);
+  
   virtual const unsigned long accountId(void) { return m_nextAccountID; };
   virtual const unsigned long transactionId(void) { return m_nextTransactionID; };
   virtual const unsigned long payeeId(void) { return m_nextPayeeID; };
   virtual const unsigned long institutionId(void) { return m_nextInstitutionID; };
-
+  virtual const unsigned long scheduleId(void) { return m_nextScheduleID; };
+  
   /**
     * This method is used to extract a value from 
     * KeyValueContainer. For details see MyMoneyKeyValueContainer::value().
@@ -531,12 +562,96 @@ public:
   // documented in IMyMoneySerialize base class
   void setPairs(const QMap<QCString, QString>& list);
 
+  /**
+    * This method is used to add a scheduled transaction to the engine.
+    * It must be sure, that the id of the object is not filled. When the
+    * method returns to the caller, the id will be filled with the
+    * newly created object id value.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param sched reference to the MyMoneySchedule object
+    */
+  void addSchedule(MyMoneySchedule& sched);
+
+  /**
+    * This method is used to modify an existing MyMoneySchedule
+    * object. Therefor, the id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param sched const reference to the MyMoneySchedule object to be updated
+    */
+  void modifySchedule(const MyMoneySchedule& sched);
+
+  /**
+    * This method is used to remove an existing MyMoneySchedule object
+    * from the engine. The id attribute of the object must be set.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param sched const reference to the MyMoneySchedule object to be updated
+    */
+  void removeSchedule(const MyMoneySchedule& sched);
+
+  /**
+    * This method is used to retrieve a single MyMoneySchedule object.
+    * The id of the object must be supplied in the parameter @p id.
+    *
+    * An exception will be thrown upon erronous situations.
+    *
+    * @param id QCString containing the id of the MyMoneySchedule object
+    * @return MyMoneySchedule object
+    */
+  const MyMoneySchedule schedule(const QCString& id) const;
+
+  /**
+    * This method is used to extract a list of scheduled transactions
+    * according to the filter criteria passed as arguments.
+    *
+    * @param accountId only search for scheduled transactions that reference
+    *                  accound @p accountId. If accountId is the empty string,
+    *                  this filter is off. Default is @p QCString().
+    * @param type      only schedules of type @p type are searched for.
+    *                  See MyMoneySchedule::typeE for details.
+    *                  Default is MyMoneySchedule::TYPE_ANY
+    * @param occurance only schedules of occurance type @p occurance are searched for.
+    *                  See MyMoneySchedule::occuranceE for details.
+    *                  Default is MyMoneySchedule::OCCUR_ANY
+    * @param paymentType only schedules of payment method @p paymentType
+    *                  are searched for.
+    *                  See MyMoneySchedule::paymentTypeE for details.
+    *                  Default is MyMoneySchedule::STYPE_ANY
+    * @param startDate only schedules with payment dates after @p startDate
+    *                  are searched for. Default is all dates (QDate()).
+    * @param endDate   only schedules with payment dates ending prior to @p endDate
+    *                  are searched for. Default is all dates (QDate()).
+    * @param overdue   if true, only those schedules that are overdue are
+    *                  searched for. Default is false (all schedules will be returned).
+    *
+    * @return const QValueList<MyMoneySchedule> list of schedule objects.
+    */
+  const QValueList<MyMoneySchedule> scheduleList(const QCString& = QCString(),
+                                     const MyMoneySchedule::typeE = MyMoneySchedule::TYPE_ANY,
+                                     const MyMoneySchedule::occurenceE = MyMoneySchedule::OCCUR_ANY,
+                                     const MyMoneySchedule::paymentTypeE = MyMoneySchedule::STYPE_ANY,
+                                     const QDate& = QDate(),
+                                     const QDate& = QDate(),
+                                     const bool = false) const;
+
+  QValueList<MyMoneySchedule> scheduleListEx( int scheduleTypes,
+                                              int scheduleOcurrences,
+                                              int schedulePaymentTypes,
+                                              QDate startDate,
+                                              const QCStringList& accounts=QCStringList()) const;
+    
 private:
 
   static const int INSTITUTION_ID_SIZE = 6;
   static const int ACCOUNT_ID_SIZE = 6;
   static const int TRANSACTION_ID_SIZE = 18;
   static const int PAYEE_ID_SIZE = 6;
+  static const int SCHEDULE_ID_SIZE = 6;
 
   static const int YEAR_SIZE = 4;
   static const int MONTH_SIZE = 2;
@@ -564,14 +679,6 @@ private:
     */
   void invalidateBalanceCache(const QCString& id);
 
-  /**
-    * This method reconstructs the transaction list of an account
-    * in the m_accountList.
-    *
-    * @param acc iterator to the m_accountList
-    */
-  void refreshAccountTransactionList(QMap<QCString, MyMoneyAccount>::Iterator acc) const;
-
   /**
     * This member variable keeps the name of the User
     * @see userName(), setUserName()
@@ -642,6 +749,13 @@ private:
     */
   unsigned long m_nextPayeeID;
 
+  /**
+    * The member variable m_nextScheduleID keeps the number that will be
+    * assigned to the next schedule created. It is maintained by
+    * nextScheduleID()
+    */
+  unsigned long m_nextScheduleID;
+
   /**
     * The member variable m_institutionList is the container for the
     * institutions known within this file.
@@ -679,6 +793,11 @@ private:
     */
   QMap<QCString, MyMoneyPayee> m_payeeList;
 
+  /**
+    * A list containing all the scheduled transactions
+    */
+  QMap<QCString, MyMoneySchedule> m_scheduleList;
+  
   /**
     * This member signals if the file has been modified or not
     */
@@ -717,10 +836,16 @@ private:
 
   /**
     * This method is used to get the next valid ID for a payee
-    * @return id for a transaction
+    * @return id for a payee
     */
   const QCString nextPayeeID(void);
 
+  /**
+    * This method is used to get the next valid ID for a scheduled transaction
+    * @return id for a scheduled transaction
+    */
+  const QCString nextScheduleID(void);
+
   /**
     * This method re-parents an existing account
     *
