@@ -34,20 +34,25 @@ void MyMoneySeqAccessMgrTest::tearDown()
 
 void MyMoneySeqAccessMgrTest::testEmptyConstructor()
 {
-	CPPUNIT_ASSERT(m->m_userName == "");
-	CPPUNIT_ASSERT(m->m_userStreet == "");
-	CPPUNIT_ASSERT(m->m_userTown == "");
-	CPPUNIT_ASSERT(m->m_userCounty == "");
-	CPPUNIT_ASSERT(m->m_userPostcode == "");
-	CPPUNIT_ASSERT(m->m_userTelephone == "");
-	CPPUNIT_ASSERT(m->m_userEmail == "");
+	CPPUNIT_ASSERT(m->m_userName.isEmpty());
+	CPPUNIT_ASSERT(m->m_userStreet.isEmpty());
+	CPPUNIT_ASSERT(m->m_userTown.isEmpty());
+	CPPUNIT_ASSERT(m->m_userCounty.isEmpty());
+	CPPUNIT_ASSERT(m->m_userPostcode.isEmpty());
+	CPPUNIT_ASSERT(m->m_userTelephone.isEmpty());
+	CPPUNIT_ASSERT(m->m_userEmail.isEmpty());
 	CPPUNIT_ASSERT(m->m_nextInstitutionID == 0);
 	CPPUNIT_ASSERT(m->m_nextAccountID == 0);
 	CPPUNIT_ASSERT(m->m_nextTransactionID == 0);
+	CPPUNIT_ASSERT(m->m_nextPayeeID == 0);
+	CPPUNIT_ASSERT(m->m_nextScheduleID == 0);
 	CPPUNIT_ASSERT(m->m_institutionList.count() == 0);
 	CPPUNIT_ASSERT(m->m_accountList.count() == 4);
 	CPPUNIT_ASSERT(m->m_transactionList.count() == 0);
 	CPPUNIT_ASSERT(m->m_transactionKeys.count() == 0);
+	CPPUNIT_ASSERT(m->m_payeeList.count() == 0);
+	CPPUNIT_ASSERT(m->m_scheduleList.count() == 0);
+
 	CPPUNIT_ASSERT(m->m_dirty == false);
 	CPPUNIT_ASSERT(m->m_creationDate == QDate::currentDate());
 
@@ -91,6 +96,20 @@ void MyMoneySeqAccessMgrTest::testSetFunctions() {
 	CPPUNIT_ASSERT(m->userEmail() == "Email");
 }
 
+void MyMoneySeqAccessMgrTest::testSupportFunctions()
+{
+	CPPUNIT_ASSERT(m->nextInstitutionID() == "I000001");
+	CPPUNIT_ASSERT(m->m_nextInstitutionID == 1);
+	CPPUNIT_ASSERT(m->nextAccountID() == "A000001");
+	CPPUNIT_ASSERT(m->m_nextAccountID == 1);
+	CPPUNIT_ASSERT(m->nextTransactionID() == "T000000000000000001");
+	CPPUNIT_ASSERT(m->m_nextTransactionID == 1);
+	CPPUNIT_ASSERT(m->nextPayeeID() == "P000001");
+	CPPUNIT_ASSERT(m->m_nextPayeeID == 1);
+	CPPUNIT_ASSERT(m->nextScheduleID() == "SCH000001");
+	CPPUNIT_ASSERT(m->m_nextScheduleID == 1);
+}
+
 void MyMoneySeqAccessMgrTest::testIsStandardAccount()
 {
 	CPPUNIT_ASSERT(m->isStandardAccount(STD_ACC_LIABILITY) == true);
@@ -413,14 +432,14 @@ void MyMoneySeqAccessMgrTest::testAddTransactions() {
 		s.setAccountId("A000006");	// Checkings
 		s.setShares(100000);
 		s.setValue(100000);
-		CPPUNIT_ASSERT(s.id() == "");
+		CPPUNIT_ASSERT(s.id().isEmpty());
 		t1.addSplit(s);
 
-		s.setId("");		// enable re-usage of split variable
+		s.setId(QCString());	// enable re-usage of split variable
 		s.setAccountId("A000005");	// Salary
 		s.setShares(-100000);
 		s.setValue(-100000);
-		CPPUNIT_ASSERT(s.id() == "");
+		CPPUNIT_ASSERT(s.id().isEmpty());
 		t1.addSplit(s);
 
 		t1.setPostDate(QDate(2002,5,10));
@@ -441,32 +460,32 @@ void MyMoneySeqAccessMgrTest::testAddTransactions() {
 
 	try {
 		// I spent some money, not so great
-		s.setId("");		// enable re-usage of split variable
+		s.setId(QCString());	// enable re-usage of split variable
 		s.setAccountId("A000004");	// Grosseries
 		s.setShares(10000);
 		s.setValue(10000);
-		CPPUNIT_ASSERT(s.id() == "");
+		CPPUNIT_ASSERT(s.id().isEmpty());
 		t2.addSplit(s);
 
-		s.setId("");		// enable re-usage of split variable
+		s.setId(QCString());	// enable re-usage of split variable
 		s.setAccountId("A000002");	// 16% sales tax
 		s.setShares(1200);
 		s.setValue(1200);
-		CPPUNIT_ASSERT(s.id() == "");
+		CPPUNIT_ASSERT(s.id().isEmpty());
 		t2.addSplit(s);
 
-		s.setId("");		// enable re-usage of split variable
+		s.setId(QCString());	// enable re-usage of split variable
 		s.setAccountId("A000003");	// 7% sales tax
 		s.setShares(400);
 		s.setValue(400);
-		CPPUNIT_ASSERT(s.id() == "");
+		CPPUNIT_ASSERT(s.id().isEmpty());
 		t2.addSplit(s);
 
-		s.setId("");		// enable re-usage of split variable
+		s.setId(QCString());	// enable re-usage of split variable
 		s.setAccountId("A000006");	// Checkings account
 		s.setShares(-11600);
 		s.setValue(-11600);
-		CPPUNIT_ASSERT(s.id() == "");
+		CPPUNIT_ASSERT(s.id().isEmpty());
 		t2.addSplit(s);
 
 		t2.setPostDate(QDate(2002,5,9));
@@ -501,7 +520,8 @@ void MyMoneySeqAccessMgrTest::testAddTransactions() {
 
 		// check that the account's transaction list is updated
 		QValueList<MyMoneyTransaction> list;
-		list = m->transactionList("A000006");
+		MyMoneyTransactionFilter filter("A000006");
+		list = m->transactionList(filter);
 		CPPUNIT_ASSERT(list.size() == 2);
 
 		QValueList<MyMoneyTransaction>::ConstIterator it;
@@ -613,7 +633,8 @@ void MyMoneySeqAccessMgrTest::testModifyTransaction() {
 
 		// check that the account's transaction list is updated
 		QValueList<MyMoneyTransaction> list;
-		list = m->transactionList("A000006");
+		MyMoneyTransactionFilter filter("A000006");
+		list = m->transactionList(filter);
 		CPPUNIT_ASSERT(list.size() == 2);
 
 		QValueList<MyMoneyTransaction>::ConstIterator it;
@@ -735,7 +756,7 @@ void MyMoneySeqAccessMgrTest::testRemoveInstitution() {
 		m->removeInstitution(i);
 		a = m->account("A000006");
 		CPPUNIT_ASSERT(m->dirty() == true);
-		CPPUNIT_ASSERT(a.institutionId() == "");
+		CPPUNIT_ASSERT(a.institutionId().isEmpty());
 		CPPUNIT_ASSERT(m->institutionCount() == 0);
 	} catch (MyMoneyException *e) {
 		delete e;
@@ -766,17 +787,20 @@ void MyMoneySeqAccessMgrTest::testTransactionList() {
 	testAddTransactions();
 
 	QValueList<MyMoneyTransaction> list;
-
-	list = m->transactionList("A000006");
+	MyMoneyTransactionFilter filter("A000006");
+	list = m->transactionList(filter);
 	CPPUNIT_ASSERT(list.count() == 2);
 	CPPUNIT_ASSERT((*(list.at(0))).id() == "T000000000000000002");
 	CPPUNIT_ASSERT((*(list.at(1))).id() == "T000000000000000001");
 
-	list = m->transactionList("A000003");
+	filter.clear();
+	filter.addAccount("A000003");
+	list = m->transactionList(filter);
 	CPPUNIT_ASSERT(list.count() == 1);
 	CPPUNIT_ASSERT((*(list.at(0))).id() == "T000000000000000002");
 
-	list = m->transactionList("");
+	filter.clear();
+	list = m->transactionList(filter);
 	CPPUNIT_ASSERT(list.count() == 2);
 	CPPUNIT_ASSERT((*(list.at(0))).id() == "T000000000000000002");
 	CPPUNIT_ASSERT((*(list.at(1))).id() == "T000000000000000001");
@@ -923,7 +947,7 @@ void MyMoneySeqAccessMgrTest::testRemoveAccountFromTree() {
 	c.setName("Acc C");
 
 	// build a tree A -> B -> C, remove B and see if A -> C
-	// remains in the storage manager
+	// remains in the storag manager
 
 	try {
 		m->newAccount(a);
@@ -942,7 +966,7 @@ void MyMoneySeqAccessMgrTest::testRemoveAccountFromTree() {
 
 		m->removeAccount(b);
 
-		// reload account info from storage
+		// reload account info from titutionIDtorage
 		a = m->account(a.id());
 		c = m->account(c.id());
 
@@ -986,3 +1010,340 @@ void MyMoneySeqAccessMgrTest::testPayeeName() {
 		delete e;
 	}
 }
+
+void MyMoneySeqAccessMgrTest::testAssignment() {
+	testAddTransactions();
+
+	m->setUserName("Thomas");
+
+	MyMoneySeqAccessMgr test = *m;
+	testEquality(&test);
+}
+
+void MyMoneySeqAccessMgrTest::testEquality(const MyMoneySeqAccessMgr *t)
+{
+	CPPUNIT_ASSERT( m->m_userName == t->m_userName);
+	CPPUNIT_ASSERT( m->m_userStreet == t->m_userStreet);
+	CPPUNIT_ASSERT( m->m_userTown == t->m_userTown);
+	CPPUNIT_ASSERT( m->m_userCounty == t->m_userCounty);
+	CPPUNIT_ASSERT( m->m_userPostcode == t->m_userPostcode);
+	CPPUNIT_ASSERT( m->m_userTelephone == t->m_userTelephone);
+	CPPUNIT_ASSERT( m->m_userEmail == t->m_userEmail);
+	CPPUNIT_ASSERT( m->m_nextInstitutionID == t->m_nextInstitutionID);
+	CPPUNIT_ASSERT( m->m_nextAccountID == t->m_nextAccountID);
+	CPPUNIT_ASSERT( m->m_nextTransactionID == t->m_nextTransactionID);
+	CPPUNIT_ASSERT( m->m_nextPayeeID == t->m_nextPayeeID);
+	CPPUNIT_ASSERT( m->m_nextScheduleID == t->m_nextScheduleID);
+	CPPUNIT_ASSERT( m->m_dirty == t->m_dirty);
+	CPPUNIT_ASSERT( m->m_creationDate == t->m_creationDate);
+	CPPUNIT_ASSERT( m->m_lastModificationDate == t->m_lastModificationDate);
+
+	/*
+	 * make sure, that the keys and values are the same
+	 * on the left and the right side
+	 */
+	CPPUNIT_ASSERT(m->m_payeeList.keys() == t->m_payeeList.keys());
+	CPPUNIT_ASSERT(m->m_payeeList.values() == t->m_payeeList.values());
+	CPPUNIT_ASSERT(m->m_transactionKeys.keys() == t->m_transactionKeys.keys()); 
+	CPPUNIT_ASSERT(m->m_transactionKeys.values() == t->m_transactionKeys.values()); 
+	CPPUNIT_ASSERT(m->m_institutionList.keys() == t->m_institutionList.keys()); 
+	CPPUNIT_ASSERT(m->m_institutionList.values() == t->m_institutionList.values()); 
+	CPPUNIT_ASSERT(m->m_accountList.keys() == t->m_accountList.keys()); 
+	CPPUNIT_ASSERT(m->m_accountList.values() == t->m_accountList.values()); 
+	CPPUNIT_ASSERT(m->m_transactionList.keys() == t->m_transactionList.keys()); 
+	CPPUNIT_ASSERT(m->m_transactionList.values() == t->m_transactionList.values()); 
+	CPPUNIT_ASSERT(m->m_balanceCache.keys() == t->m_balanceCache.keys()); 
+	CPPUNIT_ASSERT(m->m_balanceCache.values() == t->m_balanceCache.values()); 
+
+//	CPPUNIT_ASSERT(m->m_scheduleList.keys() == t->m_scheduleList.keys());
+//	CPPUNIT_ASSERT(m->m_scheduleList.values() == t->m_scheduleList.values());
+}
+
+void MyMoneySeqAccessMgrTest::testDuplicate() {
+	const MyMoneySeqAccessMgr* t;
+
+	testModifyTransaction();
+
+	t = m->duplicate();
+	testEquality(t);
+	delete t;
+}
+
+void MyMoneySeqAccessMgrTest::testAddSchedule() {
+	/* Note addSchedule() now calls validate as it should
+	 * so we need an account id.  Later this will
+	 * be checked to make sure its a valid account id.  The
+	 * tests currently fail because no splits are defined
+	 * for the schedules transaction.
+	*/
+
+
+	try {
+	CPPUNIT_ASSERT(m->m_scheduleList.count() == 0);
+	MyMoneyTransaction t1;
+	MyMoneySplit s1, s2;
+	s1.setAccountId("A000001");
+	t1.addSplit(s1);
+	s2.setAccountId("A000002");
+	t1.addSplit(s2);
+	MyMoneySchedule schedule("Sched-Name",
+				 MyMoneySchedule::TYPE_DEPOSIT,
+				 MyMoneySchedule::OCCUR_DAILY,
+				 MyMoneySchedule::STYPE_MANUALDEPOSIT,
+				 QDate(2003,7,10),
+				 QDate(),
+				 true,
+				 false);
+	schedule.setTransaction(t1);
+
+		m->addSchedule(schedule);
+
+		CPPUNIT_ASSERT(m->m_scheduleList.count() == 1);
+		CPPUNIT_ASSERT(schedule.id() == "SCH000001");
+		CPPUNIT_ASSERT(m->m_scheduleList["SCH000001"].id() == "SCH000001");
+	} catch(MyMoneyException *e) {
+		delete e;
+		CPPUNIT_FAIL("Unexpected exception");
+	}
+
+	try {
+		MyMoneySchedule schedule("Sched-Name",
+					 MyMoneySchedule::TYPE_DEPOSIT,
+					 MyMoneySchedule::OCCUR_DAILY,
+					 MyMoneySchedule::STYPE_MANUALDEPOSIT,
+					 QDate(2003,7,10),
+					 QDate(),
+					 true,
+					 false);
+		m->addSchedule(schedule);
+		CPPUNIT_FAIL("Exception expected");
+	} catch(MyMoneyException *e) {
+		delete e;
+	}
+}
+
+void MyMoneySeqAccessMgrTest::testSchedule() {
+	testAddSchedule();
+	MyMoneySchedule sched;
+
+	sched = m->schedule("SCH000001");
+	CPPUNIT_ASSERT(sched.name() == "Sched-Name");
+	CPPUNIT_ASSERT(sched.id() == "SCH000001");
+
+	try {
+		m->schedule("SCH000002");
+		CPPUNIT_FAIL("Exception expected");
+	} catch(MyMoneyException *e) {
+		delete e;
+	}
+}
+
+void MyMoneySeqAccessMgrTest::testModifySchedule() {
+	testAddSchedule();
+	MyMoneySchedule sched;
+
+	sched = m->schedule("SCH000001");
+	sched.setId("SCH000002");
+	try {
+		m->modifySchedule(sched);
+		CPPUNIT_FAIL("Exception expected");
+	} catch(MyMoneyException *e) {
+		delete e;
+	}
+
+	sched = m->schedule("SCH000001");
+	sched.setName("New Sched-Name");
+	try {
+		m->modifySchedule(sched);
+		CPPUNIT_ASSERT(m->m_scheduleList.count() == 1);
+		CPPUNIT_ASSERT(m->m_scheduleList["SCH000001"].name() == "New Sched-Name");
+		
+	} catch(MyMoneyException *e) {
+		delete e;
+		CPPUNIT_FAIL("Unexpected exception");
+	}
+	
+}
+
+void MyMoneySeqAccessMgrTest::testRemoveSchedule() {
+	testAddSchedule();
+	MyMoneySchedule sched;
+
+	sched = m->schedule("SCH000001");
+	sched.setId("SCH000002");
+	try {
+		m->removeSchedule(sched);
+		CPPUNIT_FAIL("Exception expected");
+	} catch(MyMoneyException *e) {
+		delete e;
+	}
+
+	sched = m->schedule("SCH000001");
+	try {
+		m->removeSchedule(sched);
+		CPPUNIT_ASSERT(m->m_scheduleList.count() == 0);
+		
+	} catch(MyMoneyException *e) {
+		delete e;
+		CPPUNIT_FAIL("Unexpected exception");
+	}
+}
+
+void MyMoneySeqAccessMgrTest::testScheduleList() {
+	QDate	testDate = QDate::currentDate();
+	QDate	notOverdue = testDate.addDays(2);
+	QDate	overdue = testDate.addDays(-2);
+
+
+	MyMoneyTransaction t1;
+	MyMoneySplit s1, s2;
+	s1.setAccountId("A000001");
+	t1.addSplit(s1);
+	s2.setAccountId("A000002");
+	t1.addSplit(s2);
+	MyMoneySchedule schedule1("Schedule 1",
+				 MyMoneySchedule::TYPE_BILL,
+				 MyMoneySchedule::OCCUR_ONCE,
+				 MyMoneySchedule::STYPE_DIRECTDEBIT,
+				 notOverdue,
+				 QDate(),
+				 false,
+				 false);
+	schedule1.setTransaction(t1);
+	schedule1.setLastPayment(notOverdue);
+
+	MyMoneyTransaction t2;
+	MyMoneySplit s3, s4;
+	s3.setAccountId("A000001");
+	t2.addSplit(s3);
+	s4.setAccountId("A000003");
+	t2.addSplit(s4);
+	MyMoneySchedule schedule2("Schedule 2",
+				 MyMoneySchedule::TYPE_DEPOSIT,
+				 MyMoneySchedule::OCCUR_DAILY,
+				 MyMoneySchedule::STYPE_DIRECTDEPOSIT,
+				 notOverdue.addDays(1),
+				 QDate(),
+				 false,
+				 false);
+	schedule2.setTransaction(t2);
+	schedule2.setLastPayment(notOverdue.addDays(1));
+
+	MyMoneyTransaction t3;
+	MyMoneySplit s5, s6;
+	s5.setAccountId("A000005");
+	t3.addSplit(s5);
+	s6.setAccountId("A000006");
+	t3.addSplit(s6);
+	MyMoneySchedule schedule3("Schedule 3",
+				 MyMoneySchedule::TYPE_TRANSFER,
+				 MyMoneySchedule::OCCUR_WEEKLY,
+				 MyMoneySchedule::STYPE_OTHER,
+				 notOverdue.addDays(2),
+				 QDate(),
+				 false,
+				 false);
+	schedule3.setTransaction(t3);
+	schedule3.setLastPayment(notOverdue.addDays(2));
+
+	MyMoneyTransaction t4;
+	MyMoneySplit s7, s8;
+	s7.setAccountId("A000005");
+	t4.addSplit(s7);
+	s8.setAccountId("A000006");
+	t4.addSplit(s8);
+	MyMoneySchedule schedule4("Schedule 4",
+				 MyMoneySchedule::TYPE_BILL,
+				 MyMoneySchedule::OCCUR_WEEKLY,
+				 MyMoneySchedule::STYPE_WRITECHEQUE,
+				 overdue.addDays(-7),
+				 notOverdue.addDays(31),
+				 false,
+				 false);
+	schedule4.setTransaction(t4);
+
+	try {
+		m->addSchedule(schedule1);
+		m->addSchedule(schedule2);
+		m->addSchedule(schedule3);
+		m->addSchedule(schedule4);
+	} catch(MyMoneyException *e) {
+		qDebug("Error: %s", e->what().latin1());
+		delete e;
+		CPPUNIT_FAIL("Unexpected exception");
+	}
+
+	QValueList<MyMoneySchedule> list;
+
+	// no filter
+	list = m->scheduleList();
+	CPPUNIT_ASSERT(list.count() == 4);
+
+	// filter by type
+	list = m->scheduleList("", MyMoneySchedule::TYPE_BILL);
+	CPPUNIT_ASSERT(list.count() == 2);
+	CPPUNIT_ASSERT(list[0].name() == "Schedule 1");
+	CPPUNIT_ASSERT(list[1].name() == "Schedule 4");
+
+	// filter by occurence
+	list = m->scheduleList("", MyMoneySchedule::TYPE_ANY,
+				MyMoneySchedule::OCCUR_DAILY);
+	CPPUNIT_ASSERT(list.count() == 1);
+	CPPUNIT_ASSERT(list[0].name() == "Schedule 2");
+
+	// filter by payment type
+	list = m->scheduleList("", MyMoneySchedule::TYPE_ANY,
+				MyMoneySchedule::OCCUR_ANY,
+				MyMoneySchedule::STYPE_DIRECTDEPOSIT);
+	CPPUNIT_ASSERT(list.count() == 1);
+	CPPUNIT_ASSERT(list[0].name() == "Schedule 2");
+
+	// filter by account
+	list = m->scheduleList("A01");
+	CPPUNIT_ASSERT(list.count() == 0);
+	list = m->scheduleList("A000001");
+	CPPUNIT_ASSERT(list.count() == 2);
+	list = m->scheduleList("A000002");
+	CPPUNIT_ASSERT(list.count() == 1);
+
+	// filter by start date
+	list = m->scheduleList("", MyMoneySchedule::TYPE_ANY,
+				MyMoneySchedule::OCCUR_ANY,
+				MyMoneySchedule::STYPE_ANY,
+				notOverdue.addDays(31));
+	CPPUNIT_ASSERT(list.count() == 3);
+	CPPUNIT_ASSERT(list[0].name() == "Schedule 2");
+	CPPUNIT_ASSERT(list[1].name() == "Schedule 3");
+	CPPUNIT_ASSERT(list[2].name() == "Schedule 4");
+
+	// filter by end date
+	list = m->scheduleList("", MyMoneySchedule::TYPE_ANY,
+				MyMoneySchedule::OCCUR_ANY,
+				MyMoneySchedule::STYPE_ANY,
+				QDate(),
+				notOverdue.addDays(1));
+	CPPUNIT_ASSERT(list.count() == 3);
+	CPPUNIT_ASSERT(list[0].name() == "Schedule 1");
+	CPPUNIT_ASSERT(list[1].name() == "Schedule 2");
+	CPPUNIT_ASSERT(list[2].name() == "Schedule 4");
+
+	// filter by start and end date
+	list = m->scheduleList("", MyMoneySchedule::TYPE_ANY,
+				MyMoneySchedule::OCCUR_ANY,
+				MyMoneySchedule::STYPE_ANY,
+				notOverdue.addDays(-1),
+				notOverdue.addDays(1));
+	CPPUNIT_ASSERT(list.count() == 2);
+	CPPUNIT_ASSERT(list[0].name() == "Schedule 1");
+	CPPUNIT_ASSERT(list[1].name() == "Schedule 2");
+
+	// filter by overdue status
+	list = m->scheduleList("", MyMoneySchedule::TYPE_ANY,
+				MyMoneySchedule::OCCUR_ANY,
+				MyMoneySchedule::STYPE_ANY,
+				QDate(),
+				QDate(),
+				true);
+	CPPUNIT_ASSERT(list.count() == 1);
+	CPPUNIT_ASSERT(list[0].name() == "Schedule 4");
+}
