@@ -184,7 +184,7 @@ void MyMoneyStorageBin::addCategory(IMyMoneySerialize* storage,
     categories[name] = majorCat.id();
   }
 
-  if(minorName != "") {
+  if(!minorName.isEmpty()) {
     name += ":" + minorName;
     if(!categories.contains(name)) {
       MyMoneyAccount minorCat;
@@ -386,10 +386,10 @@ void MyMoneyStorageBin::readOldFormat(QDataStream& s, IMyMoneySerialize* storage
 
         s >> category;    // major category
         s >> tmp;         // minor category
-        if(tmp != "")
+        if(!tmp.isEmpty())
           category += ":" + tmp;
         // for now, we keep it as it is and convert sp2.account later on
-        sp2.setAccountId(static_cast<QCString> (category));
+        sp2.setAccountId(category.latin1());
         s >> tmp; // ATM bank name
         s >> tmp; // account from
         s >> tmp; // account to
@@ -457,18 +457,18 @@ void MyMoneyStorageBin::readOldFormat(QDataStream& s, IMyMoneySerialize* storage
       accname = accname.mid(1, accname.length()-2);
       sp.setAccountId(accountConversion[accname]);
     } else {
-     	int colonindex = accname.find(":");
-     	QString catmajor;
-    	QString catminor;
+      int colonindex = accname.find(":");
+      QString catmajor;
+      QString catminor;
       if(colonindex == -1) {
-       	catmajor = accname;
-    		catminor = "";
-      }	else {
-    		int len = accname.length();
-    		len--;
-       	catmajor = accname.left(colonindex);
-    		catminor = accname.right(len - colonindex);
-    	}
+        catmajor = accname;
+        catminor = QString();
+      } else {
+        int len = accname.length();
+        len--;
+        catmajor = accname.left(colonindex);
+        catminor = accname.right(len - colonindex);
+      }
       addCategory(storage, categoryConversion, catmajor, catminor,
            sp.value() >= 0 ? MyMoneyAccount::Expense : MyMoneyAccount::Income);
       sp.setAccountId(categoryConversion[accname]);
@@ -477,7 +477,6 @@ void MyMoneyStorageBin::readOldFormat(QDataStream& s, IMyMoneySerialize* storage
     storage->addTransaction(*it, true);
   }
 
-  storage->refreshAllAccountTransactionLists();
   storage->setLastModificationDate(lastModificationDate);
 
 }
@@ -522,16 +521,19 @@ void MyMoneyStorageBin::readNewFormat(QDataStream&s, IMyMoneySerialize* storage)
 
   readTransactions(s, storage);
 
-  if(s.atEnd())
-    return;
-
-  // for now, we don't read the scheduled transactions back, as there
-  // might be files out there that don't have it.
-  readScheduledTransactions(s, storage);
+  // if we're at the end of file here, then we deal with an old file
+  // and stop processing. Else we continue as long as we can.
   if(s.atEnd())
     return;
 
   storage->setPairs(readKeyValueContainer(s));
+  if(s.atEnd())
+    return;
+    
+  readScheduledTransactions(s, storage);
+  if(s.atEnd())
+    return;
+
 }
 
 void MyMoneyStorageBin::writeFile(QIODevice* qfile, IMyMoneySerialize* storage)
@@ -582,10 +584,13 @@ void MyMoneyStorageBin::writeStream(QDataStream& s, IMyMoneySerialize* storage)
   s << storage->lastModificationDate();
 
   // leave an estimate on the number of items that follow
+  MyMoneyTransactionFilter filter;
   s << storage->institutionList().count() +
        storage->payeeList().count() +
        storage->accountList().count() +
-       storage->transactionList().count();
+       storage->transactionList(filter).count() +
+       storage->pairs().count() +
+       storage->scheduleList().count();
 
   writeInstitutions(s, storage);
   writePayees(s, storage);
@@ -594,6 +599,8 @@ void MyMoneyStorageBin::writeStream(QDataStream& s, IMyMoneySerialize* storage)
   writeKeyValueContainer(s, storage->pairs());
   writeScheduledTransactions(s, storage);
 
+  // add new items to be saved in front of this line
+  
   // this seems to be nonsense, but it clears the dirty flag
   // as a side-effect.
   storage->setLastModificationDate(storage->lastModificationDate());
@@ -866,11 +873,12 @@ void MyMoneyStorageBin::writeTransactions(QDataStream& s, IMyMoneySerialize* sto
   Q_INT32 tmp;
   QValueList<MyMoneyTransaction> list;
   QValueList<MyMoneyTransaction>::ConstIterator it;
-
+  MyMoneyTransactionFilter filter;
+  
   tmp = 1;      // version
   s << tmp;
 
-  list = storage->transactionList();
+  list = storage->transactionList(filter);
 
   s << list.count();
 
@@ -995,7 +1003,7 @@ const MyMoneySplit MyMoneyStorageBin::readSplit(QDataStream& s)
     case MyMoneyAccount::Liability:
       break;
     default:
-      sp.setPayeeId("");
+      sp.setPayeeId(QCString());
       break;
   }
   return sp;
@@ -1017,19 +1025,34 @@ void MyMoneyStorageBin::writeScheduledTransactions(QDataStream& s, IMyMoneySeria
 {
   s << (Q_INT32) 1;   // version
 
-  s << (Q_INT32) 0;   // for now there's nothing
+  QValueList<MyMoneySchedule> list;
+  QValueList<MyMoneySchedule>::ConstIterator it;
+
+  list = storage->scheduleList();
+  s << list.count();
+  for(it = list.begin(); it != list.end(); ++it) {
+    writeSchedule(s, *it);
+  }
 }
 
 void MyMoneyStorageBin::readScheduledTransactions(QDataStream& s, IMyMoneySerialize* storage)
 {
   Q_INT32 version;
   Q_INT32 cnt;
+  unsigned long id;
 
   s >> version;
-
   s >> cnt;
-
+  
+  signalProgress(0, cnt, QObject::tr("Loading schedules..."));
   for(int i = 0; i < cnt; ++i) {
+    MyMoneySchedule sched = readSchedule(s);
+    storage->loadSchedule(sched);
+
+    id = extractId(sched.id().data());
+    if(id > storage->scheduleId())
+      storage->loadScheduleId(id);
+    signalProgress(i, 0);
   }
 }
 
@@ -1056,3 +1079,170 @@ void MyMoneyStorageBin::signalProgress(int current, int total, const QString& ms
   if(m_progressCallback != 0)
     (*m_progressCallback)(current, total, msg);
 }
+
+void MyMoneyStorageBin::writeSchedule(QDataStream&s, const MyMoneySchedule& sc)
+{
+  Q_INT32 tmp;
+  tmp = 6;    // version
+  s << tmp;
+
+  s << sc.occurence();
+  s << sc.type();
+  s << sc.startDate();
+  s << sc.paymentType();
+  s << sc.isFixed();
+  s << sc.endDate();
+  s << sc.autoEnter();
+  s << sc.id();
+  s << sc.lastPayment();
+  s << sc.name();
+  s << sc.weekendOption();
+
+  QValueList<QDate> payments = sc.recordedPayments();
+  QValueList<QDate>::Iterator it;
+
+  s << payments.count();
+  for (it=payments.begin(); it!=payments.end(); ++it)
+    s << *it;
+    
+  writeTransaction(s, sc.transaction());
+}
+
+const MyMoneySchedule MyMoneyStorageBin::readSchedule(QDataStream& s)
+{
+  QCString id;
+  MyMoneySchedule  sc;
+  Q_INT32 version;
+  QString tmp_s;
+  Q_INT32 tmp_n;
+  QDate tmp_d;
+
+  s >> version;
+  s >> tmp_n;
+  if (version <= 3)
+    tmp_n = convertOldOccurenceEnum(tmp_n);
+  sc.setOccurence((MyMoneySchedule::occurenceE)tmp_n);
+  s >> tmp_n;
+  if (version <= 3)
+    tmp_n = convertOldTypeEnum(tmp_n);
+  sc.setType((MyMoneySchedule::typeE)tmp_n);
+  s >> tmp_d; sc.setStartDate(tmp_d);
+  s >> tmp_n;
+  if (version <= 3)
+    tmp_n = convertOldPaymentType(tmp_n);
+  sc.setPaymentType((MyMoneySchedule::paymentTypeE)tmp_n);
+  s >> tmp_n; sc.setFixed(tmp_n);
+  
+  if(version < 3)
+    s >> tmp_n; // sc.setWillEnd(tmp_n);
+    
+  if (version < 2)
+  {
+    s >> tmp_n; /* sc.setTransactionsRemaining(tmp_n); */
+  }
+  s >> tmp_d; sc.setEndDate(tmp_d);
+  s >> tmp_n; sc.setAutoEnter(tmp_n);
+  s >> id; sc.setId(id);
+  s >> tmp_d; sc.setLastPayment(tmp_d);
+  s >> tmp_s; sc.setName(tmp_s);
+  if (version < 2)
+  {
+    // Discard the old account id's
+    s >> id;
+    s >> id;
+  }
+
+  if (version >= 6)
+  {
+      s >> tmp_n;
+      sc.setWeekendOption((MyMoneySchedule::weekendOptionE)tmp_n);
+  }
+
+  if (version >= 5)
+  {
+    s >> tmp_n;
+    for (int i=0; i<tmp_n; ++i)
+    {
+      s >> tmp_d;
+      sc.recordPayment(tmp_d);
+    }
+  }
+  
+  MyMoneyTransaction t = readTransaction(s);
+  sc.setTransaction(t);
+
+  return sc;
+}
+
+int MyMoneyStorageBin::convertOldOccurenceEnum(int tmp_n)
+{
+  switch (tmp_n)
+  {
+    case 0:
+      return 1;
+    case 1:
+      return 2;
+    case 2:
+      return 4;
+    case 3:
+      return 8;
+    case 4:
+      return 16;
+    case 5:
+      return 32;
+    case 6:
+      return 64;
+    case 7:
+      return 128;
+    case 8:
+      return 256;
+    case 9:
+      return 512;
+    case 10:
+      return 1024;
+    case 11:
+      return 2048;
+    case 12:
+      return 4096;
+    case 13:
+      return 8192;
+    case 14:
+      return 16384;
+    default:
+      return 0;
+  }
+}
+
+int MyMoneyStorageBin::convertOldTypeEnum(int tmp_n)
+{
+  switch (tmp_n)
+  {
+    case 15:
+      return 1;
+    case 16:
+      return 2;
+    case 17:
+      return 4;
+    default:
+      return 0;
+  }
+}
+
+int MyMoneyStorageBin::convertOldPaymentType(int tmp_n)
+{
+  switch (tmp_n)
+  {
+    case 19:
+      return 1;
+    case 20:
+      return 2;
+    case 21:
+      return 4;
+    case 22:
+      return 8;
+    case 23:
+      return 16;
+    default:
+      return 0;
+  }
+}
