@@ -30,6 +30,7 @@
 #include <klocale.h>
 #include <klistview.h>
 #include <kconfig.h>
+#include <kmessagebox.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -37,6 +38,8 @@
 #include "kbanksview.h"
 #include "kbanklistitem.h"
 #include "../mymoney/mymoneyfile.h"
+#include "../dialogs/knewaccountdlg.h"
+#include "../kmymoneyutils.h"
 
 static const char* const assetIconImage[] = {
 "32 32 9 1",
@@ -206,7 +209,7 @@ static const char* const checkingsIconImage[] = {
 ".............cc##aa##cc.........",
 "...............cc##cc..........."};
 
-static const char* const investmentIconImage[] = {
+static const char* const liabilityIconImage[] = {
 "32 32 5 1",
 ". c None",
 "# c #000000",
@@ -291,11 +294,12 @@ static const char* const loanIconImage[] = {
 
 
 KAccountsView::KAccountsView(QWidget *parent, const char *name)
- : KBankViewDecl(parent,name)
+ : KBankViewDecl(parent,name),
+   m_suspendUpdate(false)
 {
   KConfig *config = KGlobal::config();
   config->setGroup("List Options");
-  m_bViewNormalAccountsView = config->readBoolEntry("NormalAccountsView", true);
+  m_bViewNormalAccountsView = config->readBoolEntry("NormalAccountsView", false);
 
   accountListView->setRootIsDecorated(true);
   accountListView->setAllColumnsShowFocus(true);
@@ -303,14 +307,14 @@ KAccountsView::KAccountsView(QWidget *parent, const char *name)
   accountListView->addColumn(i18n("Account"));
   accountListView->addColumn(i18n("Entries"));
   accountListView->addColumn(i18n("Balance"));
-  accountListView->addColumn(i18n("Type"));
+  // accountListView->addColumn(i18n("Type"));
 
   accountListView->setMultiSelection(false);
 
   accountListView->setColumnWidthMode(0, QListView::Maximum);
   accountListView->setColumnWidthMode(1, QListView::Maximum);
   accountListView->setColumnWidthMode(2, QListView::Maximum);
-  accountListView->setColumnWidthMode(2, QListView::Maximum);
+  // accountListView->setColumnWidthMode(3, QListView::Maximum);
 
   accountListView->setColumnAlignment(2, Qt::AlignRight);
   accountListView->setColumnAlignment(1, Qt::AlignRight);
@@ -319,8 +323,7 @@ KAccountsView::KAccountsView(QWidget *parent, const char *name)
 
   accountListView->header()->setResizeEnabled(true);
 
-  QFont defaultFont = QFont("helvetica", 12);
-  accountListView->header()->setFont(config->readFontEntry("listHeaderFont", &defaultFont));
+  accountListView->header()->setFont(KMyMoneyUtils::headerFont());
 
   // select the type the user viewed last
   config->setGroup("Last Use Settings");
@@ -361,6 +364,7 @@ KAccountsView::KAccountsView(QWidget *parent, const char *name)
   MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassAccount, this);
   MyMoneyFile::instance()->attach(MyMoneyFile::NotifyClassInstitution, this);
 
+  refresh(QCString());
 }
 
 KAccountsView::~KAccountsView()
@@ -370,7 +374,7 @@ KAccountsView::~KAccountsView()
   MyMoneyFile::instance()->detach(MyMoneyFile::NotifyClassInstitution, this);
 }
 
-void KAccountsView::slotListDoubleClicked(QListViewItem* item, const QPoint& pos, int c)
+void KAccountsView::slotListDoubleClicked(QListViewItem* item, const QPoint& /* pos */, int /* c */)
 {
   KAccountListItem* accountItem = static_cast<KAccountListItem*> (item);
   if(accountItem)
@@ -409,6 +413,7 @@ void KAccountsView::slotListDoubleClicked(QListViewItem* item, const QPoint& pos
 
 void KAccountsView::slotIconDoubleClicked(QIconViewItem* item)
 {
+
   KAccountIconItem *accountItem = static_cast<KAccountIconItem*> (item);
 
   if(accountItem) {
@@ -434,7 +439,7 @@ void KAccountsView::slotIconRightMouse(QIconViewItem* item, const QPoint&)
       m_bSelectedAccount=true;
       m_bSelectedInstitution=false;
       m_selectedAccount = accountItem->accountID();
-
+      
       emit accountRightMouseClick();
 
     } catch (MyMoneyException *e) {
@@ -450,6 +455,9 @@ void KAccountsView::slotListRightMouse(QListViewItem* item, const QPoint& , int
 
   } else {
     KAccountListItem* accountItem = static_cast<KAccountListItem*> (item);
+
+
+
     if (accountItem) {
       try {
         MyMoneyFile *file = MyMoneyFile::instance();
@@ -459,13 +467,15 @@ void KAccountsView::slotListRightMouse(QListViewItem* item, const QPoint& , int
         m_bSelectedAccount=true;
         m_bSelectedInstitution=false;
         m_selectedAccount = accountItem->accountID();
+
         emit accountRightMouseClick();
 
       } catch (MyMoneyException *e) {
         m_bSelectedAccount=false;
         m_bSelectedInstitution=true;
         m_selectedInstitution = accountItem->accountID();
-
+        delete e;
+        
         emit bankRightMouseClick();
       }
     }
@@ -475,67 +485,108 @@ void KAccountsView::slotListRightMouse(QListViewItem* item, const QPoint& , int
 QCString KAccountsView::currentAccount(bool& success)
 {
   success=m_bSelectedAccount;
-  return (success) ? m_selectedAccount : QCString("");
+  return (success) ? m_selectedAccount : QCString();
 }
 
 QCString KAccountsView::currentInstitution(bool& success)
 {
   success=m_bSelectedInstitution;
-  return (success) ? m_selectedInstitution : QCString("");
+  return (success) ? m_selectedInstitution : QCString();
 }
 
 void KAccountsView::update(const QCString& id)
 {
-  if(id == MyMoneyFile::NotifyClassAccountHierarchy
-  || (id == MyMoneyFile::NotifyClassInstitution && m_bViewNormalAccountsView == true))
-    refresh(id);
-  if(id == MyMoneyFile::NotifyClassAccount)
-    refreshTotalProfit();
+  // to avoid constant update when a lot of accounts are added
+  // (e.g. during creation of a new MyMoneyFile object when the
+  // default accounts are loaded) a switch is supported to suppress
+  // updates in this phase. The switch is controlled with suspendUpdate().
+  if(m_suspendUpdate == false) {
+    if(id == MyMoneyFile::NotifyClassAccountHierarchy
+    || (id == MyMoneyFile::NotifyClassInstitution && m_bViewNormalAccountsView == true)) {
+      refresh(id);
+    }
+    if(id == MyMoneyFile::NotifyClassAccount)
+      refreshNetWorth();
+  }
 }
 
-void KAccountsView::refreshView(void)
+void KAccountsView::suspendUpdate(const bool suspend)
 {
+  KAccountListItem* item = static_cast<KAccountListItem *>(accountListView->firstChild());
+
+  // inform all children
+  while(item) {
+    item->suspendUpdate(suspend);
+    item = static_cast<KAccountListItem *>(item->itemBelow());
+  }
+
+  // force a refresh, if update was off
+  if(m_suspendUpdate == true
+  && suspend == false) {
+    refresh(MyMoneyFile::NotifyClassAccountHierarchy);
+    if(m_bViewNormalAccountsView == true)
+      refresh(MyMoneyFile::NotifyClassInstitution);
+    refreshNetWorth();
+  }
+  
+  m_suspendUpdate = suspend;
+}
+
+
+void KAccountsView::slotRefreshView(void)
+{
+
   refresh(m_selectedAccount);
 }
 
-void KAccountsView::refreshTotalProfit(void)
+void KAccountsView::refreshNetWorth(void)
 {
-  KConfig *config = KGlobal::config();
-  config->setGroup("List Options");
-  QFont defaultFont = QFont("helvetica", 12);
-
-  MyMoneyMoney totalProfit;
+  MyMoneyMoney netWorth;
   MyMoneyFile* file = MyMoneyFile::instance();
 
   MyMoneyAccount liabilityAccount = file->liability();
   MyMoneyAccount assetAccount = file->asset();
 
-  totalProfit = file->totalBalance(assetAccount.id()) +
-                file->totalBalance(liabilityAccount.id());
+  netWorth = file->totalBalance(assetAccount.id()) +
+             file->totalBalance(liabilityAccount.id());
 
-  QString s(i18n("Total Profits: "));
-  s += totalProfit.formatMoney();
+  QString s(i18n("Net Worth: "));
+  s += netWorth.formatMoney();
 
 
-  totalProfitsLabel->setFont(config->readFontEntry("listCellFont", &defaultFont));
+  totalProfitsLabel->setFont(KMyMoneyUtils::cellFont());
   totalProfitsLabel->setText(s);
 }
 
+
+
+
 const QPixmap KAccountsView::accountImage(const MyMoneyAccount::accountTypeE type) const
 {
   QPixmap rc;
   switch(type) {
     default:
+      if(MyMoneyFile::instance()->accountGroup(type) == MyMoneyAccount::Asset)
+        rc = QPixmap(assetIconImage);
+      else
+        rc = QPixmap(liabilityIconImage);
+      break;
+      
     case MyMoneyAccount::Checkings:
     case MyMoneyAccount::Savings:
       rc = QPixmap(checkingsIconImage);
       break;
 
+    case MyMoneyAccount::Loan:
+      rc = QPixmap(loanIconImage);
+      break;
+      
     case MyMoneyAccount::CreditCard:
       rc = QPixmap(creditCardIconImage);
       break;
 
     case MyMoneyAccount::Asset:
+    case MyMoneyAccount::AssetLoan:
       rc = QPixmap(assetIconImage);
       break;
 
@@ -547,6 +598,7 @@ const QPixmap KAccountsView::accountImage(const MyMoneyAccount::accountTypeE typ
   return rc;
 }
 
+/*
 void KAccountsView::fillTransactionCountMap(void)
 {
   QValueList<MyMoneyTransaction> list;
@@ -562,37 +614,41 @@ void KAccountsView::fillTransactionCountMap(void)
     for(it_s = (*it_t).splits().begin(); it_s != (*it_t).splits().end(); ++it_s) {
       m_transactionCountMap[(*it_s).accountId()]++;
     }
+
   }
 }
+*/
 
 void KAccountsView::fillAccountMap(void)
 {
   QValueList<MyMoneyAccount> accountList;
   accountList = MyMoneyFile::instance()->accountList();
 
+
   m_accountMap.clear();
 
   QValueList<MyMoneyAccount>::ConstIterator it_a;
   for(it_a = accountList.begin(); it_a != accountList.end(); ++it_a)
     m_accountMap[(*it_a).id()] = *it_a;
-
 }
 
 
 void KAccountsView::refresh(const QCString& selectAccount)
 {
+  accountListView->header()->setFont(KMyMoneyUtils::headerFont());
+
   KConfig *config = KGlobal::config();
   config->setGroup("List Options");
-  QFont defaultFont = QFont("helvetica", 12);
-  accountListView->header()->setFont(config->readFontEntry("listHeaderFont", &defaultFont));
-  m_bViewNormalAccountsView = config->readBoolEntry("NormalAccountsView", true);
+  m_bViewNormalAccountsView = config->readBoolEntry("NormalAccountsView", false);
+  m_hideCategory = config->readBoolEntry("HideUnusedCategory", false);
+  bool accountUsed;
 
   clear();
 
   m_selectedAccount = selectAccount;
 
   fillAccountMap();
-  fillTransactionCountMap();
+  m_transactionCountMap = MyMoneyFile::instance()->transactionCountMap();
 
   MyMoneyFile *file = MyMoneyFile::instance();
 
@@ -600,6 +656,38 @@ void KAccountsView::refresh(const QCString& selectAccount)
     accountListView->header()->setLabel(0, i18n("Institution"));
 
     try {
+      // scan for all asset/liability accounts w/o an institution
+      KAccountListItem *topLevelInstitution = new KAccountListItem(accountListView,
+                       i18n("Accounts with no institution assigned"));
+      QValueList<MyMoneyAccount> acclist = file->accountList();
+      QValueList<MyMoneyAccount>::ConstIterator accountIterator;
+      for(accountIterator = acclist.begin();
+          accountIterator != acclist.end();
+          ++accountIterator) {
+
+        switch((*accountIterator).accountGroup()) {
+          case MyMoneyAccount::Asset:
+
+          case MyMoneyAccount::Liability:
+            if((*accountIterator).institutionId().isEmpty()) {
+              KAccountListItem *accountItem;
+              accountItem = new KAccountListItem(topLevelInstitution, (*accountIterator));
+              accountItem->setText(1, QString("%1").arg(m_transactionCountMap[(*accountIterator).id()]));
+              new KAccountIconItem(accountIconView, (*accountIterator),
+                accountImage((*accountIterator).accountType()));
+            }
+            break;
+          default:
+            break;
+        }
+      }
+      
+      // in case there is no account w/o reference to an institution
+      // we can safely remove this entry to avoid user's confusion
+      if(topLevelInstitution->childCount() == 0) {
+        delete topLevelInstitution;
+      }
+      
       QValueList<MyMoneyInstitution> list = file->institutionList();
       QValueList<MyMoneyInstitution>::ConstIterator institutionIterator;
       for (institutionIterator = list.begin();
@@ -618,13 +706,14 @@ void KAccountsView::refresh(const QCString& selectAccount)
               m_accountMap[*it]);
           accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
 
-          KAccountIconItem* accountIcon = new KAccountIconItem(accountIconView,
+          new KAccountIconItem(accountIconView,
               m_accountMap[*it],
               accountImage(m_accountMap[*it].accountType()));
 
           QCStringList subAccounts = m_accountMap[*it].accountList();
           if (subAccounts.count() >= 1) {
-            showSubAccounts(subAccounts, accountItem, i18n(""));
+
+            showSubAccounts(subAccounts, accountItem, "");
           }
         }
       }
@@ -650,7 +739,7 @@ void KAccountsView::refresh(const QCString& selectAccount)
             m_accountMap[*it]);
         accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
 
-        KAccountIconItem* accountIcon = new KAccountIconItem(accountIconView,
+        new KAccountIconItem(accountIconView,
             m_accountMap[*it],
             accountImage(m_accountMap[*it].accountType()));
 
@@ -672,7 +761,8 @@ void KAccountsView::refresh(const QCString& selectAccount)
             m_accountMap[*it]);
         accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
 
-        KAccountIconItem* accountIcon = new KAccountIconItem(accountIconView,
+        new KAccountIconItem(accountIconView,
+
             m_accountMap[*it],
             accountImage(m_accountMap[*it].accountType()));
 
@@ -687,6 +777,7 @@ void KAccountsView::refresh(const QCString& selectAccount)
       KAccountListItem *incomeTopLevelAccount = new KAccountListItem(accountListView,
             incomeAccount);
 
+
       for ( QCStringList::ConstIterator it = incomeAccount.accountList().begin();
             it != incomeAccount.accountList().end();
             ++it ) {
@@ -694,9 +785,17 @@ void KAccountsView::refresh(const QCString& selectAccount)
             m_accountMap[*it]);
         accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
 
+        accountUsed = m_transactionCountMap[*it] > 0;
+
         QCStringList subAccounts = m_accountMap[*it].accountList();
         if (subAccounts.count() >= 1) {
-          showSubAccounts(subAccounts, accountItem, i18n("Income"));
+          accountUsed |= showSubAccounts(subAccounts, accountItem, i18n("Income"));
+        }
+        if(accountUsed == false && m_hideCategory == true) {
+          // in case hide category is on and the account or any of it's
+          // subaccounts has no split, we can safely remove it and all
+          // it's sub-ordinate accounts from the list
+          delete accountItem;
         }
       }
 
@@ -712,10 +811,18 @@ void KAccountsView::refresh(const QCString& selectAccount)
             m_accountMap[*it]);
         accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
 
+        accountUsed = m_transactionCountMap[*it] > 0;
+
         QCStringList subAccounts = m_accountMap[*it].accountList();
 
         if (subAccounts.count() >= 1) {
-          showSubAccounts(subAccounts, accountItem, i18n("Expense"));
+          accountUsed |= showSubAccounts(subAccounts, accountItem, i18n("Expense"));
+        }
+        if(accountUsed == false && m_hideCategory == true) {
+          // in case hide category is on and the account or any of it's
+          // subaccounts has no split, we can safely remove it and all
+          // it's sub-ordinate accounts from the list
+          delete accountItem;
         }
       }
 
@@ -728,50 +835,60 @@ void KAccountsView::refresh(const QCString& selectAccount)
   m_accountMap.clear();
   m_transactionCountMap.clear();
 
-  refreshTotalProfit();
+  refreshNetWorth();
 
 /*
+
   if (m_bSelectedBank || m_bSelectedAccount)
     accountListView->setSelected(item, true);
 */
 }
 
-void KAccountsView::showSubAccounts(const QCStringList& accounts, KAccountListItem *parentItem, const QString& group)
+const bool KAccountsView::showSubAccounts(const QCStringList& accounts, KAccountListItem *parentItem, const QString& group)
 {
+  bool  accountUsed = false;
+
   for ( QCStringList::ConstIterator it = accounts.begin(); it != accounts.end(); ++it ) {
     KAccountListItem *accountItem  = new KAccountListItem(parentItem,
           m_accountMap[*it]);
     accountItem->setText(1, QString("%1").arg(m_transactionCountMap[*it]));
 
+    accountUsed = m_transactionCountMap[*it] > 0;
+
     QCStringList subAccounts = m_accountMap[*it].accountList();
     if (subAccounts.count() >= 1) {
-      showSubAccounts(subAccounts, accountItem, group);
+      accountUsed |= showSubAccounts(subAccounts, accountItem, group) || m_transactionCountMap[*it] > 0;
+    }
+
+    if(accountUsed == false && m_hideCategory == true) {
+      // in case hide category is on and the account or any of it's
+      // subaccounts has no split, we can safely remove it and all
+      // it's sub-ordinate accounts from the list
+      delete accountItem;
     }
   }
+  return accountUsed;
 }
 
 void KAccountsView::clear(void)
 {
   accountListView->clear();
   accountIconView->clear();
+
+
   m_bSelectedAccount = false;
   m_bSelectedInstitution=false;
 }
 
 void KAccountsView::resizeEvent(QResizeEvent* e)
 {
-
-  //accountListView->setColumnWidth(0, 400);
-  //accountListView->setColumnWidth(1,150);
-  //int totalWidth=accountListView->width();
-  // accountListView->setColumnWidth(2, totalWidth-550-5);
-
   // call base class resizeEvent()
-  //KBankViewDecl::resizeEvent(e);
+  KBankViewDecl::resizeEvent(e);
 }
 
 void KAccountsView::slotSelectionChanged(QListViewItem *item)
 {
+
   KAccountListItem *accountItem = (KAccountListItem*)item;
   if (accountItem)
   {
@@ -799,15 +916,99 @@ void KAccountsView::slotSelectionChanged(QListViewItem *item)
 
 void KAccountsView::show()
 {
+
   emit signalViewActivated();
 
   QWidget::show();
 }
 
-void KAccountsView::slotViewSelected(QWidget* view)
+void KAccountsView::slotViewSelected(QWidget* /* view */)
 {
   KConfig *config = KGlobal::config();
   config->setGroup("Last Use Settings");
   config->writeEntry("KAccountsView_LastType", accountTabWidget->currentPageIndex());
   config->sync();
 }
+
+void KAccountsView::slotEditClicked(void)
+
+{
+  QCString  accountId;
+  
+  if(accountTabWidget->currentPageIndex() == 0) {   // ListView?
+    KAccountListItem *item = (KAccountListItem *)accountListView->currentItem();
+    if(!item)
+      return;
+    accountId = item->accountID();
+    
+  } else {
+    KAccountIconItem *item = (KAccountIconItem *)accountIconView->currentItem();
+    if(!item)
+      return;
+    accountId = item->accountID();
+  }
+
+  try
+  {
+    MyMoneyFile* file = MyMoneyFile::instance();
+    MyMoneyAccount account = file->account(accountId);
+
+    KNewAccountDlg dlg(account, true, false, this, "hi", i18n("Edit an account"));
+
+    if (dlg.exec())
+    {
+      MyMoneyAccount account = dlg.account();
+      MyMoneyAccount parent = dlg.parentAccount();
+
+      file->modifyAccount(account);
+      if(account.parentAccountId() != parent.id()) {
+        file->reparentAccount(account, parent);
+      }
+    }
+  }
+  catch (MyMoneyException *e)
+  {
+    QString errorString = i18n("Cannot edit category: ");
+    errorString += e->what();
+    KMessageBox::error(this, errorString);
+    delete e;
+  }
+}
+
+void KAccountsView::slotDeleteClicked(void)
+{
+  QCString  accountId;
+
+  if(accountTabWidget->currentPageIndex() == 0) {   // ListView?
+    KAccountListItem *item = (KAccountListItem *)accountListView->currentItem();
+    if(!item)
+      return;
+    accountId = item->accountID();
+
+  } else {
+    KAccountIconItem *item = (KAccountIconItem *)accountIconView->currentItem();
+    if(!item)
+      return;
+    accountId = item->accountID();
+  }
+
+  try
+  {
+    MyMoneyFile *file = MyMoneyFile::instance();
+    MyMoneyAccount account = file->account(accountId);
+    QString prompt = i18n("Do you really want to delete the account '%1'")
+      .arg(account.name());
+
+    if ((KMessageBox::questionYesNo(this, prompt)) == KMessageBox::Yes) {
+      MyMoneyFile *file = MyMoneyFile::instance();
+      file->removeAccount(account);
+    }
+  }
+  catch (MyMoneyException *e)
+  {
+    QString message(i18n("Unable to remove account: "));
+    message += e->what();
+    KMessageBox::error(this, message);
+    delete e;
+  }
+}
