@@ -42,7 +42,6 @@ class QWidgetStack;
 // ----------------------------------------------------------------------------
 // Project Includes
 
-class kMyMoneyRegister;
 class kMyMoneyTransactionForm;
 class kMyMoneyTransactionFormTable;
 class kMyMoneyPayee;
@@ -57,10 +56,11 @@ class KPopupMenu;
 #include "../mymoney/mymoneyaccount.h"
 #include "../mymoney/mymoneytransaction.h"
 #include "../mymoney/mymoneyobserver.h"
-
+#include "../widgets/kmymoneyregister.h"
+#include "../views/kmymoneytransaction.h"
 
 /**
-  *@author Thomas Baumgart
+  * @author Thomas Baumgart
   *
   * @todo Add account (hierarchy) upon new category
   */
@@ -71,7 +71,7 @@ class KPopupMenu;
   * vector is created, the sort method is set to SortPostDate.
   * The sort type can be changed using the method setSortType().
   */
-class KTransactionPtrVector : public QPtrVector<MyMoneyTransaction> {
+class KTransactionPtrVector : public QPtrVector<KMyMoneyTransaction> {
 public:
   /**
     * This enumerator defines the possible sort methods.
@@ -94,7 +94,7 @@ public:
     SortValue,              /**< Sort the vector so that the transactions appear sorted
                               *     according to their value
                               */
-    SortNr,                 /**< Sort the vector so that the transactions appear sorted
+    SortNr                  /**< Sort the vector so that the transactions appear sorted
                               *     according to nr field contents
                               */
 
@@ -155,7 +155,11 @@ private:
   * of the account which ledger is shown. The following specific ledger views are
   * available:
   *
-  * @li KLedgerViewCheckings
+  * - KLedgerViewCheckings
+  * - KLedgerViewAsset
+  * - KLedgerViewCreditCard
+  * - KLedgerViewSavings
+  * - KLedgerViewCash
   *
   * Each KLedgerView is devided into three parts:
   *
@@ -191,7 +195,7 @@ private:
   * form of the tab order for the keyboard focus while editing a transaction.
   * It has to be filled and maintained by the derived class.
   */
-class KLedgerView : public QWidget, MyMoneyObserver  {
+class KLedgerView : public QWidget, MyMoneyObserver, public IMyMoneyRegisterParent  {
   Q_OBJECT
 
   friend class kMyMoneyTransactionFormTable;
@@ -210,27 +214,18 @@ public:
     Reconciliation
   };
 
-	KLedgerView(QWidget *parent=0, const char *name=0);
-	virtual ~KLedgerView();
+  KLedgerView(QWidget *parent=0, const char *name=0);
+  virtual ~KLedgerView();
 
   /**
-    * This method is called by KGlobalLedgerView::selectAccount to set
-    * the current account to @p accountId. It calls
-    * loadAccount() to do the actual loading of data from the engine.
-    * If will also register
-    * this object as an observer with the engine's account represented by
-    * @p accountId which causes update() to be called whenever the engine's
-    * representation of the account changes.
+    * This method is the overridden version of the QWidget method. It
+    * checks that a transaction is selected if transactions are visible
+    * and if no transaction is selected selects the last one available.
     *
-    * Any previously registered observer will be detached.
-    *
-    * @param accountId id of the account to be loaded.
-    * @param force if true, the account is loaded independant if the
-    *              account is already loaded or not
-    *
-    * @see update
+    * If form is available, it is filled with the data of the selected
+    * transaction by calling fillForm().
     */
-  void setCurrentAccount(const QCString& accountId, const bool force=false);
+  void show();
 
   /**
     * This is the observer function called by the MyMoneyFile notify
@@ -243,16 +238,17 @@ public:
   void update(const QCString& accountId);
 
   /**
-    * This method returns a pointer to the transaction data
-    * in the ledger of this account. The transaction is identified
-    * by the parameter @p idx.
+    * utility method to suspend/activate updates of the MyMoney engine on
+    * all views. This is used to speed up operations with lot's of updates
+    * of engine data in a short time (e.g. importing data, creating a
+    * new file).
     *
-    * @param idx index into ledger starting at 0
-    * @return pointer to MyMoneyTransaction object representing the
-    *         selected transaction. If idx is out of bounds,
-    *         0 will be returned.
+    * @param suspend Suspend updates or not. Possible values are
+    *
+    * @li true updates are suspended
+    * @li false updates will be performed immediately
     */
-  MyMoneyTransaction* const transaction(const int idx) const;
+  void suspendUpdate(const bool suspend);
 
   /**
     * This method returns the balance of any visible transaction
@@ -276,6 +272,28 @@ public:
     */
   const QCString accountId(void) { return m_account.id(); }
 
+  /**
+    * This method returns the id of the account that is currently
+    * shown by this widget. It simply calls the above method but
+    * the interface must be provided as this class implements the
+    * interface defined in IMyMoneyRegisterParent.
+    *
+    * @return const QCString containing the account's id.
+    */
+  const QCString accountId(const MyMoneyTransaction* const, int match) const { return m_account.id(); }
+
+  /**
+    * This method returns a pointer to the transaction data
+    * in the ledger of this account. The transaction is identified
+    * by the parameter @p idx.
+    *
+    * @param idx index into ledger starting at 0
+    * @return pointer to MyMoneyTransaction object representing the
+    *         selected transaction. If idx is out of bounds,
+    *         0 will be returned.
+    */
+  KMyMoneyTransaction* transaction(const int idx) const;
+
   /**
     * This method is used to convert a string used in the
     * context of the user interface to an action string
@@ -297,14 +315,6 @@ public:
     */
   const QString action2str(const QCString& action, const bool showHotkey = false) const;
 
-  /**
-    * This method is used to select a specific transaction
-    *
-    * @param id const reference to the ID of the transaction to be selected
-    * @return true if the transaction was found and selected, false otherwise.
-    */
-  bool selectTransaction(const QCString& id);
-
   /**
     * This method is used to check if the transaction is being edited
     *
@@ -313,17 +323,47 @@ public:
   const bool isEditMode(void) const;
 
 public slots:
+  /**
+    * This method is used to select a specific transaction. If @p id is equal
+    * to "", then the last available transaction will be selected.
+    *
+    * @param id const reference to the ID of the transaction to be selected
+    * @retval true the selected transaction was found and selected
+    * @retval false the selected transaction does not exist or is currently not
+    *               visible
+    */
+  bool selectTransaction(const QCString& id);
+
+  /**
+    * This slot is called by KGlobalLedgerView::selectAccount to set
+    * the current account to @p accountId. It calls
+    * loadAccount() to do the actual loading of data from the engine.
+    * If will also register
+    * this object as an observer with the engine's account represented by
+    * @p accountId which causes update() to be called whenever the engine's
+    * representation of the account changes.
+    *
+    * Any previously registered observer will be detached.
+    *
+    * @param accountId id of the account to be loaded.
+    *
+    * @see update
+    */
+  void slotSelectAccount(const QCString& accountId);
+
   /**
     * This method refreshes the current view. This includes reading the
     * configuration options for the filters and calling updateView()
     */
   virtual void refreshView(void);
 
+  virtual void refreshView(const bool transactionFormVisible);
+
   /**
     * This method refreshes the current view including rebuild
     * of the filters by calling filterTransactions().
     */
-  virtual void updateView(void);
+  virtual void updateView(const QCString& transactionId);
 
   /**
     *
@@ -464,6 +504,9 @@ public slots:
     * This will ensure, that the transaction is visible in the
     * register, the state of the buttons is updated and that
     * the edit widgets will be shown (see showWidgets()).
+    *
+    * @note If no account is loaded into the ledger view, this
+    *       method does nothing.
     */
   virtual void slotStartEdit(void);
 
@@ -493,6 +536,9 @@ public slots:
     * This will select the 'new' transaction (last line of the register)
     * fill the form, update button states and show the edit widgets
     * (see showWidgets()).
+    *
+    * @note If no account is loaded into the ledger view, this
+    *       method does nothing.
     */
   virtual void slotNew(void);
 
@@ -553,6 +599,12 @@ protected slots:
     */
   virtual void slotSortOrderChanged(int orderId);
 
+  /**
+    * This method will fill the edit schedule dialog with the current
+    * transaction data and creates a scheduled transaction this way.
+    */
+  void slotCreateSchedule(void);
+  
 protected:
   /**
     * This method is called to create the widget stack for the
@@ -580,31 +632,19 @@ protected:
   virtual void fillSummary(void) = 0;
 
   /**
-    * This method reloads the account data from the engine, refreshes
-    * the view using refreshView() and repaints the register if not
-    * suppressed by an argument. If repainting is requested, the transaction
-    * form is also updated using fillForm().
+    * This method enables or disables widgets who's availability depends
+    * on a selected account. These are:
     *
-    * @param repaint If true, the register is repainted with the new
-    *                the new values, if false, repainting is suppressed.
-    *                The default is true.
+    * - the transaction form
+    *
+    * More widgets can be controlled if you override this method
+    * in a derived class.
+    *
+    * @param enable true enables the widgets, false disables them
     */
-  void reloadAccount(const bool repaint = true);
+  virtual void enableWidgets(const bool enable);
 
-  /**
-    * This method reloads the account data using reloadAccount(), selects
-    * the last transaction in the current register as the current
-    * transaction and fills the form with the data of this transaction
-    * using fillForm().
-    */
-  void loadAccount(void);
-
-  /**
-    * This method clears the m_TransactionPtrVector and rebuilds and sorts
-    * it according to the current settings. Once the m_transactionPtrVector
-    * is available, it rebuilds the m_balance vector.
-    */
-  void filterTransactions(void);
+  void setupPointerAndBalanceArrays(void);
 
   /**
     * This method converts the actions strings contained in a split
@@ -655,6 +695,15 @@ protected:
 
   virtual void reloadEditWidgets(const MyMoneyTransaction& t) = 0;
 
+  /**
+    * This method checks if transfer transactions are possible (the
+    * engine contains more than one asset/liability account).
+    *
+    * @retval true if transfer transactions are possible
+    * @retval false if transfer transactions are not (yet) possible
+    */
+  const bool transfersPossible(void) const;
+  
 protected:
   /**
     * This member keeps a pointer to the specific info stack for the account
@@ -699,7 +748,7 @@ protected:
     * This member holds a list of all transactions that belong to the account
     * shown in this view.
     */
-  QValueList<MyMoneyTransaction> m_transactionList;
+  QValueList<KMyMoneyTransaction> m_transactionList;
 
   /**
     * This member holds vector of the balances for each transaction
@@ -717,7 +766,7 @@ protected:
     * This member keeps a pointer to the currently selected transaction
     * It is NULL, if an empty (new) transaction is selected.
     */
-  MyMoneyTransaction *m_transactionPtr;
+  KMyMoneyTransaction *m_transactionPtr;
 
   /**
     * This member keeps a copy of the currently selected transaction
@@ -731,6 +780,14 @@ protected:
     */
   MyMoneySplit m_split;
 
+  /**
+    * This member keeps the date that was used as the last posting date.
+    * It will be updated whenever the user modifies the @ref m_editDate member
+    * and is used to preset the posting date when new transactions are set.
+    * This member is initialised to the current date when the program is started.
+    */
+  static  QDate         m_lastPostDate;
+  
   /*
    * The following members are pointers to the edit widgets
    */
@@ -782,9 +839,7 @@ private:
   void createSecondSplit(void);
 
 private:
-  QTimer*       m_timer;
-
-  /**
+   /**
     * This is the blink timer used to toggle the color for
     * erronous transactions in the register. The current state (on/off)
     * is kept in m_blinkState.
@@ -797,6 +852,12 @@ private:
     */
   bool          m_blinkState;
 
+  /**
+    * This member holds the state of the toggle switch used
+    * to suppress updates due to MyMoney engine data changes
+    */
+  bool m_suspendUpdate;
+
 private slots:
   void timerDone(void);
 
