@@ -51,12 +51,14 @@
 #include "../dialogs/kendingbalancedlg.h"
 #include "../dialogs/ksplittransactiondlg.h"
 #include "../dialogs/knewaccountdlg.h"
+#include "../mymoney/mymoneyfile.h"
+#include "../views/kmymoneyview.h"
 
 KLedgerViewCheckings::KLedgerViewCheckings(QWidget *parent, const char *name )
   : KLedgerView(parent,name)
 {
-  QGridLayout* formLayout = new QGridLayout( this, 1, 1, 11, 6, "FormLayout");
-  QVBoxLayout* ledgerLayout = new QVBoxLayout( 0, 0, 6, "LedgerLayout");
+  QGridLayout* formLayout = new QGridLayout( this, 1, 2, 11, 6, "FormLayout");
+  QVBoxLayout* ledgerLayout = new QVBoxLayout( 6, "LedgerLayout");
 
   createInfoStack();
   formLayout->addWidget(m_infoStack, 0, 1 );
@@ -68,6 +70,8 @@ KLedgerViewCheckings::KLedgerViewCheckings(QWidget *parent, const char *name )
   ledgerLayout->addLayout(m_summaryLayout);
 
   createForm();
+  // make sure, transfers are disabled if required
+  m_tabTransfer->setEnabled(transfersPossible());
   ledgerLayout->addWidget(m_form);
 
   formLayout->addLayout( ledgerLayout, 0, 0);
@@ -100,31 +104,59 @@ KLedgerViewCheckings::~KLedgerViewCheckings()
 
 void KLedgerViewCheckings::refreshView(void)
 {
-  KLedgerView::refreshView();
+  if(m_inReconciliation)
+    KLedgerView::refreshView(m_transactionCheckBox->isChecked());
+  else
+    KLedgerView::refreshView();
+
+  QDate date;
+  if(!m_account.value("lastStatementDate").isEmpty())
+    date = QDate::fromString(m_account.value("lastStatementDate"), Qt::ISODate);
+    
+  if(date.isValid())  
+    m_lastReconciledLabel->setText(i18n("Reconciled: %1").arg(KGlobal::locale()->formatDate(date, true)));
+  else
+    m_lastReconciledLabel->setText(QString());
+
+  m_tabTransfer->setEnabled(transfersPossible());
 }
 
-void KLedgerViewCheckings::resizeEvent(QResizeEvent* ev)
+void KLedgerViewCheckings::resizeEvent(QResizeEvent* /* ev */)
 {
   
   // resize the register
   int w = m_register->visibleWidth();
 
-  int m_debitWidth = 80;
-  int m_creditWidth = 80;
-  int m_balanceWidth = 100;
+  // check which space we need
+  m_register->adjustColumn(0);
+  m_register->adjustColumn(4);
+  m_register->adjustColumn(5);
+  m_register->adjustColumn(6);
 
-  m_register->setColumnWidth(0, 80);
-
-  // Resize the date field to the size required by the input widget
-  kMyMoneyDateInput* datefield = new kMyMoneyDateInput();
-  datefield->setFont(m_register->cellFont());
-  m_register->setColumnWidth(1, datefield->minimumSizeHint().width());
-  delete datefield;
+  // make amount columns all the same size
+  int width = m_register->columnWidth(4);
+  if(width < m_register->columnWidth(5))
+    width = m_register->columnWidth(5);
+  if(width < m_register->columnWidth(6))
+    width = m_register->columnWidth(6);
 
+  m_register->setColumnWidth(4, width);
+  m_register->setColumnWidth(5, width);
+  m_register->setColumnWidth(6, width);
+  
+  // Resize the date field to either
+  // a) the size required by the input widget if no transaction form is shown
+  // b) the adjusted value for the date if the transaction form is visible
+  if(!m_transactionFormActive) {
+    kMyMoneyDateInput* datefield = new kMyMoneyDateInput();
+    datefield->setFont(m_register->cellFont());
+    m_register->setColumnWidth(1, datefield->minimumSizeHint().width());
+    delete datefield;
+  } else {
+    m_register->adjustColumn(1);
+  }
+  
   m_register->setColumnWidth(3, 20);
-  m_register->setColumnWidth(4, m_debitWidth);
-  m_register->setColumnWidth(5, m_creditWidth);
-  m_register->setColumnWidth(6, m_balanceWidth);
 
   for(int i = 0; i < m_register->numCols(); ++i) {
     switch(i) {
@@ -157,17 +189,11 @@ void KLedgerViewCheckings::resizeEvent(QResizeEvent* ev)
   table->setColumnWidth(1, w);
 }
 
-void KLedgerViewCheckings::show()
+void KLedgerViewCheckings::enableWidgets(const bool enable)
 {
-  // make sure, the QTabbar does not send out it's selected() signal
-  // which would drive us crazy here. fillForm calls slotTypeSelected()
-  // later on anyway.
-  m_form->tabBar()->blockSignals(true);
-  KLedgerView::show();
-  m_form->tabBar()->blockSignals(false);
-
-  fillForm();
-  resizeEvent(NULL);
+  m_detailsButton->setEnabled(enable);
+  m_reconcileButton->setEnabled(enable);
+  KLedgerView::enableWidgets(enable);
 }
 
 void KLedgerViewCheckings::slotTypeSelected(int type)
@@ -232,49 +258,34 @@ void KLedgerViewCheckings::slotTypeSelected(int type)
     slotNew();
 }
 
-void KLedgerViewCheckings::slotRegisterDoubleClicked(int row, int col, int button, const QPoint &mousePos)
+void KLedgerViewCheckings::slotRegisterDoubleClicked(int /* row */,
+                                                     int /* col */,
+                                                     int /* button */,
+                                                     const QPoint & /* mousePos */)
 {
   slotStartEdit();
 }
 
 void KLedgerViewCheckings::createRegister(void)
 {
-  m_register = new kMyMoneyRegisterCheckings(this);
-  m_register->setView(this);
-
-  m_register->setNumCols(7);
-  m_register->setCurrentCell(0, 1);
-  m_register->horizontalHeader()->setClickEnabled(true);
-  m_register->horizontalHeader()->setLabel(0, i18n("Nr."));
-	m_register->horizontalHeader()->setLabel(1, i18n("Date"));
-	m_register->horizontalHeader()->setLabel(2, i18n("Payee"));
-	m_register->horizontalHeader()->setLabel(3, i18n("C"));
-	m_register->horizontalHeader()->setLabel(4, i18n("Payment"));
-	m_register->horizontalHeader()->setLabel(5, i18n("Deposit"));
-	m_register->horizontalHeader()->setLabel(6, i18n("Balance"));
- 	m_register->setLeftMargin(0);
-	m_register->verticalHeader()->hide();
-  m_register->setColumnStretchable(0, false);
-  m_register->setColumnStretchable(1, false);
-	m_register->setColumnStretchable(2, false);
-  m_register->setColumnStretchable(3, false);
-	m_register->setColumnStretchable(4, false);
-  m_register->setColumnStretchable(5, false);
-	m_register->setColumnStretchable(6, false);
-		
-	m_register->horizontalHeader()->setResizeEnabled(false);
-	m_register->horizontalHeader()->setMovingEnabled(false);
-
-  // never show horizontal scroll bars
-  m_register->setHScrollBarMode(QScrollView::AlwaysOff);
+  m_register = new kMyMoneyRegisterCheckings(this, "Checkings");
+  m_register->setParent(this);
 
+  m_register->setAction(QCString(MyMoneySplit::ActionATM), i18n("ATM"));
+  m_register->setAction(QCString(MyMoneySplit::ActionCheck), i18n("Cheque"));
+  m_register->setAction(QCString(MyMoneySplit::ActionDeposit), i18n("Deposit"));
+  m_register->setAction(QCString(MyMoneySplit::ActionWithdrawal), i18n("Withdrawal"));
+  m_register->setAction(QCString(MyMoneySplit::ActionTransfer), i18n("Transfer"));
+    
   connect(m_register, SIGNAL(clicked(int, int, int, const QPoint&)), this, SLOT(slotRegisterClicked(int, int, int, const QPoint&)));
   connect(m_register, SIGNAL(doubleClicked(int, int, int, const QPoint&)), this, SLOT(slotRegisterDoubleClicked(int, int, int, const QPoint&)));
 
   connect(m_register, SIGNAL(signalEnter()), this, SLOT(slotStartEdit()));
   connect(m_register, SIGNAL(signalNextTransaction()), this, SLOT(slotNextTransaction()));
   connect(m_register, SIGNAL(signalPreviousTransaction()), this, SLOT(slotPreviousTransaction()));
-
+  connect(m_register, SIGNAL(signalDelete()), this, SLOT(slotDeleteTransaction()));
+  connect(m_register, SIGNAL(signalSelectTransaction(const QCString&)), this, SLOT(selectTransaction(const QCString&)));
+    
   connect(m_register->horizontalHeader(), SIGNAL(clicked(int)), this, SLOT(slotRegisterHeaderClicked(int)));
 }
 
@@ -284,12 +295,17 @@ void KLedgerViewCheckings::createMoreMenu(void)
   KLedgerView::createMoreMenu();
 
   // and now the specific entries for checkings/savings etc.
+  KIconLoader *kiconloader = KGlobal::iconLoader();
   m_moreMenu->insertItem(i18n("Edit splits ..."), this, SLOT(slotStartEditSplit()),
       QKeySequence(), -1, 1);
-  m_moreMenu->insertItem(i18n("Goto payee/receiver"), this, SLOT(slotPayeeSelected()),
-      QKeySequence(), -1, 2);
+  m_moreMenu->insertItem(kiconloader->loadIcon("goto", KIcon::Small),
+                         i18n("Goto payee/receiver"), this, SLOT(slotPayeeSelected()),
+                         QKeySequence(), -1, 2);
+  m_moreMenu->insertItem(kiconloader->loadIcon("bookmark_add", KIcon::Small),
+                         i18n("Create schedule..."), this, SLOT(slotCreateSchedule()),
+                         QKeySequence(), -1, 3);
 
-  connect(m_form->moreButton(), SIGNAL(clicked()), this, SLOT(slotMorePressed()));
+  m_form->moreButton()->setPopup(m_moreMenu);
 }
 
 void KLedgerViewCheckings::createContextMenu(void)
@@ -298,10 +314,16 @@ void KLedgerViewCheckings::createContextMenu(void)
   KLedgerView::createContextMenu();
 
   // and now the specific entries for checkings/savings etc.
+  KIconLoader *kiconloader = KGlobal::iconLoader();
+  
   m_contextMenu->insertItem(i18n("Edit splits ..."), this, SLOT(slotStartEditSplit()),
       QKeySequence(), -1, 2);
-  m_contextMenu->insertItem(i18n("Goto payee/receiver"), this, SLOT(slotPayeeSelected()),
-      QKeySequence(), -1, 3);
+  m_contextMenu->insertItem(kiconloader->loadIcon("goto", KIcon::Small),
+                            i18n("Goto payee/receiver"), this, SLOT(slotPayeeSelected()),
+                            QKeySequence(), -1, 3);
+  m_contextMenu->insertItem(kiconloader->loadIcon("bookmark_add", KIcon::Small),
+                            i18n("Create schedule..."), this, SLOT(slotCreateSchedule()),
+                            QKeySequence(), -1, 4);
 }
 
 void KLedgerViewCheckings::createForm(void)
@@ -338,7 +360,7 @@ void KLedgerViewCheckings::createForm(void)
 
 void KLedgerViewCheckings::createSummary(void)
 {
-  m_summaryLayout = new QHBoxLayout(0, 0, 6, "SummaryLayout");
+  m_summaryLayout = new QHBoxLayout(6, "SummaryLayout");
 
   QSpacerItem* spacer = new QSpacerItem( 20, 1, QSizePolicy::Expanding, QSizePolicy::Minimum );
   m_summaryLayout->addItem(spacer);
@@ -363,14 +385,27 @@ void KLedgerViewCheckings::createInfoStack(void)
 
   QVBoxLayout* buttonLayout = new QVBoxLayout( frame, 0, 6, "ButtonLayout");
 
+  KIconLoader* il = KGlobal::iconLoader();
+  
   m_detailsButton = new KPushButton(frame, "detailsButton" );
-  m_detailsButton->setText(i18n("Account Details"));
+  KGuiItem detailsButtenItem( i18n("&Account Details"),
+                    QIconSet(il->loadIcon("viewmag", KIcon::Small, KIcon::SizeSmall)),
+                    i18n("Open the account dialog"),
+                    i18n("Use this view and modify the account details."));
+  m_detailsButton->setGuiItem(detailsButtenItem);
   buttonLayout->addWidget(m_detailsButton);
 
   m_reconcileButton = new KPushButton(frame, "reconcileButton");
-  m_reconcileButton->setText(i18n("&Reconcile ..."));
+  KGuiItem reconcileButtenItem( i18n("&Reconcile ..."),
+                    QIconSet(il->loadIcon("reconcile", KIcon::Small, KIcon::SizeSmall)),
+                    i18n("Start the account reconciliation"),
+                    i18n("Use this to reconcile your account against the bank statement."));
+  m_reconcileButton->setGuiItem(reconcileButtenItem);
   buttonLayout->addWidget(m_reconcileButton);
 
+  m_lastReconciledLabel = new QLabel("", frame);
+  buttonLayout->addWidget(m_lastReconciledLabel);
+  
   connect(m_reconcileButton, SIGNAL(clicked()), this, SLOT(slotReconciliation()));
   connect(m_detailsButton, SIGNAL(clicked()), this, SLOT(slotAccountDetail()));
 
@@ -407,12 +442,13 @@ void KLedgerViewCheckings::createInfoStack(void)
   QVBoxLayout* innerLayout = new QVBoxLayout( innerFrame, 6, 0, "InnerLayout");
 
   QLabel* txt;
-  QFont defaultFont = QFont("helvetica", 10);
-
   txt = new QLabel(i18n("<center><b>Reconcile account</b></center><hr>\n"
                         "<b>1.</b> Click on column 'C' "
                         "to clear the transactions "
                         "appearing on your bank statement."), innerFrame);
+
+  
+  QFont defaultFont = KMyMoneyUtils::cellFont();
   txt->setFont(defaultFont);
   innerLayout->addWidget(txt);
 
@@ -501,7 +537,7 @@ void KLedgerViewCheckings::createInfoStack(void)
 
   connect(m_postponeButton, SIGNAL(clicked()), this, SLOT(slotPostponeReconciliation()));
 
-  spacer = new QSpacerItem( 20, 20,
+  spacer = new QSpacerItem( 1, 1,
                    QSizePolicy::Minimum, QSizePolicy::Expanding );
   reconcileLayout->addItem( spacer );
 
@@ -540,7 +576,7 @@ void KLedgerViewCheckings::fillForm(void)
   if(m_transactionPtr != 0) {
     // get my local copy of the selected transaction
     m_transaction = *m_transactionPtr;
-    m_split = m_transaction.split(accountId());
+    m_split = m_transaction.splitByAccount(accountId());
 
     MyMoneyMoney amount = m_split.value();
 
@@ -574,14 +610,23 @@ void KLedgerViewCheckings::fillForm(void)
       MyMoneySplit s;
       switch(m_transaction.splitCount()) {
         case 2:
-          s = m_transaction.split(accountId(), false);
-          category = MyMoneyFile::instance()->accountToCategory(s.accountId());
+          if(m_transaction.isLoanPayment())
+            category = i18n("Loan Payment");
+          else {
+            s = m_transaction.splitByAccount(accountId(), false);
+            category = MyMoneyFile::instance()->accountToCategory(s.accountId());
+          }
           break;
+          
         case 1:
           category = " ";
           break;
+          
         default:
-          category = i18n("Splitted transaction");
+          if(m_transaction.isLoanPayment())
+            category = i18n("Loan Payment");
+          else
+            category = i18n("Splitted transaction");
           break;
       }
     } catch (MyMoneyException *e) {
@@ -696,12 +741,12 @@ void KLedgerViewCheckings::createEditWidgets(void)
 {
   m_editPayee = new kMyMoneyPayee(0, "editPayee");
   m_editCategory = new kMyMoneyCategory(0, "editCategory");
-  m_editMemo = new kMyMoneyLineEdit(0, "editMemo");
+  m_editMemo = new kMyMoneyLineEdit(0, "editMemo", AlignLeft|AlignVCenter);
   m_editAmount = new kMyMoneyEdit(0, "editAmount");
   m_editDate = new kMyMoneyDateInput(0, "editDate");
   m_editNr = new kMyMoneyLineEdit(0, "editNr");
-  m_editFrom = new kMyMoneyCategory(0, "editFrom", static_cast<kMyMoneyCategory::categoryTypeE> (kMyMoneyCategory::asset | kMyMoneyCategory::liability));
-  m_editTo = new kMyMoneyCategory(0, "editTo", static_cast<kMyMoneyCategory::categoryTypeE> (kMyMoneyCategory::asset | kMyMoneyCategory::liability));
+  m_editFrom = new kMyMoneyCategory(0, "editFrom", static_cast<KMyMoneyUtils::categoryTypeE> (KMyMoneyUtils::asset | KMyMoneyUtils::liability));
+  m_editTo = new kMyMoneyCategory(0, "editTo", static_cast<KMyMoneyUtils::categoryTypeE> (KMyMoneyUtils::asset | KMyMoneyUtils::liability));
   m_editSplit = new KPushButton("Split", 0, "editSplit");
   m_editPayment = new kMyMoneyEdit(0, "editPayment");
   m_editDeposit = new kMyMoneyEdit(0, "editDeposit");
@@ -754,7 +799,7 @@ void KLedgerViewCheckings::reloadEditWidgets(const MyMoneyTransaction& t)
   QString category, payee;
 
   m_transaction = t;
-  m_split = m_transaction.split(accountId());
+  m_split = m_transaction.splitByAccount(accountId());
   amount = m_split.value();
 
   if(m_editCategory != 0)
@@ -764,14 +809,14 @@ void KLedgerViewCheckings::reloadEditWidgets(const MyMoneyTransaction& t)
     m_editType->loadCurrentItem(m_form->tabBar()->indexOf(transactionType(m_split)));
 
   try {
-    if(m_split.payeeId() != "")
+    if(!m_split.payeeId().isEmpty())
       payee = MyMoneyFile::instance()->payee(m_split.payeeId()).name();
 
     MyMoneySplit s;
     MyMoneyAccount acc;
     switch(t.splitCount()) {
       case 2:
-        s = t.split(accountId(), false);
+        s = t.splitByAccount(accountId(), false);
         acc = MyMoneyFile::instance()->account(s.accountId());
 
         // if the second account is also an asset or liability account
@@ -780,27 +825,44 @@ void KLedgerViewCheckings::reloadEditWidgets(const MyMoneyTransaction& t)
           case MyMoneyAccount::Expense:
           case MyMoneyAccount::Income:
             category = MyMoneyFile::instance()->accountToCategory(s.accountId());
+            m_editCategory->loadList(static_cast<KMyMoneyUtils::categoryTypeE> (KMyMoneyUtils::income | KMyMoneyUtils::expense));
             break;
 
           case MyMoneyAccount::Asset:
           case MyMoneyAccount::Liability:
-            if(m_split.action() != MyMoneySplit::ActionTransfer) {
-              m_split.setAction(MyMoneySplit::ActionTransfer);
-              m_transaction.modifySplit(m_split);
-            }
-            if(s.action() != MyMoneySplit::ActionTransfer) {
-              s.setAction(MyMoneySplit::ActionTransfer);
-              m_transaction.modifySplit(s);
-            }
-            if(m_split.value() > 0) {
-              m_editFrom->loadText(MyMoneyFile::instance()->accountToCategory(s.accountId()));
-              m_editTo->loadText(MyMoneyFile::instance()->accountToCategory(m_account.id()));
+            if(!m_transaction.isLoanPayment()) {
+              if(m_editCategory)
+                m_editCategory->loadList(static_cast<KMyMoneyUtils::categoryTypeE> (KMyMoneyUtils::asset | KMyMoneyUtils::liability));
+              if(m_split.action() != MyMoneySplit::ActionTransfer) {
+                m_split.setAction(MyMoneySplit::ActionTransfer);
+                m_transaction.modifySplit(m_split);
+              }
+              if(s.action() != MyMoneySplit::ActionTransfer) {
+                s.setAction(MyMoneySplit::ActionTransfer);
+                m_transaction.modifySplit(s);
+              }
+              if(m_split.value() > 0) {
+                category = MyMoneyFile::instance()->accountToCategory(m_account.id());
+                if(m_editFrom)
+                  m_editFrom->loadText(MyMoneyFile::instance()->accountToCategory(s.accountId()));
+                if(m_editTo)
+                  m_editTo->loadText(category);
+                  
+              } else {
+                category = MyMoneyFile::instance()->accountToCategory(s.accountId());
+                if(m_editTo)
+                  m_editTo->loadText(category);
+                if(m_editFrom)
+                  m_editFrom->loadText(MyMoneyFile::instance()->accountToCategory(m_account.id()));
+                amount = -amount;       // make it positive
+              }
+              
             } else {
-              m_editTo->loadText(MyMoneyFile::instance()->accountToCategory(s.accountId()));
-              m_editFrom->loadText(MyMoneyFile::instance()->accountToCategory(m_account.id()));
-              amount = -amount;       // make it positive
+              // Make sure that we do not allow the user to modify the category
+              // directly, but only through the split edit dialog
+              connect(m_editCategory, SIGNAL(signalFocusIn()), this, SLOT(slotOpenSplitDialog()));
+              category = i18n("Loan payment");
             }
-            category = m_editTo->text();
             break;
 
           default:
@@ -810,16 +872,21 @@ void KLedgerViewCheckings::reloadEditWidgets(const MyMoneyTransaction& t)
         break;
 
       case 1:
-        category = "";
+        category = QString();
         break;
 
       default:
+        // Make sure that we do not allow the user to modify the category
+        // directly, but only through the split edit dialog
         connect(m_editCategory, SIGNAL(signalFocusIn()), this, SLOT(slotOpenSplitDialog()));
-        category = i18n("Splitted transaction");
+        if(t.isLoanPayment())
+          category = i18n("Loan payment");
+        else
+          category = i18n("Splitted transaction");
     }
   } catch(MyMoneyException *e) {
-    qDebug("Exception '%s' thrown in %s, line %ld caught in KLedgerViewCheckings::showWidgets():%d",
-      e->what().latin1(), e->file().latin1(), e->line(), __LINE__);
+    qDebug("Exception '%s' thrown in %s, line %ld caught in KLedgerViewCheckings::reloadEditWidgets():%d",
+      e->what().latin1(), e->file().latin1(), e->line(), __LINE__-2);
     delete e;
   }
 
@@ -847,9 +914,9 @@ void KLedgerViewCheckings::reloadEditWidgets(const MyMoneyTransaction& t)
   if(m_editPayment != 0 && m_editDeposit != 0) {
     if(m_split.value() < 0) {
       m_editPayment->loadText((-m_split.value()).formatMoney());
-      m_editDeposit->loadText("");
+      m_editDeposit->loadText(QString());
     } else {
-      m_editPayment->loadText("");
+      m_editPayment->loadText(QString());
       m_editDeposit->loadText((m_split.value()).formatMoney());
     }
   }
@@ -872,7 +939,7 @@ void KLedgerViewCheckings::loadEditWidgets(int& transType)
     reloadEditWidgets(*m_transactionPtr);
     transType = transactionType(m_split);
   } else {
-    m_editDate->setDate(QDate::currentDate());
+    m_editDate->setDate(m_lastPostDate);
     transType = m_form->tabBar()->currentTab();
 
     try {
@@ -1062,7 +1129,7 @@ void KLedgerViewCheckings::arrangeEditWidgetsInForm(QWidget*& focusWidget, const
   m_tabOrderWidgets.append(m_editAmount);
 }
 
-void KLedgerViewCheckings::arrangeEditWidgetsInRegister(QWidget*& focusWidget, const int transType)
+void KLedgerViewCheckings::arrangeEditWidgetsInRegister(QWidget*& focusWidget, const int /* transType */)
 {
   delete m_editAmount; m_editAmount = 0;
 
@@ -1166,23 +1233,46 @@ void KLedgerViewCheckings::slotReconciliation(void)
 {
   slotCancelEdit();
 
+/*
   MyMoneyMoney lastReconciledBalance(m_account.value("lastReconciledBalance"));
   if(lastReconciledBalance == 0)
     lastReconciledBalance = MyMoneyMoney(m_account.value("lastStatementBalance"));
 
   MyMoneyMoney statementBalance(m_account.value("statementBalance"));
-  QStringList vals(QStringList::split("-", QString(m_account.value("statementDate"))));
-  QDate statementDate = QDate::currentDate();
-
-  if(vals.count() == 3) {
-    statementDate.setYMD(vals[0].toInt(), vals[1].toInt(), vals[2].toInt());
-  }
-
-  KEndingBalanceDlg dlg(lastReconciledBalance, statementBalance, statementDate);
+  
+  QDate statementDate;
+  if(!m_account.value("statementDate").isEmpty())
+    statementDate = QDate::fromString(m_account.value("statementDate"), Qt::ISODate);
+    
+  if(!statementDate.isValid())
+    statementDate = QDate::currentDate();
+*/
+  KEndingBalanceDlg dlg(m_account);
 
   if(dlg.exec()) {
     QCString transactionId;
 
+    // check if the user requests us to create interest
+    // or charge transactions.
+    MyMoneyTransaction ti = dlg.interestTransaction();
+    MyMoneyTransaction tc = dlg.chargeTransaction();
+    if(ti != MyMoneyTransaction()) {
+      try {
+        MyMoneyFile::instance()->addTransaction(ti);
+      } catch(MyMoneyException *e) {
+        qWarning("interest transaction not stored: '%s'", e->what().data());
+        delete e;
+      }
+    }
+    if(tc != MyMoneyTransaction()) {
+      try {
+        MyMoneyFile::instance()->addTransaction(tc);
+      } catch(MyMoneyException *e) {
+        qWarning("charge transaction not stored: '%s'", e->what().data());
+        delete e;
+      }
+    }
+    
     m_infoStack->raiseWidget(KLedgerView::Reconciliation);
     m_inReconciliation = true;
     m_summaryLine->hide();
@@ -1190,25 +1280,16 @@ void KLedgerViewCheckings::slotReconciliation(void)
 
     m_prevBalance = dlg.previousBalance();
     m_endingBalance = dlg.endingBalance();
-    m_endingDate = dlg.endingDate();
+    m_endingDate = dlg.statementDate();
 
-    if(m_transactionPtr != 0)
-      transactionId = m_transactionPtr->id();
-
-    // do not show frozen or reconciled transactions
-    updateView();
+    refreshView();
     fillReconcileData();
 
-    if(transactionId != "")
-      if(selectTransaction(transactionId) == false
-      && m_transactionPtrVector.size() > 0)
-        selectTransaction(m_transactionPtrVector[0]->id());
-
     // allow SPACE to be used to toggle the clear state
     connect(m_register, SIGNAL(signalSpace()), this, SLOT(slotToggleClearFlag()));
     m_transactionCheckBox->setChecked(false);
+    slotShowTransactionForm(false);
     connect(m_transactionCheckBox, SIGNAL(toggled(bool)), this, SLOT(slotShowTransactionForm(bool)));
-
   }
 }
 
@@ -1218,7 +1299,7 @@ void KLedgerViewCheckings::fillReconcileData(void)
 
   for(unsigned int i = 0; i < m_transactionPtrVector.size(); ++i) {
     MyMoneyTransaction* t = m_transactionPtrVector[i];
-    MyMoneySplit sp = t->split(m_account.id());
+    MyMoneySplit sp = t->splitByAccount(m_account.id());
     if(sp.reconcileFlag() == MyMoneySplit::Cleared)
       cleared += sp.value();
   }
@@ -1230,11 +1311,13 @@ void KLedgerViewCheckings::fillReconcileData(void)
 
 void KLedgerViewCheckings::slotRegisterClicked(int row, int col, int button, const QPoint &mousePos)
 {
-  KLedgerView::slotRegisterClicked(row, col, button, mousePos);
+  if(!m_account.id().isEmpty()) {
+    KLedgerView::slotRegisterClicked(row, col, button, mousePos);
 
-  if(m_inReconciliation == true
-  && col == 3) {    // reconcileFlag column
-    slotToggleClearFlag();
+    if(m_inReconciliation == true
+    && col == 3) {    // reconcileFlag column
+      slotToggleClearFlag();
+    }
   }
 }
 
@@ -1248,7 +1331,7 @@ void KLedgerViewCheckings::slotConfigureMoreMenu(void)
 
   m_moreMenu->changeItem(splitEditId, i18n("Edit splits ..."));
   if(m_transactionPtr != 0) {
-    if(m_split.payeeId() != "" && m_split.accountId() != "" && m_transaction.id() != "") {
+    if(!m_split.payeeId().isEmpty() && !m_split.accountId().isEmpty() && !m_transaction.id().isEmpty()) {
       MyMoneyPayee payee = file->payee(m_split.payeeId());
       m_moreMenu->changeItem(gotoPayeeId, i18n("Goto '%1'").arg(payee.name()));
       m_moreMenu->setItemEnabled(gotoPayeeId, true);
@@ -1260,9 +1343,9 @@ void KLedgerViewCheckings::slotConfigureMoreMenu(void)
     if(transactionType(m_split) != Transfer) {
       m_moreMenu->connectItem(splitEditId, this, SLOT(slotStartEditSplit()));
     } else {
-      QString dest = "";
+      QString dest;
       try {
-        MyMoneySplit split = m_transaction.split(m_account.id(), false);
+        MyMoneySplit split = m_transaction.splitByAccount(m_account.id(), false);
         MyMoneyAccount acc = file->account(split.accountId());
         dest = acc.name();
       } catch(MyMoneyException *e) {
@@ -1282,7 +1365,7 @@ void KLedgerViewCheckings::slotConfigureContextMenu(void)
 {
   int splitEditId = m_contextMenu->idAt(2);
   int gotoPayeeId = m_contextMenu->idAt(3);
-  int deleteId = m_contextMenu->idAt(8);
+  int deleteId = m_contextMenu->idAt(9);
   MyMoneyFile* file = MyMoneyFile::instance();
 
   m_contextMenu->disconnectItem(splitEditId, this, SLOT(slotStartEditSplit()));
@@ -1290,9 +1373,11 @@ void KLedgerViewCheckings::slotConfigureContextMenu(void)
 
   m_contextMenu->changeItem(splitEditId, i18n("Edit splits ..."));
   if(m_transactionPtr != 0) {
-    if(m_split.payeeId() != "" && m_split.accountId() != "" && m_transaction.id() != "") {
+    if(!m_split.payeeId().isEmpty() && !m_split.accountId().isEmpty() && !m_transaction.id().isEmpty()) {
       MyMoneyPayee payee = file->payee(m_split.payeeId());
-      m_contextMenu->changeItem(gotoPayeeId, i18n("Goto '%1'").arg(payee.name()));
+      QString name = payee.name();
+      name.replace(QRegExp("&(?!&)"), "&&");
+      m_contextMenu->changeItem(gotoPayeeId, i18n("Goto '%1'").arg(name));
       m_contextMenu->setItemEnabled(gotoPayeeId, true);
     } else {
       m_contextMenu->changeItem(gotoPayeeId, i18n("Goto payee/receiver"));
@@ -1301,14 +1386,14 @@ void KLedgerViewCheckings::slotConfigureContextMenu(void)
     if(transactionType(m_split) != Transfer) {
       m_contextMenu->connectItem(splitEditId, this, SLOT(slotStartEditSplit()));
     } else {
-      QString dest = "";
+      QString dest;
       try {
-        MyMoneySplit split = m_transaction.split(m_account.id(), false);
+        MyMoneySplit split = m_transaction.splitByAccount(m_account.id(), false);
         MyMoneyAccount acc = MyMoneyFile::instance()->account(split.accountId());
         dest = acc.name();
       } catch(MyMoneyException *e) {
         delete e;
-        dest = "opposite account";
+        dest = i18n("opposite account");
       }
       m_contextMenu->changeItem(splitEditId, i18n("Goto '%1'").arg(dest));
       m_contextMenu->connectItem(splitEditId, this, SLOT(slotGotoOtherSideOfTransfer()));
@@ -1324,7 +1409,7 @@ void KLedgerViewCheckings::slotConfigureContextMenu(void)
 void KLedgerViewCheckings::slotToggleClearFlag(void)
 {
   if(m_transactionPtr != 0) {
-    MyMoneySplit sp = m_transactionPtr->split(m_account.id());
+    MyMoneySplit sp = m_transactionPtr->splitByAccount(m_account.id());
     switch(sp.reconcileFlag()) {
       case MyMoneySplit::NotReconciled:
         sp.setReconcileFlag(MyMoneySplit::Cleared);
@@ -1351,9 +1436,9 @@ void KLedgerViewCheckings::slotToggleClearFlag(void)
 void KLedgerViewCheckings::slotPostponeReconciliation(void)
 {
   if(m_inReconciliation == true) {
-    m_account.setValue("lastReconciledBalance", m_prevBalance.formatMoney());
-    m_account.setValue("statementBalance", m_endingBalance.formatMoney());
-    m_account.setValue("statementDate", m_endingDate.toString("yyyy-MM-dd"));
+    m_account.setValue("lastReconciledBalance", m_prevBalance.toString());
+    m_account.setValue("statementBalance", m_endingBalance.toString());
+    m_account.setValue("statementDate", m_endingDate.toString(Qt::ISODate));
 
     try {
       MyMoneyFile::instance()->modifyAccount(m_account);
@@ -1377,17 +1462,8 @@ void KLedgerViewCheckings::endReconciliation(void)
   m_summaryLine->show();
   m_transactionFormActive = config->readBoolEntry("TransactionForm", true);
 
-  if(m_transactionPtr != 0)
-    transactionId = m_transactionPtr->id();
-
-  updateView();
-
-  if(transactionId != "") {
-    if(selectTransaction(transactionId) == false
-    && m_transactionPtrVector.size() > 0)
-      selectTransaction(m_transactionPtrVector[m_transactionPtrVector.size()-1]->id());
-  }
-
+  refreshView();
+  
   disconnect(m_register, SIGNAL(signalSpace()), this, SLOT(slotToggleClearFlag()));
   disconnect(m_transactionCheckBox, SIGNAL(toggled(bool)), this, SLOT(slotShowTransactionForm(bool)));
 }
@@ -1395,20 +1471,22 @@ void KLedgerViewCheckings::endReconciliation(void)
 void KLedgerViewCheckings::slotEndReconciliation(void)
 {
   if(m_inReconciliation == true) {
-    m_account.setValue("lastStatementBalance", m_endingBalance.formatMoney());
-    m_account.setValue("lastStatementDate", m_endingDate.toString("yyyy-MM-dd"));
+    m_account.setValue("lastStatementBalance", m_endingBalance.toString());
+    m_account.setValue("lastStatementDate", m_endingDate.toString(Qt::ISODate));
 
     m_account.deletePair("lastReconciledBalance");
     m_account.deletePair("statementBalance");
     m_account.deletePair("statementDate");
 
+    MyMoneyFile::instance()->suspendNotify(true);
     try {
       MyMoneyFile::instance()->modifyAccount(m_account);
-      QValueList<MyMoneyTransaction> list = MyMoneyFile::instance()->transactionList(m_account.id());
+      MyMoneyTransactionFilter filter(m_account.id());
+      QValueList<MyMoneyTransaction> list = MyMoneyFile::instance()->transactionList(filter);
       QValueList<MyMoneyTransaction>::Iterator it;
 
       for(it = list.begin(); it != list.end(); ++it) {
-        MyMoneySplit sp = (*it).split(m_account.id());
+        MyMoneySplit sp = (*it).splitByAccount(m_account.id());
         if(sp.reconcileFlag() == MyMoneySplit::Cleared) {
           sp.setReconcileFlag(MyMoneySplit::Reconciled);
           sp.setReconcileDate(m_endingDate);
@@ -1420,6 +1498,7 @@ void KLedgerViewCheckings::slotEndReconciliation(void)
       qDebug("Unexpected exception when setting cleared to reconcile");
       delete e;
     }
+    MyMoneyFile::instance()->suspendNotify(false);
     endReconciliation();
   }
 }
@@ -1446,6 +1525,7 @@ void KLedgerViewCheckings::slotOpenSplitDialog(void)
                                                        m_account,
                                                        isValidAmount,
                                                        isDeposit,
+                                                       0,
                                                        this);
 
   if(dlg->exec()) {
@@ -1477,14 +1557,9 @@ void KLedgerViewCheckings::slotAccountDetail(void)
   }
 }
 
-void KLedgerViewCheckings::slotMorePressed(void)
-{
-  m_moreMenu->exec(m_form->moreButton()->mapToGlobal(QPoint(0,0)));
-}
-
 void KLedgerViewCheckings::slotPayeeSelected(void)
 {
   slotCancelEdit();
-  if(m_split.payeeId() != "" && m_split.accountId() != "" && m_transaction.id() != "")
+  if(!m_split.payeeId().isEmpty() && !m_split.accountId().isEmpty() && !m_transaction.id().isEmpty())
     emit payeeSelected(m_split.payeeId(), m_split.accountId(), m_transaction.id());
 }
