@@ -34,7 +34,7 @@
 
 // ----------------------------------------------------------------------------
 // KDE Includes
-
+#include <kdeversion.h>
 #include <kglobal.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
@@ -51,10 +51,10 @@ kMyMoneyDateInput::kMyMoneyDateInput(QWidget *parent, const char *name, Qt::Alig
   m_date = QDate::currentDate();
 
   dateEdit = new QDateEdit(m_date, this, "dateEdit");
-	setFocusProxy(dateEdit);
+  setFocusProxy(dateEdit);
 
   m_dateFrame = new QVBox(0,0,WType_Popup);
-	m_dateFrame->setFrameStyle(QFrame::PopupPanel | QFrame::Raised);
+  m_dateFrame->setFrameStyle(QFrame::PopupPanel | QFrame::Raised);
   m_dateFrame->setLineWidth(3);
   m_dateFrame->hide();
 
@@ -81,18 +81,21 @@ kMyMoneyDateInput::kMyMoneyDateInput(QWidget *parent, const char *name, Qt::Alig
   }
   dateEdit->setSeparator(separator);
 
-	m_datePicker = new KDatePicker(m_dateFrame, m_date);
+  m_datePicker = new KDatePicker(m_dateFrame, m_date);
+#if KDE_VERSION >= 310
+  // Let the date picker have a close button (Added in 3.1)
+  m_datePicker->setCloseButton(true);
+#endif
 
   // the next line is a try to add an icon to the button
-	m_dateButton = new QPushButton(QIconSet(QPixmap( locate("icon","hicolor/16x16/apps/korganizer.png"))), QString(""), this);
+  m_dateButton = new QPushButton(QIconSet(QPixmap( locate("icon","hicolor/16x16/apps/korganizer.png"))), QString(""), this);
   m_dateButton->setMinimumWidth(30);
-	// m_dateButton = new QPushButton(this);
 
-	connect(m_dateButton,SIGNAL(clicked()),SLOT(toggleDatePicker()));
+  connect(m_dateButton,SIGNAL(clicked()),SLOT(toggleDatePicker()));
   connect(dateEdit, SIGNAL(valueChanged(const QDate&)), this, SLOT(slotDateChosenRef(const QDate&)));
-	connect(m_datePicker, SIGNAL(dateSelected(QDate)), this, SLOT(slotDateChosen(QDate)));
+  connect(m_datePicker, SIGNAL(dateSelected(QDate)), this, SLOT(slotDateChosen(QDate)));
   connect(m_datePicker, SIGNAL(dateEntered(QDate)), this, SLOT(slotDateChosen(QDate)));
-	connect(m_datePicker, SIGNAL(dateSelected(QDate)), m_dateFrame, SLOT(hide()));
+  connect(m_datePicker, SIGNAL(dateSelected(QDate)), m_dateFrame, SLOT(hide()));
 }
 
 void kMyMoneyDateInput::show(void)
@@ -129,11 +132,11 @@ void kMyMoneyDateInput::toggleDatePicker()
   int h = m_dateFrame->height();
 
   if(m_dateFrame->isVisible())
-	{
-		m_dateFrame->hide();
-	}
-	else
-	{
+  {
+    m_dateFrame->hide();
+  }
+  else
+  {
     QPoint tmpPoint = mapToGlobal(m_dateButton->geometry().bottomRight());
 
     // usually, the datepicker widget is shown underneath the dateEdit widget
@@ -143,26 +146,26 @@ void kMyMoneyDateInput::toggleDatePicker()
       tmpPoint.setY(tmpPoint.y() - h - m_dateButton->height());
     }
 
-    if (m_qtalignment == Qt::AlignRight)
-		{
-	    m_dateFrame->setGeometry(tmpPoint.x(), tmpPoint.y(), w, h);
-		}
-		else
-		{
-			tmpPoint.setX(tmpPoint.x() - w);
-	    m_dateFrame->setGeometry(tmpPoint.x(), tmpPoint.y(), w, h);
-		}
-
-    QDate date = dateEdit->date();
-    if(date.isValid())
-		{
-      m_datePicker->setDate(date);
+    if((m_qtalignment == Qt::AlignRight && tmpPoint.x()+w <= QApplication::desktop()->width())
+    || (tmpPoint.x()-w < 0)  )
+    {
+      m_dateFrame->setGeometry(tmpPoint.x()-width(), tmpPoint.y(), w, h);
     }
-		else
-		{
+    else
+    {
+      tmpPoint.setX(tmpPoint.x() - w);
+      m_dateFrame->setGeometry(tmpPoint.x(), tmpPoint.y(), w, h);
+    }
+
+    if(m_date.isValid())
+    {
+      m_datePicker->setDate(m_date);
+    }
+    else
+    {
       m_datePicker->setDate(QDate::currentDate());
     }
-		m_dateFrame->show();
+    m_dateFrame->show();
   }
 }
 
@@ -217,19 +220,18 @@ void kMyMoneyDateInput::keyPressEvent(QKeyEvent * k)
 
 void kMyMoneyDateInput::slotDateChosenRef(const QDate& date)
 {
-  slotDateChosen(QDate(date));
+  if(date.isValid()) {
+    emit dateChanged(date);
+    m_date=date;
+  }
 }
 
 void kMyMoneyDateInput::slotDateChosen(QDate date)
 {
-  dateEdit->blockSignals(true);
-  dateEdit->setDate(date);
-  m_datePicker->setDate(date);
-  m_date=date;
-
-  emit dateChanged(date);
-
-  dateEdit->blockSignals(false);
+  if(date.isValid()) {
+    // the next line implies a call to slotDateChosenRef() above
+    dateEdit->setDate(date);
+  }
 }
 
 QDate kMyMoneyDateInput::getQDate(void)
@@ -247,7 +249,8 @@ void kMyMoneyDateInput::loadDate(const QDate& date)
   m_date = m_prevDate = date;
 
   blockSignals(true);
-  slotDateChosen(date);
+  dateEdit->setDate(date);
+  m_date = date;
   blockSignals(false);
 }
 
