@@ -17,13 +17,15 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
+#include <qapplication.h>
+#include <qdesktopwidget.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <knumvalidator.h>
 #include <kglobal.h>
 #include <klocale.h>
+#include <kdebug.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -32,21 +34,79 @@
 #include "kmymoneycalculator.h"
 #include "../mymoney/mymoneymoney.h"
 
+kMyMoneyMoneyValidator::kMyMoneyMoneyValidator(QObject * parent, const char * name) :
+  KDoubleValidator(parent, name)
+{
+}
+
+kMyMoneyMoneyValidator::kMyMoneyMoneyValidator( double bottom, double top, int decimals,
+                                                QObject * parent, const char * name ) :
+  KDoubleValidator(bottom, top, decimals, parent, name)
+{
+}
+
+/*
+ * The code of the following function is taken from kdeui/knumvalidator.cpp
+ * and adjusted to always use the monetary symbols defined in the KDE control center
+ */
+QValidator::State kMyMoneyMoneyValidator::validate( QString & input, int & _p ) const
+{
+  QString s = input;
+  KLocale * l = KGlobal::locale();
+  // ok, we have to re-format the number to have:
+  // 1. decimalSymbol == '.'
+  // 2. negativeSign  == '-'
+  // 3. positiveSign  == <empty>
+  // 4. thousandsSeparator() == <empty> (we don't check that there
+  //    are exactly three decimals between each separator):
+  QString d = l->monetaryDecimalSymbol(),
+          n = l->negativeSign(),
+          p = l->positiveSign(),
+          t = l->monetaryThousandsSeparator();
+  // first, delete p's and t's:
+  if ( !p.isEmpty() )
+    for ( int idx = s.find( p ) ; idx >= 0 ; idx = s.find( p, idx ) )
+      s.remove( idx, p.length() );
+
+
+  if ( !t.isEmpty() )
+    for ( int idx = s.find( t ) ; idx >= 0 ; idx = s.find( t, idx ) )
+      s.remove( idx, t.length() );
+
+  // then, replace the d's and n's
+  if ( ( !n.isEmpty() && n.find('.') != -1 ) ||
+       ( !d.isEmpty() && d.find('-') != -1 ) ) {
+    // make sure we don't replace something twice:
+    kdWarning() << "KDoubleValidator: decimal symbol contains '-' or "
+                   "negative sign contains '.' -> improve algorithm" << endl;
+    return Invalid;
+  }
+
+  if ( !d.isEmpty() && d != "." )
+    for ( int idx = s.find( d ) ; idx >= 0 ; idx = s.find( d, idx + 1 ) )
+      s.replace( idx, d.length(), ".");
+
+  if ( !n.isEmpty() && n != "-" )
+    for ( int idx = s.find( n ) ; idx >= 0 ; idx = s.find( n, idx + 1 ) )
+      s.replace( idx, n.length(), "-" );
+
+  return QDoubleValidator::validate( s, _p );
+}
+
 kMyMoneyEdit::kMyMoneyEdit(QWidget *parent, const char *name )
  : KLineEdit(parent,name)
 {
-	// Yes, just a simple double validator !
-	KFloatValidator *validator = new KFloatValidator(this);
-	validator->setAcceptLocalizedNumbers(true);
-	setValidator(validator);
-	setAlignment(AlignRight | AlignVCenter);
+  // Yes, just a simple double validator !
+  kMyMoneyMoneyValidator *validator = new kMyMoneyMoneyValidator(this);
+  setValidator(validator);
+  setAlignment(AlignRight | AlignVCenter);
 
   m_calculatorFrame = new QVBox(0,0,WType_Popup);
 
-	m_calculatorFrame->setFrameStyle(QFrame::PopupPanel | QFrame::Raised);
+  m_calculatorFrame->setFrameStyle(QFrame::PopupPanel | QFrame::Raised);
   m_calculatorFrame->setLineWidth(3);
-	
-	m_calculator = new kMyMoneyCalculator(m_calculatorFrame);
+
+  m_calculator = new kMyMoneyCalculator(m_calculatorFrame);
   m_calculatorFrame->setFixedSize(m_calculator->width()+3, m_calculator->height()+3);
   m_calculatorFrame->hide();
 
@@ -59,16 +119,24 @@ kMyMoneyEdit::~kMyMoneyEdit()
   delete m_calculatorFrame;
 }
 
+const bool kMyMoneyEdit::isValid(void) const
+{
+  return !text().isEmpty();
+}
+
 MyMoneyMoney kMyMoneyEdit::getMoneyValue(void)
 {
+  ensureFractionalPart();
   MyMoneyMoney money(text());
   return money;
 }
 
-void kMyMoneyEdit::loadText(const QString& text)
+void kMyMoneyEdit::loadText(const QString& txt)
 {
-  m_text = text;
-  setText(text);
+  setText(txt);
+  if(isEnabled())
+    ensureFractionalPart();
+  m_text = text();
 }
 
 void kMyMoneyEdit::resetText(void)
@@ -78,16 +146,26 @@ void kMyMoneyEdit::resetText(void)
 
 void kMyMoneyEdit::theTextChanged(const QString & theText)
 {
+  KLocale * l = KGlobal::locale();
+  QString d = l->monetaryDecimalSymbol();
   QString l_text = theText;
   int i = 0;
-  QValidator::State state =  this->validator()->validate( l_text, i);
-  if (state==QValidator::Invalid || state==QValidator::Intermediate && (!l_text.isEmpty() && (l_text!="-"))) {
-    setText(previousText);
-  } else
-    previousText = l_text;
+  if(isEnabled()) {
+    QValidator::State state =  this->validator()->validate( l_text, i);
+    if(state == QValidator::Intermediate) {
+      if(l_text.length() == 1) {
+        if(l_text != d && l_text != "-")
+          state = QValidator::Invalid;
+      }
+    }
+    if (state==QValidator::Invalid)
+      setText(previousText);
+    else
+      previousText = l_text;
+  }
 }
 
-void kMyMoneyEdit::focusOutEvent(QFocusEvent *e)
+void kMyMoneyEdit::ensureFractionalPart(void)
 {
   KLocale* locale = KGlobal::locale();
   // If text contains no 'monetaryDecimalSymbol' then add it
@@ -101,9 +179,17 @@ void kMyMoneyEdit::focusOutEvent(QFocusEvent *e)
       setText(s);
     }
   }
-  if(MyMoneyMoney(text()) != MyMoneyMoney(m_text))
-    emit valueChanged(text());
+}
 
+void kMyMoneyEdit::focusOutEvent(QFocusEvent *e)
+{
+  ensureFractionalPart();
+  
+  if(MyMoneyMoney(text()) != MyMoneyMoney(m_text)) {
+    emit valueChanged(text());
+    m_text = text();
+  }
+  
   QLineEdit::focusOutEvent(e);
 }
 
@@ -117,8 +203,6 @@ bool kMyMoneyEdit::eventFilter(QObject *o , QEvent *e )
 
   if(e->type() == QEvent::KeyPress) {
     QKeyEvent *k = static_cast<QKeyEvent *> (e);
-    QPoint p;
-    QRect r;
 
     rc = true;
     switch(k->key()) {
@@ -157,6 +241,15 @@ bool kMyMoneyEdit::eventFilter(QObject *o , QEvent *e )
           rc = false;
           break;
         }
+        // in case of '-' we do not enter the calculator when
+        // the current position is the beginning and there is
+        // no '-' sign at the first position.
+        if(k->key() == Qt::Key_Minus) {
+          if(cursorPosition() == 0 && text()[0] != '-') {
+            rc = false;
+            break;
+          }
+        }
         // otherwise, tricky fall through here!
 
       case Qt::Key_Slash:
@@ -168,10 +261,19 @@ bool kMyMoneyEdit::eventFilter(QObject *o , QEvent *e )
         }
         m_calculator->setInitialValues(text(), k);
 
-        p = mapToGlobal(QPoint(0,0));
-        r = m_calculator->geometry();
-        r.moveTopLeft(p);
+        int h = m_calculatorFrame->height();
 
+        // usually, the calculator widget is shown underneath the MoneyEdit widget
+        // if it does not fit on the screen, we show it above this widget
+
+        QPoint p = mapToGlobal(QPoint(0,0));
+        if(p.y() + h > QApplication::desktop()->height()) {
+          p.setY(p.y() - h);
+        } else {
+          p.setY(p.y() + height());
+        }
+        QRect r = m_calculator->geometry();
+        r.moveTopLeft(p);
         m_calculatorFrame->setGeometry(r);
         m_calculatorFrame->show();
         break;
